//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import java.beans.PropertyChangeListener;

/**
 * This type represents a command, as in the GOF sense.
 * A few additions have been made to the traditional command interface.
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1998.11.16  MB              Created
 */

public interface Command {
    /**
     * The string name of the "synchronous" property
     */
    public static final String SYNCHRONOUS = "synchronous";

    /**
     * The string name of the "enabled" property
     *
     * @see #isEnabled
     * @see #setEnabled
     */
    public static final String ENABLED = "enabled";

    /**
     * The string name of the "visible" property
     *
     * @see #isVisible
     * @see #setVisible
     */
    public static final String VISIBLE = "visible";

    /**
     * The string name of the "defaultButton" property
     *
     * @see #isDefaultButton
     * @see #setDefaultButton
     */
    public static final String DEFAULT_BUTTON = "defaultButton";

    /**
     * The string name of the "timeout" property
     *
     * @see #getTimeout
     */
    public static final String TIMEOUT = "timeout";

    /**
     * The string name of the "abort" property
     *
     * @see #getAbortAfterTimeout
     * @see #setAbortAfterTimeout
     */
    public static final String ABORT_AFTER_TIMEOUT = "abort";

    /**
     * The string name of the "resourceKey" property
     *
     * @see #getResourceKey
     * @see #setResourceKey
     */
    public static final String RESOURCE_KEY = "resourceKey";

    /**
     * Indicates whether or not execution should abort following
     * a timeout, or whether execution should proceed.
     *
     * @return boolean
     */
    boolean getAbortAfterTimeout();


    /**
     * This method adds a PropertyChangeListener to the command
     *
     * @param listener PropertyChangeListener
     */
    void addPropertyChangeListener(PropertyChangeListener listener);


    /**
     * This is the main method of the command. It is called after
     * initialiseAction and before finaliseAction (see command processor)
     * @return true if the action was successfull, false if anything unusal happened.
     */
    boolean doAction();


    /**
     * This method is called if an exception is thrown during the
     * commands execution (see implementation of DefaultProcessor)
     *
     * @param t java.lang.Throwable
     */
    void exceptionAction(Throwable t);


    /**
     * This is the last of the three action methods performed during execution.
     * initialiseAction and doAction preceed this one (see command processor).
     */
    void finaliseAction();


    /**
     * Returns the name of the command
     */
    void initialiseAction(Object param);


    /**
     * This method was created in VisualAge
     */
    void timeoutAction();


    /**
     * Associates an arbirtrary object with a key
     */
    void putValue(String key, Object newValue);


    /**
     * @param listener PropertyChangeListener
     */
    void removePropertyChangeListener(PropertyChangeListener listener);


    /**
     * @param abort boolean
     */
    void setAbortAfterTimeout(boolean abort);


    /**
     * Set the synchronous execution property.
     * Warning: Setting a command to synchronous causes executing thread
     * to block until command completion. If the executing thread happens
     * to be Swing or a GUI framework which uses a single event dispatch
     * queue, then deadlock occurs as soon as the command attempts to
     * display, for example, a popup dialog which expects user input.
     * In other words: (1) Use synchronous execution only after paying
     * careful attention to possible deadlock situations. (2) Never use
     * it if the command is supposed to obtain feedback from the end-user.
     */
    void setSynchronous(boolean on);

    /**
     * make the button controlable (or gray)
     */
    void setEnabled(boolean enabled);

    void setVisible(boolean on);

    void setDefaultButton(boolean on);


    /**
     * Return the RO property commandName
     *
     * @return boolean
     */
    String getCommandName();


    /**
     * Return the value of the "timeout" property.
     * This value is number of milliseconds before a timeout occurs. If the
     * value is zero, then no timeout occurs. Note that timeouts are controlled
     * by the command processor. So exactly how this property is interpreted
     * is, strictly speaking, up to the processor.
     *
     * @return long
     */
    long getTimeout();


    /**
     * Gets the Object associated with key. Standard properties stuff
     */
    Object getValue(String key);


    /**
     * Return the value of the "enabled" property
     */
    boolean isEnabled();

    boolean isVisible();

    boolean isDefaultButton();


    /**
     * Return the value of the "synchronous" property
     */
    boolean isSynchronous();

    public void setResourceKey(String resourceKey);

    public String getResourceKey();
}
