//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import com.nexirius.util.unittest.UnitTest;

public class ContextUnitTest extends UnitTest {




    private class TestContextListener implements ContextChangeListener {
        private ContextChangeEvent event;

        public void contextChange(ContextChangeEvent event) {
            this.event = event;
        }

        public ContextChangeEvent getLatestContextChangeEvent() {
            return this.event;
        }
    }


    public ContextUnitTest() {
    }

    private void performContextCreationTest() {
        /*
         * Create a context with three commands for testing
         */
        message("Create a context with three commands");

        Command[] commands1 = new Command[3];

        commands1[0] = new TestCommand("Test command 1.1");
        commands1[1] = new TestCommand("Test command 1.2");
        commands1[2] = new TestCommand("Test command 1.3");

        Context c1 = new DefaultContext("Test Context", commands1);

        assertTrue(true);

        message("Contains command");
        assertTrue(c1.containsCommand("Test command 1.1"));

        message("Contains command");
        assertTrue(c1.containsCommand("Test command 1.2"));

        message("Contains command");
        assertTrue(c1.containsCommand("Test command 1.3"));

        message("Does not contain command");
        assertTrue(!c1.containsCommand("Test command 1.4"));

        message("get command");
        assertTrue(c1.getCommand("Test command 1.1") == commands1[0]);

        message("get command");
        assertTrue(c1.getCommand("Test command 1.2") == commands1[1]);

        message("get command");
        assertTrue(c1.getCommand("Test command 1.3") == commands1[2]);

        try {
            message("don't get command");
            Object dummy = c1.getCommand("Test command 1.4");
            assertTrue(false);
        } catch (com.nexirius.util.assertion.PreconditionException e) {
            assertTrue(true);
        }
    }


    private void performDefaultContextManagerTest() {
        /*
         * Test default context manager
         */
        ContextManager contextManager = new DefaultContextManager();

        TestContextListener listener = new TestContextListener();

        // Make listener aware of context changes
        contextManager.addContextChangeListener(listener);

        message("Null context initially");

        Command[] commands1 = new Command[3];
        commands1[0] = new TestCommand("Test command 1.1");
        commands1[1] = new TestCommand("Test command 1.2");
        commands1[2] = new TestCommand("Test command 1.3");
        Context c1 = new DefaultContext("Test Context", commands1);

        // Note: No context has beed explicitely set on the cm at this stage
        Context c2 = contextManager.getContext();

        // So fetching the command list from  this context should result in...
        Command[] commands2 = c2.getCommandList();

        // ...a context with no commands
        assertTrue(commands2.length == 0);

        // Now set context
        contextManager.addContext(c1);

        message("Context was added to default context manager");

        assertTrue(listener.getLatestContextChangeEvent().getContext() == c1);

        contextManager.removeContext(c1);
        message("Context was removed from default context manager");

        Context nullContext = listener.getLatestContextChangeEvent().getContext();
        Command[] emptyList = nullContext.getCommandList();

        assertTrue(emptyList.length == 0);
    }


    private void performContextStackManagerTest() {
        /*
         * Now test ContextStackManager
         */
        ContextManager contextManager = new ContextStackManager();

        TestContextListener listener = new TestContextListener();

        contextManager.addContextChangeListener(listener);

        // Create a new context containing three commands
        Command[] commands1 = new Command[3];
        commands1[0] = new TestCommand("Test command 1.1");
        commands1[1] = new TestCommand("Test command 1.2");
        commands1[2] = new TestCommand("Test command 1.3");
        Context c1 = new DefaultContext("Test Context", commands1);

        // Create a new context containing two commands
        Command[] commands3 = new Command[2];
        commands3[0] = new TestCommand("Test command 3.1");
        commands3[1] = new TestCommand("Test command 3.2");
        Context c3 = new DefaultContext("Test Context 3", commands3);

        message("Add initial context to context stack manager");
        contextManager.addContext(c1);
        assertTrue(true);

        Context resultContext = listener.getLatestContextChangeEvent().getContext();

        Command[] resultCommands = resultContext.getCommandList();

        message("Resultant command list is equivalent to commands1");
        assertTrue(resultCommands.length == commands1.length);

        for (int i = 0; i < resultCommands.length; i++) {
            message("Command names match up");
            String cn1 = resultCommands[i].getCommandName();
            String cn2 = commands1[i].getCommandName();
            assertTrue(cn1.equals(cn2));
        }

        // Now add a 2nd context
        message("Add 2nd context to context stack manager");
        contextManager.addContext(c3);
        assertTrue(true);

        ContextChangeEvent resultEvent = listener.getLatestContextChangeEvent();

        message("Context change type is ADD_CONTEXT");
        assertTrue(resultEvent.getChangeType() == ContextChangeEvent.ADD_CONTEXT);

        Context resultCompleteContext = resultEvent.getContext();
        Context resultAddedContext = resultEvent.getAddedContext();

        Command[] resultAllCommands = resultCompleteContext.getCommandList();
        Command[] resultAddedCommands = resultAddedContext.getCommandList();

        message("Resultant full command list is equivalent to commands1 + commands3");
        assertTrue(resultAllCommands.length == commands1.length + commands3.length);

        message("Resultant added command list is equivalent to commands3");
        assertTrue(resultAddedCommands.length == commands3.length);

        for (int i = 0; i < resultAddedCommands.length; i++) {
            message("Added command names match up");
            String cn1 = resultAddedCommands[i].getCommandName();
            String cn2 = commands3[i].getCommandName();
            assertTrue(cn1.equals(cn2));
        }

        message("Remove last context from context stack manager");
        contextManager.removeContext(c1);
        assertTrue(true);

        message("Context change type is REMOVE_CONTEXT");

        resultEvent = listener.getLatestContextChangeEvent();

        assertTrue(resultEvent.getChangeType() == ContextChangeEvent.REMOVE_CONTEXT);

        resultCompleteContext = resultEvent.getContext();
        Context resultRemovedContext = resultEvent.getRemovedContext();

        resultAllCommands = resultCompleteContext.getCommandList();
        Command[] resultRemovedCommands = resultRemovedContext.getCommandList();

        message("Resultant added command list is equivalent to commands1");
        assertTrue(resultRemovedCommands.length == commands1.length);

        message("Resultant complete command list is equivalent to commands3");
        assertTrue(resultAllCommands.length == commands3.length);

        for (int i = 0; i < resultAllCommands.length; i++) {
            message("Added command names match up");
            String cn1 = resultAddedCommands[i].getCommandName();
            String cn2 = commands3[i].getCommandName();
            assertTrue(cn1.equals(cn2));
        }
    }


    /**
     * The main test routine
     */
    public void performUnitTest() {

        performContextCreationTest();

        performDefaultContextManagerTest();

        performContextStackManagerTest();

    }
}

