//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import com.nexirius.util.assertion.Assert;

import java.util.Hashtable;

/**
 * Provide a simple model for maintaining the current context.
 * Context change listeners may register with objects of this class
 * in which case they will be informed of context changes.
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1998.11.16  MB              Created
 */
public class DefaultContext implements Context {



    // A default processor used if no processor is specified during construction
    private static final Processor DEFAULT_PROCESSOR = new DefaultProcessor();

    // The commands associated with this context
    private Command[] commandList;

    // The name of the context
    private String name;

    // An alternative storage means for access to commands via command name
    // Note: This is just an optimisation. We could have gotten away with
    // an array of commands.
    private Hashtable commandTable = new Hashtable();

    // The context's processor
    private Processor processor;

    /**
     * Construct a default context
     *
     * @author MB
     */
    public DefaultContext() {
        this(DefaultContext.class.getName(), new Command[0]);
    }

    /**
     * Construct a default context
     *
     * @param name        The name of the context
     * @param commandList List of commands
     * @author MB
     */
    public DefaultContext(String name, Command[] commandList) {
        this(DefaultContext.class.getName(), commandList,
                DefaultContext.DEFAULT_PROCESSOR);
    }

    /**
     * Construct a default context
     *
     * @param name        String the name of the context
     * @param commandList List of commands
     * @param processor   The processor used to execute these commands
     * @author MB
     */
    public DefaultContext(String name, Command[] commandList, Processor processor) {
        Assert.pre(name != null, "Parameter name is not null");
        Assert.pre(commandList != null, "Parameter commandList is not null");
        Assert.pre(processor != null, "Parameter processor is not null");

        this.name = name;
        this.commandList = commandList;
        this.processor = processor;

        // Add commands to hashtable
        for (int i = 0; i < commandList.length; i++) {
            commandTable.put(commandList[i].getCommandName(), commandList[i]);
        }

        Assert.post(getName() == name, "name is set");
        Assert.post(getCommandList() == commandList, "command list is set");
        Assert.post(processor != null, "processor list is set");
    }


    /**
     * Return whether or not there exists a command in this context with the
     * given name
     *
     * @param commandName The command name in question
     * @return boolean
     * @author Marcel Baumann
     */
    public boolean containsCommand(String commandName) {
        Assert.pre(commandName != null, "Parameter commandName is not null");
        return commandTable.containsKey(commandName);
    }


    /**
     * Return the command in this context which has the given command name
     *
     * @param commandName The command name in question
     * @return com.nexirius.framework.command.Command
     * @author Marcel Baumann
     */
    public Command getCommand(String commandName) {
        Assert.pre(containsCommand(commandName), "Context contains command");

        return (Command) commandTable.get(commandName);
    }


    /**
     * Return the full list of commands available in this context.
     *
     * @return com.nexirius.framework.command.Command[]
     * @author Marcel Baumann
     */
    public Command[] getCommandList() {
        return this.commandList;
    }


    /**
     * Return the name of this context.
     *
     * @return java.lang.String
     * @author Marcel Baumann
     */
    public String getName() {
        return this.name;
    }


    /**
     * Return a processor for this context.  In this implementation of
     * the Context interface, the processor returned is not dependent on
     * the command name (it might be in other context implementations).
     *
     * @return com.nexirius.framework.command.Processor
     * @author Marcel Baumann
     */
    public Processor getProcessor(String commandName) {
        return this.processor;
    }

}
