//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

import com.nexirius.util.assertion.Assert;

/**
 * Provide a default model for maintaining the current context.
 * Adding a context to this context manager means the added context becomes
 * the current context. Removing a context from the context manager means
 * an instance of NullContext becomes the current context.
 * Context change listeners may registerTemplate with objects of this class
 * in which case they will be informed of context changes.
 * <p/>
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1998.11.16  MB              Created
 * @see com.nexirius.framework.command.NullContext
 */
public class DefaultContextManager implements ContextManager {



    /**
     * An instance of NullContext which the DefaultContextManager has as
     * the current context when (a) the DefaultContextManager is instantiated
     * and (b) whenever a context is removed.
     *
     * @see com.nexirius.framework.command.DefaultContextManager#remove
     */
    private static final Context NULL_CONTEXT = new NullContext();

    // The current context
    private Context currentContext = NULL_CONTEXT;

    // Support for context change event multicasting
    private transient ContextChangeListener aContextChangeListener = null;


    /**
     * Construct a default context manager object
     *
     * @author MB
     */
    public DefaultContextManager() {
        super();
    }


    /**
     * Method to support context change events
     *
     * @param event com.nexirius.framework.CommandSetEvent
     * @author MB
     */
    private void fireContextChange(Context context,
                                   Context addedOrRemovedContext, int type) {
        if (aContextChangeListener != null) {
            aContextChangeListener.contextChange(new ContextChangeEvent(this, context,
                    addedOrRemovedContext, type));
        }
    }

    /**
     * Return the current context
     *
     * @return The current context
     * @author MB
     */
    public Context getContext() {
        return this.currentContext;
    }


    /**
     * Add a context.
     * In this implementation of ContextManager, this means that the context
     * provided as a parameter will become the current context.
     *
     * @param context The context to be added
     * @author MB
     */
    public void addContext(Context context) {
        Assert.pre(context != null, "Parameter context is not null");

        this.currentContext = context;

        // Notify listeners
        // Note: The complete context is equivalent to the added context
        // in this context management implementation
        fireContextChange(this.currentContext, context,
                ContextChangeEvent.ADD_CONTEXT);

        Assert.post(getContext() == context, "currentContext is set");
    }


    /**
     * Remove the context given as a parameter.
     * In this implementation of ContextManager, this implies a return to the
     * null context.
     *
     * @param context The context to be removed
     * @author MB
     */
    public void removeContext(Context context) {
        this.currentContext = NULL_CONTEXT;

        // Notify listeners
        // Note: The complete context is equivalent to the removed context
        // in this context management implementation
        fireContextChange(this.currentContext, context,
                ContextChangeEvent.REMOVE_CONTEXT);

        Assert.post(getContext() == NULL_CONTEXT, "currentContext is set");
    }


    /**
     * Add a context change listener
     *
     * @param listener The context change listener to be added
     * @author MB
     */
    public void addContextChangeListener(ContextChangeListener listener) {
        aContextChangeListener =
                ContextChangeEventMulticaster.add(aContextChangeListener, listener);
    }


    /**
     * Remove a context change listener
     *
     * @param listener The context change listener to be removed
     * @author MB
     */
    public void removeContextChangeListener(ContextChangeListener listener) {
        aContextChangeListener =
                ContextChangeEventMulticaster.remove(aContextChangeListener, listener);
    }
}
