//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datacontroller;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelAdaptor;
import com.nexirius.framework.datamodel.DataModelEvent;
import com.nexirius.framework.dataviewer.DataViewer;

import javax.swing.*;
import java.awt.*;

/**
 * This class is primarely used to handle push button actions and to create new MVC
 * triplets. It is attached to exactly one DataViewer instance which itself is attached
 * to exactly one DataModel instance.
 * The controller is a listener to the data model and handles the DataModelEvent.CALL_METHOD
 * events which are fired by the DefaultDataModelCommand instances which are attached to the
 * data model (DataModel.appendMethod).
 * When a method event reaches the controller, it automatically calls the doHandleMethodCall
 * method and if this function returns true then the event is propagated to the parent DataController
 * instance (if any).
 * <br>
 * This implementation does not hold a list of child controller instances! The user who is
 * interested in a fully hierarchical structure needs to subclass this class and hold a list
 * of controller child instances.
 * <p/>
 * <pre>
 * class MyDataController extends DataController
 * {
 * //...
 * <p/>
 * public boolean doHandleMethodCall(String method, boolean fromChildController, DataModel source)
 * {
 * if (method.equals("newCustomer")) {
 * Customer customer = new Customer();
 * PopupController pc = new PopupController();
 * <p/>
 * pc.setClientModel(customer, true, null, null, null);
 * <p/>
 * if (pc.popup(true)) {
 * ((CustomerTable)source).addCustomer(customer);
 * }
 * <p/>
 * return false;
 * }
 * <p/>
 * return true;
 * }
 * }
 * <p/>
 * //...
 * <p/>
 * DataController dc = new MyDataController();
 * <p/>
 * dc.setDataViewer(factory.createViewer(exampleModel, false));
 * <p/>
 * getMainPanel().add(dc.createJComponent());
 * <p/>
 * </pre>
 *
 * @author Marcel Baumann
 */
public class DataController extends DataModelAdaptor {
    DataController parentController = null;
    DataViewer viewer = null;

    /**
     * To really work every DataController needs a DataViewer instance (see setDataViewer)
     */
    public DataController() {
    }

    /**
     * Initialize the associated data viewer (which holds the associated DataModel instance)
     * This method registers itself to the DataModel as a DataModelListener and organizes
     * the handleMethodCall from within the dataModelEdit method.
     *
     * @param viewer The associated DataViewer instance which holds the associated DataModel instance
     */
    public void setDataViewer(DataViewer viewer) {
        this.viewer = viewer;
        this.viewer.getDataModel().addSoftDataModelListener(this);
    }

    /**
     * access the associated DataViewer
     *
     * @return The associated DataViewer instance which holds the associated DataModel instance
     */
    public DataViewer getDataViewer() {
        return viewer;
    }

    /**
     * Access the associated DataModel instance which is alwais held in the associated DataViewer instance
     */
    public DataModel getDataModel() {
        return this.viewer.getDataModel();
    }

    /**
     * Creates a dialog window which holds the associated viewer component (see DataViewer.getJComponent)
     *
     * @param title
     * @param modal
     * @return the newly created JDialog window instance (call HOTSPOT_SIZE.pack(); DialogManager.center(HOTSPOT_SIZE); HOTSPOT_SIZE.setVisible(true))
     */
    public JDialog createInJDialog(String title, boolean modal) {
        JDialog dialog = DialogManager.createJDialog(title, modal);

        dialog.getContentPane().setLayout(new GridLayout(1, 1));
        dialog.getContentPane().add(createJComponent());

        return dialog;
    }

    /**
     * just returns viewer.getJComponent()
     *
     * @return The JComponent which is created by the associated DataViewer
     */
    public JComponent createJComponent() {
        return viewer.getJComponent();
    }

    /**
     * Access the parent instance within a nested controller structure
     *
     * @return The parent or null (default) see setParentController
     */
    public DataController getParentController() {
        return parentController;
    }

    /**
     * Organize the controller instances in a nested manner. Unhandled doMethodCall call are
     * sent up in the nested hierarchy of controllers.
     *
     * @param controller
     */
    public void setParentController(DataController controller) {
        parentController = controller;
    }

    /**
     * this method is called by dataModelEdit apon data model edit events of type DataModelEvent.CALL_METHOD
     *
     * @param methodName the name of the method (DefaultDataModelCommand instance)
     * @param source     The data model instance which holds the DefaultDataModelCommand instance
     */
    private final void handleMethodCall(String methodName, DataModel source) {
        if (!doHandleMethodCall(methodName, false, source)) {
            if (parentController != null) {
                parentController.doHandleMethodCall(methodName, true, source);
            }
        }
    }

    /**
     * This method has to be implemented to react on button actions on the GUI. The name of
     * the associated DefaultDataModelCommand is passed as the first parameter. The default
     * implementation just returns false (so this call will be passed to the parent controller if present)
     * <br>
     * All the commands which come from the associated DataModel and all the commands which
     * come from the children of the associated DataModel lead to this method call!
     *
     * @param methodName          The name of the DefaultdataModelCommand
     * @param fromChildController This is true if the command was not handled in the child controller and therefor was passed to the parent instance (this)
     * @param source              The model which holds the DefaultDataModelCommand
     * @return return true if this controller handles the method call (so it will not be passed to the parent controller)
     */
    public boolean doHandleMethodCall(String methodName, boolean fromChildController, DataModel source) {

        return false;
    }

    /**
     * Handle DataModelEvents if a DataModelEvent.CALL_METHOD event is catched then handleMethodCall is
     * invoked.
     */
    public void dataModelEdit(DataModelEvent event) {
        if (event.getId() == DataModelEvent.CALL_METHOD) {
            handleMethodCall(event.getData(), event.getDataModel());
        }
    }
}
