//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import javax.swing.*;
import javax.swing.tree.TreePath;
import java.util.EventObject;

/**
 * The DataModelEvent class is used by DataModel to inform the DataModelListener
 * of changes in value size or focus.
 *
 * @author Marcel Baumann
 */
public class DataModelEvent extends EventObject {

//	public static int instanceCount = 0;

    private static final long serialVersionUID = -4321312564625052675L;

    /**
     * fired by a DataModel to inform the listeners that the value has changed
     * (parameter of DataModelListener.dataModelChangeValue)
     */
    public final static int VALUE_CHANGE = 1;
    /**
     * fired by a child DataModel to inform the listeners that the value has changed
     * (parameter of DataModelListener.dataModelChangeValue)
     */
    public final static int CHILD_VALUE_CHANGE = 2;
    /**
     * fired by a StructModel to inform the listeners that the number of children has changed
     * (parameter of DataModelListener.dataModelChangeStructure)
     */
    public final static int STRUCTURE_CHANGE = 3;
    /**
     * fired by a DataModel to inform the DataViewer that they should grab the focus (last component wins)
     * (parameter of DataModelListener.dataModelGrabFocus)
     */
    public final static int GRAB_FOCUS = 4;
    /**
     * fired by a child DataModel to inform the DataViewer that they should grab the focus (last component wins)
     * (parameter of DataModelListener.dataModelGrabFocus)
     */
    public final static int GRAB_CHILD_FOCUS = 5;
    /**
     * fired by a DataViewer before the editing phase (store copy of original value)
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int START_EDIT = 6;
    /**
     * fired by a DataModel while value is changed during the editing phase
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int VALUE_EDIT = 7;
    /**
     * fired by a DataViewer after the editing phase (restore original value)
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int CANCEL_EDIT = 8;
    /**
     * fired by a DataViewer after the editing phase (accept new value)
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int FINISH_EDIT = 9;
    /**
     * fired by a DataModelCommand after each execution (see DefaultDataModelCommand.finaliseAction())
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int CALL_METHOD = 10;
    /**
     * fired by a DataModel to make all associated DataViewers enabled
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int ENABLE_METHOD = 11;
    /**
     * fired by a DataModel to make all associated DataViewers display grayed
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int DISABLE_METHOD = 12;
    /**
     * fired after one of the associated DataViewers changes its currently edited value to a value which is not accepted by the model
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int FORMAT_ERROR = 13;
    /**
     * fired after one of the associated DataViewers changes the highlighted status
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int HIGHLIGHT_CHANGED = 14;
    /**
     * fired after a viewer has been created (with DataViewer after getJComponent was called the first time)
     * (parameter of DataModelListener.dataModelEdit)
     */
    public final static int VIEWER_CREATED = 15;
    /**
     * fired when a hyperlink on a HTMLViewer was hit
     */
    public final static int HYPERLINK = 16;
    /**
     * fired on the highlighted model, when the delete or backspace button is pressed in an ArrayViewer
     */
    public final static int DELETE = 17;
    /**
     * fired on the array model, when rearranged with Alt+VK_UP or Alt+VK_DOWN
     */
    public final static int REARRANGED = 18;
    /**
     * fired when the instance name of a model has changed (e.g. by saving it)
     */
    public final static int INSTANCE_NAME_CHANGED = 19;
    /**
     * fired when the caption of a model has changed
     */
    public final static int CAPTION_CHANGED = 20;
    /**
     * fired from a DataModel to inform about a planned change.
     * The listeners may cancel such a planned change through their veto.
     */
    public final static int WANT_TO_CHANGE = 21;

    public final static int SUBTYPE_DEFAULT = 0;
    public final static int SUBTYPE_ITEM_REMOVED = 1;
    public final static int SUBTYPE_ITEM_ADDED = 2;
    public final static int SUBTYPE_CLEAR = 3;
    public final static int SUBTYPE_FLAG_CHANGED = 4;

    public static final String types[] = {
            "UNDEFINED"
            , "VALUE_CHANGE"
            , "CHILD_VALUE_CHANGE"
            , "STRUCTURE_CHANGE"
            , "GRAB_FOCUS"
            , "GRAB_CHILD_FOCUS"
            , "START_EDIT"
            , "VALUE_EDIT"
            , "CANCEL_EDIT"
            , "FINISH_EDIT"
            , "CALL_METHOD"
            , "ENABLE_METHOD"
            , "DISABLE_METHOD"
            , "FORMAT_ERROR"
            , "HIGHLIGHT_CHANGED"
            , "VIEWER_CREATED"
            , "HYPERLINK"
            , "DELETE"
            , "REARRANGED"
            , "INSTANCE_NAME_CHANGED"
    };

    public static final String subtypes[] = {
            "SUBTYPE_DEFAULT"
            , "SUBTYPE_ITEM_REMOVED"
            , "SUBTYPE_ITEM_ADDED"
            , "SUBTYPE_CLEAR"
            , "SUBTYPE_FLAG_CHANGED"
    };

    int id;
    boolean fromChild = false;
    DataModel transmitter;
    String data = null;
    int subtype = SUBTYPE_DEFAULT;
    int index = -1;
    DataModel item = null;
    Object viewer = null;

    /**
     * Copy constructor.
     *
     * @param other The data source for copy
     */
    private DataModelEvent(DataModelEvent other) {
        super(other.getDataModel());
//		++instanceCount;
        this.id = other.id;
        this.fromChild = other.fromChild;
        this.data = other.data;
        this.subtype = other.subtype;
        this.index = other.index;
        this.item = other.item;
        this.transmitter = other.transmitter;
        this.viewer = other.viewer;
    }

    /**
     * Creates a new event instance
     *
     * @param source The model which fired the event
     * @param id     The event type (VALUE_CHANGE, CHILDVALUE_CHANGE, STRUCTURE_CHANGE...)
     */
    public DataModelEvent(DataModel source, int id) {
        this(source, id, false);
    }

    /**
     * Creates a new event instance
     *
     * @param source The model which fired the event
     * @param id     The event type (VALUE_CHANGE, CHILDVALUE_CHANGE, STRUCTURE_CHANGE...)
     */
    public DataModelEvent(DataModel source, int id, int subtype, int index, DataModel item) {
        this(source, id, false);
        this.subtype = subtype;
        this.index = index;
        this.item = item;
    }

    /**
     * Creates a new event instance.
     *
     * @param source    The model which fired the event
     * @param id        The event type (VALUE_CHANGE, CHILDVALUE_CHANGE, STRUCTURE_CHANGE...)
     * @param fromChild True if the event was originally fired by a subchild
     */
    public DataModelEvent(DataModel source, int id, boolean fromChild) {
        super(source);
//		++instanceCount;
        this.id = id;
        this.fromChild = fromChild;
        transmitter = source;
    }

//	protected void finalize()
//	{
//		System.out.println("Finalize " + getClass().getName());
//		--instanceCount;
//	}

    /**
     * Creates a deep copy of this
     */
    public DataModelEvent duplicate() {
        return new DataModelEvent(this);
    }

    /**
     * Get the source (the data model which originally fired the event)
     */
    public DataModel getDataModel() {
        return (DataModel) getSource();
    }

    /**
     * Each parent which receives a child event and transmits it to the next layer
     * (its own listeners) sets the transmitter to identify himself as the direct source.
     * All the transmitters are inserted into the treePath (last transmitter is last in
     * the list).
     */
    public void setTransmitter(DataModel transmitter) {
        this.transmitter = transmitter;
    }

    /**
     * Get the last transmitter.
     *
     * @return The intermediate DataModel instance which transported the event to its parent
     */
    public DataModel getTransmitter() {
        return transmitter;
    }

    public void setFromChild(boolean fromChild)
    {
        this.fromChild = fromChild;
    }

    /**
     * Get a list of all transmitters from the child up to the own transmitter.
     *
     * @return null (if fromChild() == false)
     */
    public DataModelVector<DataModel> getTransmitterVector() {
    	DataModelVector<DataModel> ret = new DataModelVector<DataModel>();
    	DataModel dataModel = (DataModel)getSource();

    	while (dataModel != null) {
    		ret.add(dataModel);

            if (dataModel == transmitter) {
                break;
            }

    		dataModel = dataModel.getParentDataModel();
    	}
    	return(ret);
    }

    /**
     * Get the event type,
     *
     * @return VALUE_CHANGE, CHILDVALUE_CHANGE, STRUCTURE_CHANGE�
     */
    public int getId() {
        return this.id;
    }

    /**
     * Sets the event type.
     * @param i VALUE_CHANGE, CHILDVALUE_CHANGE, STRUCTURE_CHANGE�
     */
    public void setId(int i) {
        this.id = i;
    }

    public int getSubtype() {
        return this.subtype;
    }

    public int getIndex() {
        return this.index;
    }

    public DataModel getItem() {
        return item;
    }

    /**
     * This method accesses the actual viewer which initiated a VIEWER_CREATED event
     */
    public Object getViewer() {
        return viewer;
    }

    /**
     * set the actual viewer which initiated a VIEWER_CREATED event
     */
    public void setViewer(Object v) {
        viewer = v;
    }

    /**
     * Did this event go over at least one parent model (transmitter)
     */
    public boolean fromChild() {
        return this.fromChild;
    }

    /**
     * Get the swing tree path which leads to the source of the event (an array of TreeHook)
     *
     * @return the path to the source DataModel if fromChild() else null
     */
    public TreePath getTreePath() {
    	DataModelVector<DataModel> dmv = getTransmitterVector();

        Object path[] = new Object[dmv.size()];

        DataModel m = dmv.lastItem();

        for (int i = 0; i < path.length; ++i) {
            path[i] = m.getTreeHook();
            m = dmv.previousItem();
        }

        return new TreePath(path);
    }

    public void setData(String data) {
        this.data = data;
    }

    public String getData() {
        return data;
    }

    /**
     * Liefert true wenn dieses Event direkt und unmittelbar an alle Listener gesendet werden muss.
     * @return
     */
    public boolean needImmediateCalls()
    {
        return(false);
    }

    /**
     * Send the event to a DataModelListener. If the listener is of type SynchronizedDataModelListener
     * then the event is synchronized with the swing event queue to avoid graphical update problems
     * in multithreaded environments (calls SwingUtilities.invokeLater()).
     */
    public void sendTo(DataModelListener listener) {
        if (needImmediateCalls() == false &&  listener instanceof SynchronizedDataModelListener) {
            SwingUtilities.invokeLater(new EventSender(listener));
        } else {
            immediatelySendTo(listener);
        }
    }

    /**
     * Internal function which actually sends the event.
     */
    private void immediatelySendTo(DataModelListener listener) {
        switch (id) {
            case VALUE_CHANGE:
            case CHILD_VALUE_CHANGE:
                listener.dataModelChangeValue(this);
                break;

            case STRUCTURE_CHANGE:
                listener.dataModelChangeStructure(this);
                break;

            case GRAB_FOCUS:
            case GRAB_CHILD_FOCUS:
                listener.dataModelGrabFocus(this);
                break;

            case START_EDIT:
            case VALUE_EDIT:
            case CANCEL_EDIT:
            case FINISH_EDIT:
            case CALL_METHOD:
            case ENABLE_METHOD:
            case DISABLE_METHOD:
            case FORMAT_ERROR:
            case HIGHLIGHT_CHANGED:
            case VIEWER_CREATED:
            case HYPERLINK:
            case DELETE:
            case REARRANGED:
            case INSTANCE_NAME_CHANGED:
            case CAPTION_CHANGED:
                if (listener instanceof DataModelEditListener) {
                    ((DataModelEditListener) listener).dataModelEdit(this);
                }
                break;

            case WANT_TO_CHANGE:
                listener.dataModelWantToChangeValue((VetoableDataModelEvent)this);
                break;

            default:
                throw new RuntimeException("Unexpected DataModelEvent ID = " + id);
        }
    }

    /**
     * Create a string (for debugging purposes)
     */
    @Override
    public String toString() {
        StringBuffer ret = new StringBuffer();

        ret.append("-----------------DataModelEvent--------------------------\n");
        ret.append(getClass().getName() + ": " + types[this.id] + (this.fromChild ? " fromChild" : " "));
        ret.append("\nsource = " + getDataModel().getFieldName());
        ret.append("\nsubtype: " + subtypes[this.subtype] + " index: " + this.index);
        ret.append("\ndata='" + getData() + "'");
        ret.append("\nviewer: " + (this.viewer == null ? "null" : this.viewer.getClass().getName()) + "\n\n");

    	DataModelVector<DataModel> dmv = getTransmitterVector();
        for (int i = 0; i < dmv.size(); ++i) {
            ret.append("treePath[" + i + "] = " + dmv.getItem(i).getFieldName() + "\n");
        }
        ret.append("---------------------------------------------------------\n");

        return ret.toString();
    }

    public String getChildName() {
        StringBuffer ret = new StringBuffer();

    	DataModelVector<DataModel> dmv = getTransmitterVector();

        if (dmv != null) {
            int size = dmv.size();

            for (int i = size - 2; i >= 0; --i) {
                ret.append(dmv.getItem(i).getFieldName());

                if (i > 0) {
                    ret.append(DataModel.FIELD_NAME_SEPARATOR_CHAR);
                }
            }
        }

        return ret.toString();
    }

    public boolean isMethodCall(String methodName) {
        if (getId() == CALL_METHOD && methodName.equals(data)) {
            return true;
        }

        return false;
    }

    public boolean isInstanceNameChanged() {
        return getId() == INSTANCE_NAME_CHANGED;
    }

    public boolean isCaptionChanged() {
        return getId() == CAPTION_CHANGED;
    }

    /**
     * Internal class for swing synchronized event delivery
     */
    class EventSender implements Runnable
    {
        DataModelListener listener;
        //String stackTrace; // Der Stacktrace des "Auslösers". Der kann im run() dann einmal ganz interessant sein!

        EventSender(DataModelListener listener)
        {
            //stackTrace = getStackTrace();
            this.listener = listener;
        }

        @Override
        public void run()
        {
            immediatelySendTo(listener);
        }

// TODO Dieser Aufruf kann scheinbar zu einer unendlichen Loop fuehren!
//        protected String getStackTrace()
//        {
//            StringBuffer sb = new StringBuffer(256);
//            for (StackTraceElement ste : Thread.currentThread().getStackTrace())
//            {
//                sb.append(ste.toString()).append("\n");
//            }
//
//            return(sb.toString());
//        }
    }
}
