//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;

import java.io.File;

/**
 * A class that represents a FileName.
 *
 * @author Marcel Baumann
 */
public class FileNameModel extends StringModel {
    public static final String DOT_SLASH = "." + File.separator;
    FileNameModel rootDirectory = null;
    DirNameModel browseDirectory = null;
    DataModelListener listener = null;

    public FileNameModel() {
        super("");
    }

    /**
     * @param value The initial value.
     */
    public FileNameModel(String value) {
        super(value);
    }

    /**
     * @param value     The initial value.
     * @param fieldName The initial field name
     */
    public FileNameModel(String value, String fieldName) {
        super(value, fieldName);
    }

    /**
     * Creates an exact copy of the FileName model
     */
    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new FileNameModel(getText(), getFieldName());
        }

        if (instance instanceof FileNameModel) {
            ((FileNameModel) instance).setRootDirectory(rootDirectory);
            ((FileNameModel) instance).setBrowseDirectory(browseDirectory);

            if (listener != null) {
                ((FileNameModel) instance).autoUpdate();
            }
        }

        return super.duplicate(instance, copyPairs);
    }

    /**
     * The browse directory is used as the default directory, when an empty file name is edited by the browse button
     */
    public DirNameModel getBrowseDirectory() {
        return browseDirectory;
    }

    public void setBrowseDirectory(DirNameModel browseDirectory) {
        this.browseDirectory = browseDirectory;
    }

    public void setText(String text) {
        updateRoot(text);
    }

    public boolean isRelative(String text) {
        if (text == null) {
            text = getText();
        }

        return text.startsWith(DOT_SLASH);
    }

    public boolean isRelative() {
        return isRelative(null);
    }

    public void setRootDirectory(FileNameModel rootDir) {
        if (rootDir == rootDirectory) {
            return;
        }

        if (listener != null && rootDirectory != null) {
            rootDirectory.removeDataModelListener(listener);
        }

        rootDirectory = rootDir;

        if (listener != null && rootDirectory != null) {
            rootDirectory.addDataModelListener(listener);

            updateRoot();
        }
    }

    /**
     * automatically update on changes of the root directory
     */
    public void autoUpdate() {
        listener = new RootListener();
    }

    /**
     * @return null or the current root directory
     */
    public String getRootDirectory() {
        if (rootDirectory == null) {

            return null;
        }

        return rootDirectory.getDirectory(false);
    }

    /**
     * get the absolute directory name (contains a File.separator at the end)
     */
    public String getDirectory() {
        return getDirectory(false);
    }

    /**
     * get the directory name (contains a File.separator at the end)
     *
     * @param	rel only get the part of the directory name which
     * is appended to the actual root directory
     */
    public String getDirectory(boolean rel) {
        String text = null;

        if (rel) {
            text = getText();
        } else {
            text = getFullName();
        }

        // remove the file name part from the text (if any)
        int sepIndex = text.lastIndexOf(File.separatorChar);

        if (sepIndex >= 0 && sepIndex < text.length() - 1) {
            text = text.substring(0, sepIndex + 1);
        }

        return text;
    }

    /**
     * get the file name (if the current root directory is not null the name is returned as relative
     * file name starting with '.\' if this file is below the root directory)
     */
    public String getFileName() {
        String text = getFullName();

        // remove the director name part from the text (if any)
        int sepIndex = text.lastIndexOf(File.separatorChar);

        if (sepIndex >= 0) {

            if (sepIndex == text.length() - 1) {
                text = "";
            } else {
                text = text.substring(sepIndex + 1);
            }
        }

        return text;
    }

    /**
     * access the file name including full directory name
     */
    public String getFullName() {
        if (isRelative()) {
            String relativeFileName = getText().substring(2);

            if (getRootDirectory() == null) {

                return relativeFileName;
            }

            return getRootDirectory() + relativeFileName;
        }

        return getText();
    }

    protected void updateRoot() {
        updateRoot(null);
    }

    protected void updateRoot(String fileName) {
        String root = getRootDirectory();
        String text = (fileName == null ? getFullName() : fileName);

        boolean relative = isRelative(text);

        if (root != null) {
            if (!relative && text.startsWith(root)) {
                text = DOT_SLASH + text.substring(root.length());
            }
        }

        super.setText(text);
    }

    private class RootListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            updateRoot();
        }
    }
}
