//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.assertion.Assert;
import com.nexirius.util.simpletype.SimpleType_int;

/**
 * A class that represents an int information which can be displayed in a text field
 * or in a string selector viewer. The integer value has a lower and upper limitation which
 * the value never exceeds.
 *
 * @author Marcel Baumann
 */
public class LimitIntModel extends IntModel {



    int lowLimit;
    int highLimit;
    SimpleArrayModel valueArray = null;
    ObjectList objectList = null;

    public LimitIntModel() {
        this(0, 0, 0);
    }

    /**
     * Creates a new instance and initailizes the value, and its limits. If the low value is bigger than
     * the high value then a runtime exception is thrown.
     *
     * @param value     The value (it is automatically moved to the closer limit when outside range)
     * @param lowLimit  The lower limit
     * @param highLimit the upper limit (must be >= lowLimit)
     */
    public LimitIntModel(int value, int lowLimit, int highLimit) {
        super(value);
        setLimits(lowLimit, highLimit);
    }

    /**
     * Creates a new instance and initailizes the value, and its limits an the field name.
     * If the low value is bigger than the high value then a runtime exception is thrown.
     *
     * @param value     The value (it is automatically adjusted to the closer limit when outside range)
     * @param lowLimit  The lower limit
     * @param highLimit the upper limit (must be >= lowLimit)
     * @param fieldName The initial field name
     */
    public LimitIntModel(int value, int lowLimit, int highLimit, String fieldName) {
        super(value, fieldName);
        setLimits(lowLimit, highLimit);
    }

    /**
     * Creates a new instance and initailizes the value, and takes a value array to initialise the range
     * (0..valueArray.getLength()-1).
     *
     * @param value      The value (it is automatically adjusted to the closer limit when outside range)
     * @param valueArray This array is used for the definition of the low and high limits and for the setText() getText() interface
     */
    public LimitIntModel(int value, SimpleArrayModel valueArray) {
        super(value);
        setValueArray(valueArray);
    }

    /**
     * Creates a new instance and initailizes the value, and takes a value array to initialise the range
     * (0..valueArray.size()-1).
     *
     * @param value      The value (it is automatically adjusted to the closer limit when outside range)
     * @param valueArray This array is used for the definition of the low and high limits and for the setText() getText() interface
     * @param fieldName  The initial field name
     */
    public LimitIntModel(int value, SimpleArrayModel valueArray, String fieldName) {
        super(value, fieldName);
        setValueArray(valueArray);
    }

    /**
     * Creates a new instance and initailizes the value, and takes a value array to initialise the range
     * (0..valueArray.size()-1).
     *
     * @param value      The value (it is automatically adjusted to the closer limit when outside range)
     * @param fieldName  The initial field name
     * @param valueArray This array is used for the definition of the low and high limits and for the setText() getText() interface
     */
    public LimitIntModel(int value, String fieldName, Object... valueArray)
    {
        super(value, fieldName);
        setValueArray(new SimpleArrayModel(valueArray));
    }

    /**
     * Creates a new instance and initailizes the value.
     *
     * @param value      The value (it is automatically adjusted to the closer limit when outside range)
     * @param fieldName  The initial field name
     */
    public LimitIntModel(int value, String fieldName) {
        super(value, fieldName);
    }

    /**
     * Set the lower limitation and adjust the current value if it is outside the new range.
     * Throws a runtime exception if the new lower limit is bigger than the actual high limit.
     */
    public void setLowLimit(int lowLimit) {
        this.lowLimit = lowLimit;
        validate();
    }

    /**
     * Set the upper limitation and adjust the current value if it is outside the new range.
     * Throws a runtime exception if the new high limit is smaller than the actual low limit.
     */
    public void setHighLimit(int highLimit) {
        this.highLimit = highLimit;
        validate();
    }

    /**
     * Set the new range and adjust the current value if it is outside the new range.
     * Throws a runtime exception if the low limit is bigger than the high limit.
     */
    public void setLimits(int lowLimit, int highLimit) {
        this.lowLimit = lowLimit;
        this.highLimit = highLimit;
        validate();
    }

    /**
     * This method adjusts the current range to reflect the value array (0..valueArray.getLength()-1)
     */
    public void resetLimits() {
        Assert.pre(valueArray != null, "valueArray is not null");

        if (valueArray.getLength() == 0) {
            setLimits(0, 0);
        } else {
            setLimits(0, valueArray.getLength() - 1);
        }

        fireValueChange();
    }

    /**
     * Sets the limits back to plus/minus maximum!
     */
    public void clearLimits()
    {
        setLimits( Integer.MIN_VALUE, Integer.MAX_VALUE);
        fireValueChange();
    }

    /**
     * Returns true if the given integer is inbetween the limits (including the limits)
     */
    public boolean isValid(int v) {
        return v == makeValid(v);
    }

    /**
     * Returns the given value if v it is inbetween the limits or returns low limit if the value is smaller
     * than the current low limit or returns high limit if its is bigger than the current high limit.
     */
    public int makeValid(int v) {
        if (lowLimit > highLimit) {
            throw new RuntimeException("lowLimit is bigger than highlimit: [" + lowLimit + "..." + highLimit + "]");
        }

        return Math.max(Math.min(v, highLimit), lowLimit);
    }

    /**
     * Readjusts the current value so it is inbetween the current range.
     */
    public void validate() {
        int v = makeValid(getInt());

        if (v != getInt()) {
            setInt(v);
        }
    }

    /**
     * Set the new value as int. The new value is automatically made valid (makeValid()) before it is set.
     */
    @Override
    public void setInt(int i) {
        super.setInt(makeValid(i));
    }

    /**
     * Set the new value as Integer. The new value is automatically made valid (makeValid()) before it is set.
     * The value is set to empty if the parameter is null!
     */
    public void setInt(Integer i)
    {
        if (i == null)
        {
            setEmpty();
        }
        else
        {
            super.setInt(makeValid(i));
        }
    }

    /**
     * Set the new value as string.
     *
     * @param text If there is an associated value array then the given text is first compared to the string representations of the
     *             array to assign the assotiated index to the value. If none of the array values start with the given text then the
     *             text is parsed to an integer which is used to set the new value.
     * @throws Exception If the text cannot be parsed into an integer or if the new value is outside the
     *                   current range.
     */
    @Override
    public void setText(String text)
            throws Exception {
        if (objectList != null) {
            if (objectList.setStartText(text)) {
                return;
            }
        }

        if (text.length() == 0) {
            setEmpty();
        } else {
            SimpleType_int new_value = new SimpleType_int(text);

            if (!isValid(new_value.getInt())) {
                throw new Exception("Value out of bound: " + new_value + " [" + lowLimit + "..." + highLimit + "]");
            }

            setValue(new_value);
            setEmpty(false);
        }
    }

    public int getIndexOf(String text) {
        if (objectList != null) {

            return objectList.getIndexOf(text);
        }

        return -1;
    }

    /**
     * Returns the string representation of the assotiated value array member or the current
     * integer value as string if no value array is defined.
     */
    @Override
    public String getText() {
        if (objectList == null) {
            return super.getText();
        }

        return objectList.getText();
    }

    /**
     * Creates an exact copy of this instance.
     *
     * @param instance  null (or a defined subclass instance when called from subclass method duplicate)
     * @param copyPairs null (not used)
     */
    @Override
    public DataModel duplicate(DataModel instance, CopyPairs<DataModel> copyPairs) {
        if (instance == null) {
            instance = new LimitIntModel(getInt(), lowLimit, highLimit);
        }

        if (valueArray != null) {
            ((LimitIntModel) instance).setValueArray(valueArray);
        }

        return super.duplicate(instance, copyPairs);
    }

    /**
     * Returns the associated object list (which holds the value array)
     */
    public ObjectList getObjectList() {
        if (objectList == null) {
            objectList = new ObjectList(this);
        }

        return objectList;
    }

    /**
     * Returns the associated value array or null (if not set).
     */
    public SimpleArrayModel getValueArray() {
        return valueArray;
    }

    /**
     * Defines the assotiated object array (if null) and assigns a new value array to it.
     * The current value is automatically adjusted (if necessary) if it is outside the new range.
     *
     * @param valueArray The new value array
     */
    public void setValueArray(SimpleArrayModel valueArray) {
        this.valueArray = valueArray;

        if (objectList == null) {
            objectList = new ObjectList(this);
        } else {
            objectList.setArray(valueArray.getArray());
        }

        resetLimits();
    }

    public int getLowLimit() {
        return lowLimit;
    }

    public int getHighLimit() {
        return highLimit;
    }

    public void increment() {
        setInt(getInt() + 1);
    }

    public void decrement() {
        setInt(getInt() - 1);
    }

    public boolean canIncrement() {
        return getInt() < highLimit;
    }

    public boolean canDecrement() {
        return getInt() > lowLimit;
    }
}
