package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.simpletype.SimpleType_long;

public class LimitLongModel extends LongModel
{
    protected long lowLimit;
    protected long highLimit;

    public LimitLongModel()
    {
        this(0L, 0L, 0L);
    }

    /**
     * Creates a new instance and initailizes the value
     * 
     * @param value
     *            The value (it is automatically moved to the closer limit when outside range)
     */
    public LimitLongModel(long value)
    {
        super(value);
        setLimits(0L, 0L);
    }
    
    /**
     * Creates a new instance and initailizes the value, and its limits. If the low value is bigger than the high value
     * then a runtime exception is thrown.
     * 
     * @param value
     *            The value (it is automatically moved to the closer limit when outside range)
     * @param lowLimit
     *            The lower limit
     * @param highLimit
     *            the upper limit (must be >= lowLimit)
     */
    public LimitLongModel(long value, long lowLimit, long highLimit)
    {
        super(value);
        setLimits(lowLimit, highLimit);
    }

    /**
     * Creates a new instance and initailizes the value, and .
     * 
     * @param value
     *            The value (it is automatically adjusted to the closer limit when outside range)
     * @param fieldName
     *            The initial field name
     */
    public LimitLongModel(long value, String fieldName)
    {
        super(value, fieldName);
        setLimits(0L, 0L);
    }
    
    /**
     * Creates a new instance and initailizes the value, and its limits an the field name. If the low value is bigger
     * than the high value then a runtime exception is thrown.
     * 
     * @param value
     *            The value (it is automatically adjusted to the closer limit when outside range)
     * @param lowLimit
     *            The lower limit
     * @param highLimit
     *            the upper limit (must be >= lowLimit)
     * @param fieldName
     *            The initial field name
     */
    public LimitLongModel(long value, long lowLimit, long highLimit, String fieldName)
    {
        super(value, fieldName);
        setLimits(lowLimit, highLimit);
    }

    /**
     * Set the lower limitation and adjust the current value if it is outside the new range. Throws a runtime exception
     * if the new lower limit is bigger than the actual high limit.
     */
    public void setLowLimit(long lowLimit)
    {
        this.lowLimit = lowLimit;
        validate();
    }

    /**
     * Set the upper limitation and adjust the current value if it is outside the new range. Throws a runtime exception
     * if the new high limit is smaller than the actual low limit.
     */
    public void setHighLimit(long highLimit)
    {
        this.highLimit = highLimit;
        validate();
    }

    /**
     * Set the new range and adjust the current value if it is outside the new range. Throws a runtime exception if the
     * low limit is bigger than the high limit.
     */
    public void setLimits(long lowLimit, long highLimit)
    {
        this.lowLimit = lowLimit;
        this.highLimit = highLimit;
        validate();
    }

    /**
     * This method adjusts the current range to reflect the value array (0..valueArray.getLength()-1)
     */
    public void resetLimits()
    {
        setLimits(0, 0);
        fireValueChange();
    }

    /**
     * Returns true if the given integer is inbetween the limits (including the limits)
     */
    public boolean isValid(long v)
    {
        return v == makeValid(v);
    }

    /**
     * Returns the given value if v it is in between the limits or returns low limit if the value is smaller than the
     * current low limit or returns high limit if its is bigger than the current high limit.
     */
    public long makeValid(long v)
    {
        if (lowLimit > highLimit)
        {
            throw new RuntimeException("lowLimit is bigger than highlimit: [" + lowLimit + "..." + highLimit + "]");
        }

        return Math.max(Math.min(v, highLimit), lowLimit);
    }

    /**
     * Readjusts the current value so it is inbetween the current range.
     */
    public void validate()
    {
        long v = makeValid(getLong());

        if (v != getLong())
        {
            setLong(v);
        }
    }

    /**
     * Set the new value as long. The new value is automatically made valid (makeValid()) before it is set.
     * @return true wenn der Wert übernommen wurde; false wenn irgend jemand sein Veto einlegte.
     */
    public boolean setLong(long i)
    {
        return (super.setLong(makeValid(i)));
    }

    /**
     * Set the new value as string.
     * 
     * @param text
    *   the text is parsed to a long which is used to set the new value.
     * @throws Exception
     *             If the text cannot be parsed into an integer or if the new value is outside the current range.
     */
    public void setText(String text) throws Exception
    {
        if (text.length() == 0)
        {
            setEmpty();
        }
        else
        {
            SimpleType_long new_value = new SimpleType_long(text);

            if (!isValid(new_value.getLong()))
            {
                throw new Exception("Value out of bound: " + new_value + " [" + lowLimit + "..." + highLimit + "]");
            }

            setValue(new_value);
            setEmpty(false);
        }
    }

    /**
     * Creates an exact copy of this instance.
     * 
     * @param instance
     *            null (or a defined subclass instance when called from subclass method duplicate)
     * @param copyPairs
     *            null (not used)
     */
    @Override
    public DataModel duplicate(DataModel instance, CopyPairs<DataModel> copyPairs)
    {
        if (instance == null)
        {
            instance = new LimitLongModel(getLong(), lowLimit, highLimit);
        }
        return super.duplicate(instance, copyPairs);
    }

    public long getLowLimit()
    {
        return lowLimit;
    }

    public long getHighLimit()
    {
        return highLimit;
    }

    public void increment()
    {
        setLong(getLong() + 1);
    }

    public void decrement()
    {
        setLong(getLong() - 1);
    }

    public boolean canIncrement()
    {
        return getLong() < highLimit;
    }

    public boolean canDecrement()
    {
        return getLong() > lowLimit;
    }
}
