//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;


/**
 * This class represents a model which can be used to step thru a list of items. The first active status can be
 * accessed with getFrom(). The last status can be accessed with getTo(). The max number of active elements is count.
 */
public class StepperModel extends StructModel {



    public static final String STEPPER_MODEL_FROM = "StepperModelFrom";
    public static final String STEPPER_MODEL_COUNT = "StepperModelCount";
    public static final String STEPPER_MODEL_INFO = "StepperModelInfo";
    public static final String INCREMENT_STEPPER_COMMAND = "incrementStepperCommand";
    public static final String DECREMENT_STEPPER_COMMAND = "decrementStepperCommand";
    public static final String BEGIN_STEPPER_COMMAND = "beginStepperCommand";
    public static final String END_STEPPER_COMMAND = "endStepperCommand";

    LimitIntModel from;
    IntModel count;
    StringModel info;

    DefaultDataModelCommand incrementStepperCommand = new DefaultDataModelCommand(INCREMENT_STEPPER_COMMAND);
    DefaultDataModelCommand decrementStepperCommand = new DefaultDataModelCommand(DECREMENT_STEPPER_COMMAND);
    DefaultDataModelCommand beginStepperCommand = new DefaultDataModelCommand(BEGIN_STEPPER_COMMAND);
    DefaultDataModelCommand endStepperCommand = new DefaultDataModelCommand(END_STEPPER_COMMAND);

    /**
     * creates a Stepper which represents one active element
     *
     * @param fromValue
     * @param maxCount
     * @param fieldName
     */
    public StepperModel(int fromValue, int maxCount, String fieldName) {
        super(fieldName);
        append(from = new LimitIntModel(fromValue, 0, fromValue, STEPPER_MODEL_FROM));
        append(count = new IntModel(1, STEPPER_MODEL_COUNT));
        append(info = new StringModel("", STEPPER_MODEL_INFO));
        appendMethod(incrementStepperCommand);
        appendMethod(decrementStepperCommand);
        appendMethod(beginStepperCommand);
        appendMethod(endStepperCommand);

        from.addDataModelListener(new FromListener());

        updateStepper(maxCount);
    }

    /**
     * the given listener will be added to from and to count
     *
     * @param listener
     */
    public void addUpdateListener(DataModelListener listener) {
        from.addDataModelListener(listener);
        count.addDataModelListener(listener);
    }

    /**
     * sets the actual high limit of from and disables buttons accordingly
     *
     * @param maxCount the maximum value of from + 1
     */
    protected void updateStepper(int maxCount) {
        if (maxCount == 0) {
            from.setHighLimit(0);
        } else {
            from.setHighLimit(maxCount - 1);
        }

        if (count.getInt() == 1) {
            info.setText((from.getInt() + 1) + "/" + maxCount);
        } else {
            info.setText((from.getInt() + 1) + "..." + (getTo() + 1) + "/" + maxCount);
        }

        incrementStepperCommand.setEnabled(getTo() < maxCount - 1);
        endStepperCommand.setEnabled(getTo() < maxCount - 1);
        decrementStepperCommand.setEnabled(getFrom() > 0);
        beginStepperCommand.setEnabled(getFrom() > 0);
    }

    /**
     * set the from value (automatically goes to the high limit when bigger or equal than most recent maxCount)
     *
     * @param n new from value
     */
    public void setFrom(int n) {
        from.setInt(n);
    }

    /**
     * set the width of the window representing active items
     * (this value is also used to increment and decrement)
     *
     * @param n
     */
    public void setCount(int n) {
        count.setInt(n);
    }

    /**
     * the upper limit of the stepper
     *
     * @param maxCount
     */
    public void setMaxCount(int maxCount) {
        updateStepper(maxCount);
    }

    /**
     * access the current lower value
     *
     * @return
     */
    public int getFrom() {
        return from.getInt();
    }

    /**
     * access the current upper value
     *
     * @return
     */
    public int getTo() {
        int to = from.getInt() + count.getInt() - 1;

        if (to > from.getHighLimit()) {
            to = from.getHighLimit();
        }

        return to;
    }

    /**
     * get the max number of active items
     *
     * @return
     */
    public int getCount() {
        return count.getInt();
    }

    /**
     * increments from with count (if maxCount is not reached)
     */
    public void incrementStepperCommand() {
        if (getTo() < from.getHighLimit()) {
            setFrom(getFrom() + getCount());
        }
    }

    /**
     * decrements from with count (until from is == 0)
     */
    public void decrementStepperCommand() {
        setFrom(getFrom() - getCount());
    }

    /**
     * reset from to 0
     */
    public void beginStepperCommand() {
        setFrom(0);
    }

    /**
     * set from to hight limit - count
     */
    public void endStepperCommand() {
        setFrom(from.getHighLimit() - getCount() + 1);
    }

    class FromListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent ev) {
            updateStepper(from.getHighLimit() + 1);
        }
    }
}
