package com.nexirius.framework.datamodel.json;

import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.datamodel.json.format.*;

import java.io.*;
import java.util.Hashtable;

public class DefaultJsonFormatMap implements IJsonFormatMap {
    public static final String CHARSET = "UTF-8";

    Hashtable formats = new Hashtable();
    private static IJsonFormatMap instance;
    private IClassNameMap classNameMap = new DefaultClassNameMap();

    private DefaultJsonFormatMap() {
        registerJsonFormat(StringModel.class, new StringJsonFormat());
        registerJsonFormat(DateModel.class, new DateJsonFormat());
        registerJsonFormat(IntModel.class, new IntJsonFormat());
        registerJsonFormat(DoubleModel.class, new DoubleJsonFormat());
        registerJsonFormat(LongModel.class, new LongJsonFormat());
        registerJsonFormat(BooleanModel.class, new BooleanJsonFormat());
        registerJsonFormat(BigDecimalModel.class, new BigDecimalJsonFormat());
        registerJsonFormat(StructModel.class, new StructJsonFormat(this));
        registerJsonFormat(ArrayModel.class, new ArrayJsonFormat(this, null));
        registerJsonFormat(SimpleArrayModel.class, new SimpleArrayJsonFormat());
    }

    public static IJsonFormatMap getInstance() {
        if (instance == null) {
            instance = new DefaultJsonFormatMap();
        }

        return instance;
    }

    public static void setInstance(IJsonFormatMap map) {
        instance = map;
    }

    public static void readDataModel(DataModel dataModel, String json) throws Exception {
        PushbackInputStream in = createPushbackInputStream(json);
        DefaultJsonFormatMap.getInstance().getJsonFormat(dataModel).parseJson(in, dataModel);
    }

    public static String createJson(DataModel model) throws Exception {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DefaultJsonFormatMap.getInstance().getJsonFormat(model).writeJson(model, out);

        return new String(out.toByteArray(), "UTF-8");
    }

    /**
     * Try to find a json format for the data model (if the data model class is not
     * found then the parent class is used as key into the format table)
     *
     * @param model for this model type (class) a format is accessed
     * @return null or a json format matching th class of the given model
     */
    public IJsonFormat getJsonFormat(DataModel model) {
        Class clazz = model.getClass();

        while (clazz != null) {
            IJsonFormat format = (IJsonFormat) formats.get(clazz);

            if (format != null) {
                return format;
            }

            clazz = clazz.getSuperclass();
        }

        return null;
    }

    /**
     * register a specific json format for a given type
     *
     * @param dataModelClass
     * @param format
     */
    public void registerJsonFormat(Class dataModelClass, IJsonFormat format) {
        formats.put(dataModelClass, format);
    }

    /**
     * creates a new DataModel instance based on the next non-whitespace character
     * in the json stream. The charater is pushed back to the stream after reading.
     *
     * @param in
     * @return
     * @throws IOException
     */
    public DataModel createModel(PushbackInputStream in) throws IOException {
        DataModel ret = null;

        while (true) {
            int c = in.read();

            if (c < 0) {
                return null;
            }

            if (Character.isWhitespace((char) c)) {
                continue;
            }

            switch (c) {
                case '{':
                    ret = new StructModel("");
                    break;

                case '[':
                    ret = new ArrayModel(new IntModel(0, "foo"), "");
                    break;

                case '\"':
                    ret = new StringModel("", "");
                    break;

                case 't': // true
                case 'f': // false
                    ret = new BooleanModel(false, "");
                    break;

                case 'n': // null
                case '-':
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                    ret = new DoubleModel(0, "");
                    break;
            }

            if (ret != null) {
                in.unread(c);

                return ret;
            }
        }
    }

    public void setClassNameMap(IClassNameMap classNameMap) {
        this.classNameMap = classNameMap;
    }

    @Override
    public IClassNameMap getClassNameMap() {
        return classNameMap;
    }

    public static PushbackInputStream createPushbackInputStream(String s) throws UnsupportedEncodingException {
        return new PushbackInputStream(new ByteArrayInputStream(s.getBytes(CHARSET)));
    }
}
