package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.datamodel.json.*;
import com.nexirius.util.CopyPairs;
import com.nexirius.util.TextToken;

import java.io.OutputStream;
import java.io.PushbackInputStream;

public class HeterogeneousArrayJsonFormat implements IJsonFormat {
    private IJsonFormatMap jsonFormatMap;

    private static class ItemModel extends StructModel {
        private StringModel className;

        private ItemModel() {
            append(className = new StringModel("", "className"));
        }
    }

    public HeterogeneousArrayJsonFormat(IJsonFormatMap jsonFormatMap) {
        this.jsonFormatMap = jsonFormatMap;
    }

    public void parseJson(PushbackInputStream in, DataModel target) throws Exception, JsonParseException {
        ArrayModel array = (ArrayModel) target;
        TextToken token = TextToken.nextToken(in);

        if (token == null) {
            throw new JsonParseExceptionPrematureEndOfFile("in array " + array.getFieldName() + " before [");
        }

        if (!token.isChar('[')) {
            throw new JsonParseException(token, "[", "parsing array " + array.getFieldName());
        }

        while (true) {
            int c = in.read();

            if (c < 0) {
                throw new JsonParseExceptionPrematureEndOfFile("in array " + array.getFieldName() + " after [");
            }

            if (c == ']') {
                // we're done
                return;
            }

            if (c == ',' || Character.isWhitespace((char)c)) {
                // ignore
                continue;
            }

            // expect {"c":"classname","v":value}

            if (c != '{') {
                throw new JsonParseExceptionPrematureEndOfFile("expect { in heterogeneous array " + array.getFieldName() + " after [");
            }

            token = TextToken.nextToken(in);

            if (!token.isString() || !"c".equals(token.getString())) {
                throw new JsonParseExceptionPrematureEndOfFile("expect \"c\" in heterogeneous array " + array.getFieldName() + " after [{");
            }

            token = TextToken.nextToken(in);

            if (!token.isChar(':')) {
                throw new JsonParseExceptionPrematureEndOfFile("expect : in heterogeneous array " + array.getFieldName() + " after [{\"c\"");
            }

            token = TextToken.nextToken(in);

            if (!token.isString()) {
                throw new JsonParseExceptionPrematureEndOfFile("expect \"classname\" in heterogeneous array " + array.getFieldName() + " after [{\"c\":");
            }

            Class clazz = jsonFormatMap.getClassNameMap().getClassForName(token.getString());

            DataModel childModel = (DataModel) clazz.newInstance();

            token = TextToken.nextToken(in);

            if (!token.isChar(',')) {
                throw new JsonParseExceptionPrematureEndOfFile("expect , in heterogeneous array " + array.getFieldName() + " after [{\"c\":\"classname\"");
            }

            token = TextToken.nextToken(in);

            if (!token.isString() || !"v".equals(token.getString())) {
                throw new JsonParseExceptionPrematureEndOfFile("expect \"v\" in heterogeneous array " + array.getFieldName() + " after [{\"c\":\"classname\",");
            }

            token = TextToken.nextToken(in);

            if (!token.isChar(':')) {
                throw new JsonParseExceptionPrematureEndOfFile("expect : in heterogeneous array " + array.getFieldName() + " after [{\"c\":\"classname\",\"v\"");
            }

            IJsonFormat jsonFormat = jsonFormatMap.getJsonFormat(childModel);

            if (jsonFormat == null) {
                throw new JsonParseException("no json format for: " + childModel.getClass());
            }

            jsonFormat.parseJson(in, childModel);

            array.append(childModel);

            token = TextToken.nextToken(in);

            if (!token.isChar('}')) {
                throw new JsonParseExceptionPrematureEndOfFile("expect } in heterogeneous array " + array.getFieldName() + " after [{\"c\":\"classname\",\"v\":value");
            }
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        out.write('[');
        out.write('\n');

        DataModelEnumeration en = source.getEnumeration();
        boolean firstChild = true;

        while (en.hasMore()) {
            if (firstChild) {
                firstChild = false;
            } else {
                out.write(',');
            }

            DataModel childModel = en.next();

            IJsonFormat jsonFormat = jsonFormatMap.getJsonFormat(childModel);

            if (jsonFormat == null) {
                throw new Exception("no json format for: " + childModel.getClass());
            }

            String className = jsonFormatMap.getClassNameMap().getClassName(childModel.getClass());

            out.write("{\"c\":".getBytes(DefaultJsonFormatMap.CHARSET));
            out.write(TextToken.quoteString(className).getBytes(DefaultJsonFormatMap.CHARSET));
            out.write(",\"v\":".getBytes(DefaultJsonFormatMap.CHARSET));

            jsonFormat.writeJson(childModel, out);

            out.write('}');
            out.write('\n');
        }

        out.write(']');
        out.write('\n');
    }
}