package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.FWLog;
import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.datamodel.json.*;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class StructJsonFormat implements IJsonFormat {
    private static final String INSTANCE_NAME = "__instanceName__";
    private static final String CHARSET = "UTF-8";
    private IJsonFormatMap jsonFormatMap;

    public StructJsonFormat(IJsonFormatMap jsonFormatMap) {
        this.jsonFormatMap = jsonFormatMap;
    }

    public void parseJson(PushbackInputStream in, DataModel target) throws Exception, JsonParseException {
        StructModel struct = (StructModel) target;
        TextToken token = TextToken.nextToken(in);

        if (token == null) {
            throw new JsonParseExceptionPrematureEndOfFile("in struct " + struct.getFieldName() + " before {");
        }

        if (!token.isChar('{')) {
            throw new JsonParseException(token, "{", "parsing struct " + struct.getFieldName() + ")");
        }

        boolean createChildren = !target.hasChildren();

        while (true) {
            token = TextToken.nextToken(in);

            if (token == null) {
                throw new JsonParseExceptionPrematureEndOfFile("in struct " + struct.getFieldName() + " after {");
            }

            if (token.isChar('}')) {
                // we're done
                return;
            }

            if (token.isChar(',')) {
                // ignore
                continue;
            }

            if (token.isString()) {
                String fieldName = token.getString();
                // expect :
                token = TextToken.nextToken(in);

                if (!token.isChar(':')) {
                    throw new JsonParseException(token, ":", "in struct right after fieldName: " + fieldName);
                }

                DataModel childModel = null;

                if (createChildren) {
                    // create and read child
                    childModel = getJsonFormatMap().createModel(in);
                    childModel.setFieldName(fieldName);

                    if (childModel == null) {
                        throw new JsonParseException("cannot create child model");
                    }

                    struct.append(childModel);
                } else {
                    if (fieldName.equals(INSTANCE_NAME)) {
                        struct.setInstanceName(readInstanceName(in));
                    } else {
                        try {
                            childModel = struct.getChild(fieldName);
                        } catch (Exception e) {
                            FWLog.error(e);
                            try {
                                struct.ignoreChild(fieldName, in);
                            } catch (Exception e1) {
                                throw new JsonParseException(e1);
                            }
                        }
                    }
                }

                if (childModel != null) {
                    IJsonFormat jsonFormat = jsonFormatMap.getJsonFormat(childModel);

                    if (jsonFormat == null) {
                        throw new JsonParseException("no json format for: " + childModel.getClass());
                    }

                    jsonFormat.parseJson(in, childModel);

                    if (createChildren && childModel instanceof DoubleModel) {
                        DoubleModel dm = (DoubleModel) childModel;

                        // eventurally replace created DoubleModel with LongModel
                        if (dm.getDouble() <= Long.MAX_VALUE && dm.getDouble() >= Long.MIN_VALUE && Math.abs(dm.getDouble()) == dm.getDouble()) {
                            struct.removeItem(childModel);
                            childModel = new LongModel((long)dm.getDouble(), childModel.getFieldName());
                            struct.append(childModel);
                        }
                    }
                }
            } else {
                throw new JsonParseException(token, "string literal as struct field name", "in struct");
            }
        }
    }

    private String readInstanceName(PushbackInputStream in) throws JsonParseException, IOException {
        TextToken token = TextToken.nextToken(in);

        if (token.isString()) {
            return token.getString();
        }

        throw new JsonParseException(token, "string literal as " + INSTANCE_NAME, "in struct");
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        boolean firstChild = true;

        out.write('{');
        out.write('\n');

        if (source.getInstanceName() != null) {
            out.write('\"');
            out.write(INSTANCE_NAME.getBytes(CHARSET));
            out.write('\"');
            out.write(':');
            String instanceName = TextToken.quoteString(source.getInstanceName());
            out.write(instanceName.getBytes(CHARSET));
            out.write('\n');
            firstChild = false;
        }

        DataModelEnumeration en = source.getEnumeration();

        while (en.hasMore()) {
            DataModel childModel = en.next();

            if (childModel.isTransient()) {
                continue;
            }

            if (firstChild) {
                firstChild = false;
            } else {
                out.write(',');
            }

            out.write(TextToken.quoteString(childModel.getFieldName()).getBytes(DefaultJsonFormatMap.CHARSET));
            out.write(':');

            IJsonFormat jsonFormat = jsonFormatMap.getJsonFormat(childModel);

            if (jsonFormat == null) {
                throw new Exception("no json format for: " + childModel.getClass());
            }

            jsonFormat.writeJson(childModel, out);

            out.write('\n');
        }

        out.write('}');
        out.write('\n');
    }

    public IJsonFormatMap getJsonFormatMap() {
        if (jsonFormatMap == null) {
            jsonFormatMap = DefaultJsonFormatMap.getInstance();
        }
        return jsonFormatMap;
    }
}
