package com.nexirius.framework.datamodel.persistence.generic;

import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.jdbc.JdbcConnectionHandler;
import com.nexirius.util.CopyPairs;
import com.nexirius.util.StringVector;
import com.nexirius.util.XFile;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.InputStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.UUID;

public class GenericDatabase implements IGenericDatabase {
    private IGenericNamespace namespace;
    private static final String TABLE_JNEX_INSTANCE = "jnex_instance";
    private static final String TABLE_JNEX_BLOB = "jnex_blob";
    private JdbcConnectionHandler jdbcConnectionHandler;
    private DataModelSearchAttributesTable searchAttributesTable;

    public GenericDatabase(JdbcConnectionHandler jdbcConnectionHandler, IGenericNamespace namespace, DataModelSearchAttributesTable searchAttributesTable) {
        this.jdbcConnectionHandler = jdbcConnectionHandler;
        this.namespace = namespace;
        this.searchAttributesTable = searchAttributesTable;
    }

    public DataModelSearchAttributesTable getSearchAttributesTable() {
        return searchAttributesTable;
    }

    public IGenericNamespace getGenericNamespace() {
        return namespace;
    }

    public void createReference(DataModel owner, DataModel reference)
            throws Exception {
        String ownerUuid = owner.getInstanceName();
        String referenceUuid = reference.getInstanceName();
        if (ownerUuid != null && ownerUuid.length() > 0 && referenceUuid != null && referenceUuid.length() > 0) {
            StringBuffer sql = new StringBuffer("INSERT INTO jnex_reference_list (UUID, REFERENCE_UUID) VALUES (?,?);");
            print(sql);

            PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
            statement.setString(1, ownerUuid);
            statement.setString(2, referenceUuid);
            statement.execute();
        }

    }

    public void create(DataModel model) throws Exception {
        if (model.getInstanceName() == null) {
            model.setInstanceName(UUID.randomUUID().toString());
        }

        String modelClass = model.getClass().getName();
        StringBuffer sql = new StringBuffer("INSERT INTO ");

        sql.append(TABLE_JNEX_INSTANCE);
        if (namespace == null) {
            sql.append(" (UUID,TYPE,VALUE) VALUES (?,?,?);");
        } else {
            sql.append(" (NAMESPACE_UUID,UUID,TYPE,VALUE) VALUES (?,?,?,?);");
        }

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }
        statement.setString(i++, model.getInstanceName());
        statement.setString(i++, modelClass);
        if (model instanceof DataModelContainer) {
            ((DataModelContainer) model).prepareStreamAsReference();
        }
        statement.setString(i++, model.dragData());

        statement.execute();

        statement.close();

        // create search attributes table entry
        DataModelSearchAttributes dataModelSearchAttributes = searchAttributesTable.getSearchAttributes(modelClass);

        if (dataModelSearchAttributes != null) {
            dataModelSearchAttributes.insertAttributes(jdbcConnectionHandler, model, namespace);
        }
    }

    public boolean read(DataModel model, String instanceName) throws Exception {
        if (instanceName == null || instanceName.length() == 0) {
            return false;
        }

        String s = readAsString(instanceName);

        if (s == null || s.length() == 0) {
            return false;
        }

        model.dropData(s);
        model.setInstanceName(instanceName);

        return true;
    }

    public String readAsString(String instanceName) throws Exception {
        String s = "";
        StringBuffer sql = new StringBuffer("SELECT VALUE FROM ");

        sql.append(TABLE_JNEX_INSTANCE);
        if (namespace == null) {
            sql.append(" WHERE UUID=?;");
        } else {
            sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
        }

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        statement.setString(i++, instanceName);

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        statement.execute();

        ResultSet resultSet = statement.getResultSet();

        if (resultSet.next()) {
            s = resultSet.getString(1);
        } else {
            throw new Exception("No such entry: " + instanceName);
        }

        resultSet.close();
        statement.close();
        return s;
    }

    public void update(DataModel model) throws Exception {
        String modelClass = model.getClass().getName();
        if (model.getInstanceName() == null) {
            create(model);

            return;
        }

        StringBuffer sql = new StringBuffer("UPDATE ");

        sql.append(TABLE_JNEX_INSTANCE);
        sql.append(" SET TYPE=?,VALUE=?");

        if (namespace == null) {
            sql.append(" WHERE UUID=?;");
        } else {
            sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
        }

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        statement.setString(i++, modelClass);
        if (model instanceof DataModelContainer) {
            ((DataModelContainer) model).prepareStreamAsReference();
        }
        statement.setString(i++, model.dragData());
        statement.setString(i++, model.getInstanceName());

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        statement.execute();
        statement.close();

        // update search attributes
        DataModelSearchAttributes dataModelSearchAttributes = searchAttributesTable.getSearchAttributes(modelClass);

        if (dataModelSearchAttributes != null) {
            dataModelSearchAttributes.updateAttributes(jdbcConnectionHandler, model, namespace);
        }
    }

    public void delete(String instanceName) throws Exception {
        StringBuffer sql = new StringBuffer("DELETE FROM ");

        sql.append(TABLE_JNEX_INSTANCE);
        sql.append(" WHERE UUID=?;");

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        statement.setString(i++, instanceName);

        statement.execute();

        statement.close();

        // TODO delete entry from search attributes
    }

    //select jnex_instance.uuid,jnex_instance.value from jnex_instance join t1 on jnex_instance.uuid=t1.uuid where 1=1 ;
    public int search(ArrayModel result, DataModel template, StringBuffer whereClause) throws SQLException {
        String wc;
        if (whereClause == null || whereClause.toString().length() == 0) {
            wc = "1=1";
        } else {
            wc = whereClause.toString();
        }

        StringBuffer sql = new StringBuffer("SELECT jnex_instance.UUID,jnex_instance.VALUE FROM ");

        sql.append(TABLE_JNEX_INSTANCE);

        print(sql);

        DataModelSearchAttributes dataModelSearchAttributes = searchAttributesTable.getSearchAttributes(template.getClass().getName());

        if (whereClause != null && dataModelSearchAttributes != null) {
            sql.append(" JOIN ");
            sql.append(dataModelSearchAttributes.getDatabaseTableName());
            sql.append(" on jnex_instance.uuid=");
            sql.append(dataModelSearchAttributes.getDatabaseTableName());
            sql.append(".uuid ");
        }

        if (namespace == null) {
            sql.append(" WHERE TYPE=? AND ");
        } else {
            sql.append(" WHERE TYPE=? AND NAMESPACE_UUID=? AND ");
        }

        sql.append(wc);

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;

        statement.setString(i++, template.getClass().getName());

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        statement.execute();

        ResultSet resultSet = statement.getResultSet();
        int ret = 0;

        while (resultSet.next()) {
            try {
                String uuid = resultSet.getString(1);
                String value = resultSet.getString(2);

                DataModel dataModel = template.duplicate(null, new CopyPairs());

                dataModel.setInstanceName(uuid);
                dataModel.dropData(value);
                result.append(dataModel);
                ++ret;
            } catch (Exception e) {
                throw new RuntimeException(e);  //To change body of catch statement use File | Settings | File Templates.
            }
        }

        resultSet.close();
        statement.close();

        return ret;
    }

    public int search(ArrayModel result, DataModel template, String orderBy, WhereClause... whereClause) throws Exception {
        throw new Exception("not implemented");
    }

    private void print(StringBuffer sql) {
        System.out.println("sql.toString() = " + sql.toString());
    }

    public int load(ArrayModel result, DataModel template) throws SQLException {
        return search(result, template, null);
    }

    public int load(ArrayModel result, DataModel template, IDataModelFilter filter) throws Exception {
        if (filter == null) {
            return load(result, template);
        }

        ArrayModel array = new ArrayModel(template, "array");
        load(array, template);

        result.clear();
        DataModelEnumeration en = array.getEnumeration();

        while (en.hasMore()) {
            DataModel dataModel = en.next();

            if (filter.contains(dataModel)) {
                dataModel.setParentDataModelContainer(null);
                result.append(dataModel);
            }
        }

        return result.getSize();
    }

    //    public String putBlob(InputStream in, String mimeType) throws Exception {
    //        ByteArrayOutputStream out = new ByteArrayOutputStream();
    //
    //        int c = in.read();
    //
    //        while (c >= 0) {
    //            out.write((byte) c);
    //            c = in.read();
    //        }
    //
    //        in.close();
    //        out.close();
    //
    //        byte[] bytes = out.toByteArray();
    //
    //        return putBlob(bytes, mimeType);
    //    }

    public String putBlob(File file, String mimeType, String key) throws Exception {
        BufferedInputStream in = new XFile(file.getPath()).getBufferedInputStream();

        return putBlob(in, mimeType, key);
    }

    public String putBlob(InputStream in, String mimeType, String key) throws Exception {
        if (key == null || key.length() == 0) {
            key = UUID.randomUUID().toString();
        } else {
            // check whether key already exists
            StringBuffer sql = new StringBuffer("SELECT UUID FROM ");

            sql.append(TABLE_JNEX_BLOB);

            if (namespace == null) {
                sql.append(" WHERE UUID=?;");
            } else {
                sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
            }

            print(sql);

            PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

            int i = 1;
            statement.setString(i++, key);

            if (namespace != null) {
                statement.setString(i++, namespace.getId());
            }

            statement.execute();

            ResultSet resultSet = statement.getResultSet();

            if (resultSet.next()) {
                resultSet.close();
                statement.close();

                System.out.println("Ignore insert entry: " + key + " (already exists)");

                return key;
            }

            resultSet.close();
            statement.close();
        }

        StringBuffer sql = new StringBuffer("INSERT INTO ");
        long hashValue = 0L;

        sql.append(TABLE_JNEX_BLOB);

        if (namespace == null) {
            sql.append(" (UUID,MIME_TYPE,HASH,VALUE) VALUES (?,?,?,?);");
        } else {
            sql.append(" (NAMESPACE_UUID,UUID,MIME_TYPE,HASH,VALUE) VALUES (?,?,?,?,?);");
        }

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }
        statement.setString(i++, key);
        statement.setString(i++, mimeType);
        statement.setLong(i++, hashValue);
        statement.setBinaryStream(i++, in, Integer.MAX_VALUE);
        statement.execute();

        statement.close();

        return key;
    }

    public InputStream getBlob(String key) throws Exception {
        StringBuffer sql = new StringBuffer("SELECT VALUE FROM ");

        sql.append(TABLE_JNEX_BLOB);
        if (namespace == null) {
            sql.append(" WHERE UUID=?;");
        } else {
            sql.append(" WHERE UUID=? AND NAMESPACE_UUID=?;");
        }

        print(sql);

        PreparedStatement statement = jdbcConnectionHandler.getConnection(false).prepareStatement(sql.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);

        int i = 1;
        statement.setString(i++, key);

        if (namespace != null) {
            statement.setString(i++, namespace.getId());
        }

        statement.execute();

        ResultSet resultSet = statement.getResultSet();

        if (resultSet.next()) {
            Blob blob = resultSet.getBlob(1);

            InputStream ret = blob.getBinaryStream();

            resultSet.close();
            statement.close();

            return ret;
        } else {
            throw new Exception("No such entry: " + key);
        }
    }

    /**
     * update search attributes table of given class or all search tables if modelClass == null
     */
    public void updateSearchAttributes(String modelClass) throws Exception {
        if (modelClass == null) {
            StringVector classNames = searchAttributesTable.getDataModelClassNames();

            for (String name = classNames.firstItem(); name != null; name = classNames.nextItem()) {
                doUpdateSearchAttributes(name);
            }
        } else {
            doUpdateSearchAttributes(modelClass);
        }
    }

    private void doUpdateSearchAttributes(String modelClass) throws Exception {
        DataModelSearchAttributes dataModelSearchAttributes = searchAttributesTable.getSearchAttributes(modelClass);

        if (dataModelSearchAttributes != null) {
            Class<?> clazz = Class.forName(modelClass);
            DataModel template = (DataModel) clazz.newInstance();
            ArrayModel arr = new ArrayModel(new IntModel(0, ""), "");
            search(arr, template, null);
            DataModelEnumeration en = arr.getEnumeration();

            while (en.hasMore()) {
                DataModel dataModel = en.next();
                if (dataModelSearchAttributes.hasSearchAttributes(jdbcConnectionHandler, dataModel.getInstanceName(), namespace)) {
                    dataModelSearchAttributes.updateAttributes(jdbcConnectionHandler, dataModel, namespace);
                } else {
                    dataModelSearchAttributes.insertAttributes(jdbcConnectionHandler, dataModel, namespace);
                }
            }
        }
    }

	@Override
	public String testConnection() {
		if(jdbcConnectionHandler != null) {
			try {
				Connection con = jdbcConnectionHandler.getConnection(true);
				if(con.isValid(5)) {
					return "OK";
				}
			} catch (SQLException e) {
				return e.getMessage();
			}
		}
		return "Error";
	}
}