//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import java.awt.Component;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import com.nexirius.framework.datamodel.*;


/**
 * A class that represents any kind of viewer to display/edit DataModel information. It has an inner class
 * SynchronizedDataModelListener to make sure that visible update of the asssociated Swing component is
 * executed synchroniously with the swing redraw thread.
 *
 * @author Marcel Baumann
 * @see com.nexirius.framework.datamodel.SynchronizedDataModelListener
 */
public abstract class DataViewer implements SynchronizedDataModelListener, AncestorListener {
//	public static int instanceCount = 0;
//	public static Hashtable instanceTable = new Hashtable();
    protected DataModel model;
    protected JComponent component;
    protected LayoutItem layout = null;
    protected ViewerFactory factory;

    /**
     * Creates a new viewer based on the given data model
     */
    public DataViewer(DataModel model) {
/* FOR DEBUG
		++instanceCount;
		Integer i = (Integer)instanceTable.get(getClass());

		if (i == null) {
			instanceTable.put(getClass(), new Integer(1));
		} else {
			instanceTable.put(getClass(), new Integer(i.intValue() + 1));
		}
*/
        this.model = model;
    }

    protected void finalize() {
/* FOR DEBUG
		--instanceCount;
		System.out.println(instanceCount + " Finalize " + getClass().getName());
		Integer i = (Integer)instanceTable.get(getClass());

		instanceTable.put(getClass(), new Integer(i.intValue() - 1));
*/
    }

    public static void viewInstanceTable() {
/* FOR DEBUG
		Enumeration e = instanceTable.keys();

		System.out.println("============================");
		while (e.hasMoreElements()) {
			Class cl = (Class)e.nextElement();
			System.out.println(instanceTable.get(cl) + " " + cl.getName());
		}
*/
    }

    /**
     * Returns true if the associated Swing component was already build.
     */
    public boolean isCreated() {
        return component != null;
    }

    /**
     * Returns true is the assotioated Swing component is used to change the value if the model.
     */
    public boolean isEditor() {
        return false;
    }

    /**
     * Calls update() if the event is not coming from a child data model
     *
     * @see #update
     */
    public void dataModelChangeValue(DataModelEvent event) {
        if (!event.fromChild()) {
            update();
        }

        handleInvisibleFlag(event);
    }

    /**
     * The value of the data model is about to be changed
     */
    @Override
    public void dataModelWantToChangeValue(VetoableDataModelEvent event)
    {
        // Empty
    }
    
    protected void handleInvisibleFlag(DataModelEvent event) {
        if (isCreated() && (event == null || (event.getDataModel() == getDataModel()))) {
            getJComponent().setVisible(!getDataModel().isInvisible());
        }
    }

    /**
     * Does nothing
     */
    public void dataModelChangeStructure(DataModelEvent event) {
    }

    /**
     * Calls grabFocus() method
     *
     * @see #grabFocus
     */
    public void dataModelGrabFocus(DataModelEvent event) {
        grabFocus();
    }

    public void dataModelEdit(DataModelEvent event) {
    }

/*
	class MyDataModelListener implements SynchronizedDataModelListener
	{
		public void dataModelChangeValue(DataModelEvent event)
		{
			DataViewer.this.dataModelChangeValue(event);
		}

		public void dataModelChangeStructure(DataModelEvent event)
		{
			DataViewer.this.dataModelChangeStructure(event);
		}

		public void dataModelGrabFocus(DataModelEvent event)
		{
			DataViewer.this.dataModelGrabFocus(event);
		}
	}
*/
    public DataModelListener getDataModelListener() {
        return this;
    }

    /**
     * Returns the associated data model instance
     */
    public final DataModel getDataModel() {
        return this.model;
    }

    /**
     * Redefine this function to set the default button of a dialog.
     * Call: viewer.defineDefaultButton(dialog.getRootPane())
     */
    public void defineDefaultButton(JRootPane rootPane) {
    }

    /**
     * Recursively decends the whole subtree of JComponents and looks for one which is defined
     * by the given name. The name of each automatically (by the ViewerFactory) created component is
     * the same as the field name of the assiciated data model (or the command name of the associated command)!
     *
     * @param viewerId The field name (or command name) of the associated data model instance
     */
    public JComponent getSubcomponent(String viewerId)
            throws Exception {
        if (!isCreated()) {
            throw new Exception("Can't access a subcomponent while parent component is not yet created");
        }

        JComponent comp = component;

        return getSubcomponentByName(comp, viewerId);
    }

    public static JComponent getSubcomponentByName(JComponent parentComponent, String subcomponentName) throws Exception {
        if (parentComponent instanceof JScrollPane) {
            try {
                parentComponent = (JComponent) ((JScrollPane) parentComponent).getViewport().getView();
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }

        if (parentComponent instanceof JTabbedPane) {
            try {
                JTabbedPane tp = (JTabbedPane) parentComponent;
                Component tabs[] = new Component[tp.getComponentCount()];

                for (int i = 0; i < tp.getComponentCount(); ++i) {
                    tabs[i] = tp.getComponent(i);
                }

                JComponent ret = findSubComponent(tabs, subcomponentName);

                if (ret != null) {

                    return ret;
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }

        if (subcomponentName.equals(parentComponent.getName())) {

            return parentComponent;
        }

        Component comps[] = parentComponent.getComponents();

        JComponent ret = findSubComponent(comps, subcomponentName);

        if (ret == null) {
            throw new Exception("Could not find subcomponent by name (name = " + subcomponentName + ")");
        }

        return ret;
    }

    private static JComponent findSubComponent(Component comps[], String name) {
        JComponent ret = null;

        if (comps != null && comps.length > 0) {

            for (int i = 0; i < comps.length; ++i) {

                if (comps[i] instanceof JTabbedPane) {
                    try {
                        JTabbedPane tp = (JTabbedPane) comps[i];
                        Component tabs[] = new Component[tp.getComponentCount()];

                        for (int j = 0; j < tp.getComponentCount(); ++j) {
                            tabs[j] = tp.getComponent(j);
                        }

                        ret = findSubComponent(tabs, name);

                        if (ret != null) {

                            return ret;
                        }
                    } catch (Exception ex) {
                        throw new RuntimeException(ex);
                        //ignore
                    }
                }

                if (comps[i] instanceof JScrollPane) {
                    try {
                        comps[i] = ((JScrollPane) comps[i]).getViewport().getView();
                    } catch (Exception ex) {
                        throw new RuntimeException(ex);
                    }
                }

                if (comps[i] instanceof JComponent) {

                    if (name.equals(comps[i].getName())) {

                        return (JComponent) comps[i];
                    }

                    ret = findSubComponent(((JComponent) comps[i]).getComponents(), name);

                    if (ret != null) {

                        break;
                    }
                }
            }
        }

        return ret;
    }

    /**
     * Called by the ViewerFactory to introduce itself into the viewer after its creation.
     */
    public void setFactory(ViewerFactory factory) {
        this.factory = factory;
    }

    /**
     * This method can be called before the create method of the viewer is called to associate a
     * specific layout to the data viewer.
     *
     * @see #create
     */
    public void setLayout(LayoutItem item) {
        layout = item;
    }

    /**
     * Returns the associated data model layout or null
     */
    public LayoutItem getLayout() {
        if (layout == null) {
            layout = factory.getViewerCreatorMap().getLayoutItem(model, isEditor());
        }

        return layout;
    }

    /**
     * Returns the factory which created this instance.
     */
    public ViewerFactory getFactory() {
        return factory;
    }

    /**
     * Returns the associated Swing JComponent. If the component was not created then this
     * method automatically calls the create method first.
     *
     * @see #create
     */
    public final JComponent getJComponent() {
        if (!isCreated()) {
            create();
            this.model.addSoftDataModelListener(getDataModelListener());
        }

        return component;
    }

    /**
     * This method is called by the subclasses to introduce the associated JComponent which is
     * created by the create method. At the same time the components name is and help is set to
     * the associated data model field name (or command name).
     *
     * @see com.nexirius.framework.datamodel.DataModel#getFieldName
     * @see com.nexirius.framework.datamodel.DefaultDataModelCommand#getFieldName
     */
    protected final void setJComponent(JComponent component) {
        this.component = component;

        if (component != null) {
            Component c = component;

            if (component instanceof JScrollPane) {
                c = ((JScrollPane) component).getViewport().getView();
            }

            c.setName(getDataModel().getFieldName());

            // registerTemplate this instance to be a property listener of the created jcomponent
            // to avoid garbage collector removing this instance before the associated component is dropped
            component.addAncestorListener(this);

            // fire VIEWER_CREATED event on the associated DataModel

            if (getDataModel() != null) {
                DataModelEvent event = new DataModelEvent(getDataModel(), DataModelEvent.VIEWER_CREATED, true);

                event.setViewer(this);
                getDataModel().propagateChildEvent(event);
            }
        }
    }

    /**
     * This method is only implemented with data viewers which need a single function to introduce new
     * components to a container.
     */
    public void add(JComponent component) {
    }

    /**
     * Creates the actual JComponent (Actually this method is normally called the first time the getJComponent
     * method is called. The create function calls setJComponent to introduce the newly created component.
     *
     * @see #setJComponent
     * @see #getJComponent
     * @see #isCreated
     */
    public abstract void create();

    /**
     * Redisplay the data from the associated model
     */
    public abstract void update();

    /**
     * Returns the name of the viewer class (for debugging)
     */
    public abstract String getViewerName();

    /**
     * Calls grabFocus on the assotiated JComponent (only if the components isFocusTraversable() method returns true)
     */
    public void grabFocus() {
        if (isCreated()) {
            JComponent comp = getJComponent();

            if (comp instanceof JScrollPane) {
                try {
                    comp = (JComponent) ((JScrollPane) comp).getViewport().getView();
                } catch (Exception ex) {
                    throw new RuntimeException(ex);
                }
            }

            if (!(comp.getClass() == JPanel.class) &&  comp.isFocusable()) {
                comp.grabFocus();
            }
        }
    }

    /**
     * this method is called when a struct layout component was created and the background color
     * has to be changed to a specific color (setGray) or invisible
     */
    public void updateUI() {
        if (isCreated()) {
            getJComponent().setVisible(!getDataModel().isInvisible());
        }
    }

    /**
     * AncestorListener implementation (empty functions)
     */
    public void ancestorAdded(AncestorEvent event) {
    }

    public void ancestorMoved(AncestorEvent event) {
    }

    public void ancestorRemoved(AncestorEvent event) {
    }
}
