package com.nexirius.framework.gadgets;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.*;

/**
 * A panel holding components, which can be folded/unfolded by clicking on the title
 */
public class FoldingLayoutPanel extends JPanel {
    private JPanel top = new JPanel();
    private FoldingActionListener foldingActionListener = new FoldingActionListener();

    public FoldingLayoutPanel() {
        super();
        setLayout(new BorderLayout());
        top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
        add(top, BorderLayout.NORTH);
        add(new JPanel(), BorderLayout.CENTER);
    }

    public void addFoldingPanel(Object key, JComponent component, String title, boolean folded) {
        JLabel label = new JLabel(title);

        label.setOpaque(false);

        addFoldingPanel(key, component, label, folded);
    }

    public void addFoldingPanel(Object key, JComponent component, JComponent titleComponent, boolean folded) {
        top.add(new FoldingComponent(key, component, titleComponent, folded));
    }

    public void setFoldedAll(boolean folded) {
        Component[] components = top.getComponents();

        for (int i = 0; i < components.length; i++) {
            FoldingComponent component = (FoldingComponent) components[i];
            component.setFolded(folded);
        }
    }

    public void setFolded(Object key, boolean folded) {
        Component[] components = top.getComponents();

        for (int i = 0; i < components.length; i++) {
            FoldingComponent component = (FoldingComponent) components[i];

            if (component.isKey(key)) {
                component.setFolded(folded);
            }
        }
    }

    public boolean isFolded(Object key) {
        Component[] components = top.getComponents();

        for (int i = 0; i < components.length; i++) {
            FoldingComponent component = (FoldingComponent) components[i];

            if (component.isKey(key)) {
                return component.isFolded();
            }
        }

        return false;
    }

    public String[] getUnFoldedKeys() {
        String ret = "";
        Component[] components = top.getComponents();

        for (int i = 0; i < components.length; i++) {
            FoldingComponent component = (FoldingComponent) components[i];

            if (!component.getFolded()) {
                if (ret.length() == 0) {
                    ret = (String) component.getKey();
                } else {
                    ret += "," + component.getKey();
                }
            }
        }
        return ret.split(",");
    }

    private boolean isInKeys(String[] keys, String key) {
        if(keys.length > 0) {
            for(int i = 0; i < keys.length; i++) {
                if(keys[i].equalsIgnoreCase(key)) {
                    return true;
                }
            }
        }
        return false;
    }

    public void setUnFoldedKeys(String[] keys) {
        if (keys.length > 0) {
            Component[] components = top.getComponents();

            for (int i = 0; i < components.length; i++) {
                FoldingComponent component = (FoldingComponent) components[i];

                if (isInKeys(keys, component.getKey().toString())) {
                    component.setFolded(false);
                } else {
                    component.setFolded(true);
                }
            }

        }
    }

    /**
     * Set all components folded except the one registered by key
     *
     * @param key
     */
    public void setUnfoldedSingle(Object key) {
        Component[] components = top.getComponents();

        for (int i = 0; i < components.length; i++) {
            FoldingComponent component = (FoldingComponent) components[i];

            component.setFolded(!component.isKey(key));
        }
    }

    class FoldingComponent extends JPanel {
        private Object key;
        private JToggleButton foldingButton;
        private JComponent foldingComponent;
        private JComponent titleComponent;

        public FoldingComponent(Object key, JComponent component, JComponent titleComponent, boolean folded) {
            this.key = key;
            this.foldingComponent = component;
            foldingComponent.setBorder(new EmptyBorder(0, 30, 0, 0));
            this.titleComponent = titleComponent;
            init(folded);
        }

        private void init(boolean folded) {
            setLayout(new BorderLayout());
            this.foldingButton = new FoldingButton();
            foldingButton.setSelected(folded);
            foldingButton.addActionListener(foldingActionListener);
            BorderLayout borderLayout = new BorderLayout();

            borderLayout.setHgap(10);
            JPanel titlePanel = new JPanel(borderLayout);

            titlePanel.setBackground(SystemColor.controlHighlight);

            titlePanel.setBorder(new EmptyBorder(5, 5, 5, 5));

            JPanel buttonPanel = new JPanel(new BorderLayout());

            buttonPanel.setOpaque(false);
            buttonPanel.add(foldingButton, BorderLayout.NORTH);

            titlePanel.add(buttonPanel, BorderLayout.WEST);
            titlePanel.add(titleComponent, BorderLayout.CENTER);

            add(titlePanel, BorderLayout.NORTH);
            foldingComponent.setVisible(!folded);
            add(foldingComponent, BorderLayout.CENTER);
        }

        public boolean isKey(Object otherKey) {
            return otherKey != null && otherKey.equals(key);
        }

        public Object getKey() {
            return key;
        }

        public boolean isFoldingButton(Object component) {
            return component == foldingButton;
        }

        public void setFolded(boolean on) {
            if (foldingButton.isSelected() != on) {
                foldingButton.setSelected(on);
                foldingChanged();
            }
        }

        public boolean isFolded() {
            return foldingButton.isSelected();
        }

        public boolean getFolded() {
            return foldingButton.isSelected();
        }

        public void foldingChanged() {
            foldingComponent.setVisible(!getFolded());
        }
    }

    class FoldingActionListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Component[] components = top.getComponents();
            for (int i = 0; i < components.length; i++) {
                Component component = components[i];

                if (component instanceof FoldingComponent) {
                    FoldingComponent foldingComponent = (FoldingComponent) component;
                    if (foldingComponent.isFoldingButton(e.getSource())) {
                        foldingComponent.foldingChanged();

                        break;
                    }
                }
            }
        }
    }

    public static class FoldingButton extends JToggleButton {
        public FoldingButton() {
            setBorder(new EmptyBorder(1, 1, 1, 1));
            setPreferredSize(new Dimension(20, 20));
            setOpaque(false);
        }

        public void paint(Graphics g) {
            Dimension size = getSize();
            int x = size.width / 5;
            int y = size.height / 5;
            int w = size.width - x - x;
            int h = size.height - y - y;

            int[] xp;
            int[] yp;

            if (isSelected()) {
                xp = new int[]{x, x + w, x};
                yp = new int[]{y, y + h / 2, y + h};
            } else {
                xp = new int[]{x, x + w, x + w / 2};
                yp = new int[]{y, y, y + h};
            }

            Color c1 = new Color(220, 220, 250);
            Color c2 = new Color(isSelected() ? 240 : 180, isSelected() ? 180 : 240, 180);
            Color c3 = new Color(80, 80, 140);

            g.setColor(c2);
            g.fillPolygon(xp, yp, xp.length);
            g.setColor(c1);
            g.drawPolygon(xp, yp, xp.length);
            g.setColor(c3);
            g.drawLine(xp[1], yp[1], xp[2], yp[2]);
        }
    }

    public static void main(String[] argv) {
        JFrame frame = new JFrame("FoldingLayoutPanel");

        frame.getContentPane().setLayout(new GridLayout(1, 1));
        final FoldingLayoutPanel fold = new FoldingLayoutPanel();
        JScrollPane scrollpane = new JScrollPane(fold);
        scrollpane.getVerticalScrollBar().setUnitIncrement(40);
        scrollpane.getHorizontalScrollBar().setUnitIncrement(40);
        frame.getContentPane().add(scrollpane);

        JLabel l1 = new JLabel("HALLO");
        l1.setFont(new Font("Tahoma", Font.PLAIN, 40));
        l1.setOpaque(true);
        l1.setBackground(new Color(250, 220, 250));
        fold.addFoldingPanel("A", l1, "Hallo", true);

        JLabel l2 = new JLabel("VELO");
        l2.setFont(new Font("Tahoma", Font.PLAIN, 60));
        l2.setOpaque(true);
        l2.setBackground(new Color(200, 220, 250));
        fold.addFoldingPanel("B", l2, "Velo", false);

        JLabel l3 = new JLabel("Was");
        l3.setFont(new Font("Tahoma", Font.PLAIN, 60));
        l3.setOpaque(true);
        l3.setBackground(new Color(200, 240, 200));
        JButton button = new JButton("Was");
        button.setPreferredSize(new Dimension(400, 200));
        fold.addFoldingPanel("C", l3, button, true);

        button.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (fold.isFolded("C")) {
                    fold.setUnfoldedSingle("C");
                } else {
                    fold.setFolded("C", true);                    
                }
            }
        });

        frame.pack();
        frame.setLocation(100, 100);
        frame.setVisible(true);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    }
}
