//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.htmlview.HTMLParser.FunctionParameter;
import com.nexirius.framework.htmlview.function.DefHTMLFunction;
import com.nexirius.framework.htmlview.function.ForHTMLFunction;
import com.nexirius.framework.htmlview.function.HTMLFunction;
import com.nexirius.framework.htmlview.function.IfHTMLFunction;
import com.nexirius.util.TextToken;
import com.nexirius.util.StringVector;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * This class translates a HTML file which contains special functions into a real HTML output stream<BR>
 * special functions:
 * <PRE>
 * $!include("template")
 * $!includeBody("template")
 * $!field("field name", "edit|view", "template")
 * $!popup("childname", view|edit, "template", "link text", "event", "true|false") if parameter 6 is true, then the actual model is duplicated in the actual state with HTMLState.startDuplicatePopup()
 * $!event("event", "link text", "&moreParameter=value")
 * $!iconEvent("event", "iconName", "&moreParameter=value")
 * $!button("event")
 * $!array("field name of the array", "edit|view", "template")
 * $!flag("field name", "flag", "on text", "off text")
 * $!select("field name")
 * $!define("name", "value")
 * $!formStart()
 * $!formEnd()
 * </PRE>
 */
public class HTMLParser {

    public static final String NULL = "null";
    public static final String EDIT = "edit";
    public static final String VIEW = "view";
    public static final String CHECKED = "CHECKED";

    static String[] startTokens = new String[]{"$!IF", "$!FOR", "$!DEF"};
    static String[] endTokens = new String[]{IfHTMLFunction.ENDIF, ForHTMLFunction.ENDFOR, DefHTMLFunction.ENDDEF, };


    DataModel model;
    boolean isEditor;
    HTMLResolver resolver;
    OutputStream out;
    int translateCount = 0;

    /**
     * creates a new instance of HTMLParser
     *
     * @param model    the actual model which is assotiated to the input stream (see parse)
     * @param isEditor the actual general translation rule
     * @param resolver the resolver wich created this parser (and which is used to handle subsequent nested function calls
     * @param out      the actual output stream
     * @throws Exception
     */
    public HTMLParser(DataModel model, boolean isEditor, HTMLResolver resolver, OutputStream out)
            throws Exception {
        this.model = model;
        this.isEditor = isEditor;
        this.resolver = resolver;
        this.out = out;
    }

    public DataModel getModel() {
        return model;
    }

    public boolean isEditor() {
        return isEditor;
    }

    public HTMLResolver getResolver() {
        return resolver;
    }

    /**
     * Return the model which belongs to the actual state
     *
     * @return the root model
     */
    public DataModel getRootModel() {
        return resolver.getRootModel();
    }

    public OutputStream getOut() {
        return out;
    }

    public void setOut(OutputStream out) {
        this.out = out;
    }

    /**
     * parses the input stream and calls parseFunction when the $! is found
     * skips slash star star slash block comments
     *
     * @param sessionVariable
     * @param in              the actual input stream
     * @return the number of functions which are translated
     * @throws Exception
     */
    public int parse(HTMLSessionVariable sessionVariable, PushbackInputStream in)
            throws Exception {
        int state = 0;
        translateCount = 0;

        while (true) {
            int c = in.read();

            if (c < 0) {

                break;
            }

            switch (state) {
                case 0: // outside
                    if (c == '$') {
                        state = 1;
                    } else if (c == '/') {
                        state = 2;
                    } else {
                        out.write(c);
                    }
                    break;

                case 1: // after $
                    if (c == '!') {
                        state = 0;
                        try {
                            HTMLFunction f = parseFunction(sessionVariable, in);
                            List<FunctionParameter> arguments = parseFunctionArguments(sessionVariable, in);

                            if (f.getFunctionName().equals(IfHTMLFunction.FUNCTION_NAME)) {
                                // parse up to $ELSE$ and $ENDIF$
                                ArrayList<TextBlock> parts = parseBlocks(in, new String[]{IfHTMLFunction.ELSE});

                                //StringVector sv = new StringVector(arguments);

                                if (parts.size() > 2) {
                                    throw new Exception("Unexpected number of $ELSE$ in $IF: " + parts.size());
                                }

                                arguments.add(new FunctionParameter(parts.get(0).text));

                                if (parts.size() == 2) {
                                    arguments.add(new FunctionParameter(parts.get(1).text));
                                } else {
                                    arguments.add(new FunctionParameter(""));
                                }

                            } else if (f.getFunctionName().equals(ForHTMLFunction.FUNCTION_NAME)) {
                                // parse up to $ENDFOR$
                                ArrayList<TextBlock> parts = parseBlocks(in, new String[]{});


                                if (parts.size() > 1) {
                                    throw new Exception("Unexpected number of blocks in $!FOR: " + parts.size());
                                }

                                arguments.add(new FunctionParameter(parts.get(0).text));

                                
                            } else if (f.getFunctionName().equals(DefHTMLFunction.FUNCTION_NAME)) {
                                // parse up to $ENDDEF$
                                ArrayList<TextBlock> parts = parseBlocks(in, new String[]{});


                                if (parts.size() > 1) {
                                    throw new Exception("Unexpected number of blocks in $!DEF: " + parts.size());
                                }

                                arguments.add(new FunctionParameter(parts.get(0).text));
                            }

                            f.translate(sessionVariable, this, getParameterAsStrings(arguments));
                            ++translateCount;
                        } catch (Exception ex) {
                            handleFunctionCallException(ex);
                        }
                    } else {
                        out.write('$');
                        out.write(c);
                        state = 0;
                    }
                    break;

                case 2: // after /
                    if (c == '*') {
                        state = 3;
                    } else {
                        out.write('/');
                        out.write(c);
                        state = 0;
                    }
                    break;

                case 3: // in comment
                    if (c == '*') {
                        state = 4;
                    }
                    break;

                case 4: // in comment after *
                    if (c == '/') {
                        state = 0;
                    }
                    break;

            }
        }

        return translateCount;
    }

    protected void handleFunctionCallException(Exception ex) {
        ex.printStackTrace(new PrintStream(out));
    }

    private static ArrayList<TextBlock> parseBlocks(PushbackInputStream in, String[] intermediateTokens) throws Exception {
        int nested = 0;

        ArrayList<TextBlock> ret = new ArrayList<TextBlock>();
        ArrayList<ParsePosition> parsePositions = new ArrayList<ParsePosition>();
        TextBlock textBlock = new TextBlock("");
        StringBuffer text = new StringBuffer();
        initParsePositions(intermediateTokens, parsePositions);

        while (true) {
            int c = in.read();

            if (c < 0) {
                throw new Exception("Unexpected EOF");
            }

            boolean tokenFound = false;

            for (int i = 0; i < parsePositions.size(); ++i) {
                ParsePosition position = parsePositions.get(i);
                boolean isStartToken = position.isStartToken;
                boolean isEndToken = position.isEndToken;
                boolean isIntermediateToken = (!isStartToken && !isEndToken);
                String token = position.token;

                if (c == token.charAt(position.index)) {
                    position.index += 1;

                    if (position.index == token.length()) {
                        // token found
                        tokenFound = true;
                        // reset all parse position
                        initParsePositions(intermediateTokens, parsePositions);

                        if (nested == 0 && !isStartToken) {
                            textBlock.text = text.toString().substring(0, text.length() - token.length() + 1);
                            ret.add(textBlock);
                        }

                        if (isEndToken) {
                            if (nested == 0) {
                                return ret;
                            }

                            text.append((char) c);                            
                            --nested;                            
                        }

                        if (isStartToken) {
                            ++nested;
                        }

                        if (nested == 0) {
                            if (isIntermediateToken) {
                                // create new TextBlock
                                textBlock = new TextBlock(token);
                                text = new StringBuffer();
                            }
                        }
                    }
                } else {
                    position.index = 0;
                }
            }

            if (nested > 0 || !tokenFound) {
                text.append((char) c);
            }
        }
    }

    private static void initParsePositions(String[] intermediateTokens, ArrayList<ParsePosition> parsePositions) {
        parsePositions.clear();

        for (int i = 0; i < startTokens.length; ++i) {
            parsePositions.add(new ParsePosition(0, startTokens[i], true, false));
        }
        for (int i = 0; i < endTokens.length; ++i) {
            parsePositions.add(new ParsePosition(0, endTokens[i], false, true));
        }
        for (int i = 0; i < intermediateTokens.length; ++i) {
            parsePositions.add(new ParsePosition(0, intermediateTokens[i], false, false));
        }
    }

    /**
     * reads the input stream and parses !$function() calls
     *
     * @param in the input stream from which functions are extracted
     * @return the actual function
     * @throws Exception when syntax of the function is not correct
     */
    public HTMLFunction parseFunction(HTMLSessionVariable sessionVariable, PushbackInputStream in)
            throws Exception {
        HTMLFunction ret = null;
        TextToken token = TextToken.nextToken(in);

        if (token == null) {
            throw new Exception("Expecting function name at end of stream");
        }

        if (token.isIdentifier()) {
            ret = resolver.getFunction(token.getString());
        } else {
            throw new Exception("Expecting function name but have " + token.debugString());
        }

        if (ret == null) {
            throw new Exception("Unknown function name " + token.debugString());
        }

        return ret;
    }

    class FunctionParameter {
        String simpleParameter;

        public FunctionParameter(String s) {
            simpleParameter = s;
        }
        
        public FunctionParameter(HTMLFunction function, HTMLSessionVariable sessionVariable, PushbackInputStream in) throws Exception {
            List<FunctionParameter> parameterFunctionArguments = parseFunctionArguments(sessionVariable, in);
            
            StringBuffer sb = new StringBuffer("$!" + function.getFunctionName() + "(");
            boolean first = true;
            for (FunctionParameter param : parameterFunctionArguments) {
                if(!first) {
                    sb.append(",");

                } else {
                    first = false;
                }
                sb.append(TextToken.quoteString(param.simpleParameter));
            }
            
            sb.append(")");
            
            byte[] bytes = getResolver().resolve(sessionVariable, model, new ByteArrayInputStream(sb.toString().getBytes(HTMLResolver.CHARSET)), isEditor);
            simpleParameter = new String(bytes, HTMLResolver.CHARSET);
        }
        
                
    }
    
    public static String[] getParameterAsStrings(List<FunctionParameter> list) {
        String[] ret = new String[list.size()];
        int i = 0;
        for (FunctionParameter param : list) {
            ret[i++] = param.simpleParameter;
        }
        return ret;
    }

    
    public List<FunctionParameter> parseFunctionArguments(HTMLSessionVariable sessionVariable, PushbackInputStream in) throws Exception {
        List<FunctionParameter> functionArgumentList = new ArrayList<FunctionParameter>();
        TextToken token;
        token = TextToken.nextToken(in);

        if (token == null) {
            throw new Exception("Expecting '(' after function name");
        }

        if (token.isChar('(')) {
            while (true) {
                token = TextToken.nextToken(in);

                if (token.isChar(')')) {

                    break;
                }

                if(token.isChar('$')) {
                    token = TextToken.nextToken(in);
                    if(token == null) {
                        throw new Exception("Unexpected end of file in parameterlist:" + token.debugString());
                    } else {
                        if(token.isChar('!')) {
                            HTMLFunction function = parseFunction(sessionVariable, in);
                            functionArgumentList.add(new FunctionParameter(function, sessionVariable, in));
                        } else {
                            throw new Exception("Expecting ! after $ in parameterlist as agrument have:" + token.toString());
                        }
                    }

                } else if (token.isString()) {
                    String s = token.getString();
                    s = resolve(sessionVariable, s);
                    functionArgumentList.add(new FunctionParameter(s));
                } else if (token.isInt()) {
                    functionArgumentList.add(new FunctionParameter(Integer.toString(token.getInt())));
                } else if (token.isIdentifier(NULL)) {
                    functionArgumentList.add(new FunctionParameter((String) null));
                } else if (token.isIdentifier()) {
                    functionArgumentList.add(new FunctionParameter(token.getString()));
                } else {
                    throw new Exception("Unexpected token in parameter list:" + token.debugString());
                }

                token = TextToken.nextToken(in);

                if (token.isChar(',')) {
                    // expected
                } else if (token.isChar(')')) {
                    break;
                } else {
                    throw new Exception("Unexpected token in parameter list:" + token.debugString());
                }
            }
        } else {
            throw new Exception("Expecting '(' but have " + token.debugString());
        }

        return functionArgumentList;
    }

    public String resolve(HTMLSessionVariable sessionVariable, String s) throws Exception {
        byte[] bytes = getResolver().resolve(sessionVariable, model, new ByteArrayInputStream(s.getBytes(HTMLResolver.CHARSET)), isEditor);
        return new String(bytes, HTMLResolver.CHARSET);
    }

    /**
     * access the number of translated functions
     *
     * @return the number of translations
     */
    public int getNumberOfTranslations() {
        return translateCount;
    }

    private static class TextBlock {
        public String token;
        public String text;

        private TextBlock(String token) {
            this.token = token;
        }

        @Override
        public String toString() {
            return token + "[" + text + "]";
        }
    }

    private static class ParsePosition {
        public int index;
        public String token;
        public boolean isStartToken;
        public boolean isEndToken;

        private ParsePosition(int index, String token, boolean startToken, boolean endToken) {
            this.index = index;
            this.token = token;
            isStartToken = startToken;
            isEndToken = endToken;
        }
    }

    public static void main(String[] args) throws Exception {
        String[] s = {"-hier ist ein -$ELSE$- und $!IF() $ELSE$ $!FOR $ENDFOR$ $ENDIF$ ein -$ENDIF$- soso", "(i,\"10\")A[$!get(i)] $!FOR(j,\"3\") j=$!get(j) $ENDFOR$ $ENDFOR$ "};

        for (String ss: s) {
            PushbackInputStream in = new PushbackInputStream(new ByteArrayInputStream(ss.getBytes()));

            ArrayList<TextBlock> parts = parseBlocks(in, new String[]{IfHTMLFunction.ELSE});

            System.out.println(ss);
            for (TextBlock part : parts) {
                System.out.println(part);
            }

        }
    }
}
