//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.FWLog;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.LayoutItem;
import com.nexirius.framework.dataviewer.ViewerFactory;
import com.nexirius.util.assertion.Assert;

import javax.swing.*;
import java.awt.*;

/**
 * This is the base class for all the layout elements which define location,
 * size, properties and name of a specific component.
 *
 * @author marcel Baumann
 */
public class DefaultLayoutItem implements LayoutItem {



    protected String modelFieldName;
    protected String viewerId;
    protected Rectangle bounds = new Rectangle();
    protected PropertyEntry properties = null;
    protected GridBagConstraints gridBagConstraints = null;

    /**
     * @param field The associated data model field name
     * @param x     the x position of the component
     * @param y     the y position of the component
     * @param w     the width of the component (if  0 then the preferred width is used)
     * @param h     the height of the component (if  0 then the preferred height is used)
     */
    public DefaultLayoutItem(String field, int x, int y, int w, int h) {
        setModelFieldName(field);
        setBounds(x, y, w, h);
    }

    /**
     * @param field    The associated data model field name
     * @param x        the x position of the component
     * @param y        the y position of the component
     * @param w        the width of the component (if  0 then the preferred width is used)
     * @param h        the height of the component (if  0 then the preferred height is used)
     * @param viewerId Use the specified id as the component name (this name is used with DataViewer.getSubcomponent)
     * @see com.nexirius.framework.dataviewer.DataViewer#getSubcomponent
     */
    public DefaultLayoutItem(String field, int x, int y, int w, int h, String viewerId) {
        this(field, x, y, w, h);

        setViewerId(viewerId);
    }

    /**
     * @param field    The associated structure field name
     * @param x        the x position of the component
     * @param y        the y position of the component
     * @param w        the width of the component (if  0 then the preferred width is used)
     * @param h        the height of the component (if  0 then the preferred height is used)
     * @param viewerId Use the specified id as the component name (this name is used with DataViewer.getSubcomponent)
     * @param prop     Any property changer
     * @see com.nexirius.framework.dataviewer.DataViewer#getSubcomponent
     */
    public DefaultLayoutItem(String field, int x, int y, int w, int h, String viewerId, PropertyEntry prop) {
        this(field, x, y, w, h, viewerId);
        this.properties = prop;
    }

    public DefaultLayoutItem() {
        super();
    }

    /**
     * Set contraints for use with GridBagLayout
     */
    public void setGridBagConstraints(GridBagConstraints c) {
        gridBagConstraints = c;
    }

    /**
     * Set a property entry.
     *
     * @param prop SinglePropertyEntry or MultiplePropertyEntry
     * @see SinglePropertyEntry
     * @see MultiplePropertyEntry
     */
    public void setProperties(PropertyEntry prop) {
        Assert.pre(this.properties == null, "Can't assign a new property information when already defined");
        this.properties = prop;
//		append((DataModel)prop);
    }

    /**
     * Direct access to the data model field name
     */
    public String getModelFieldName() {
        return modelFieldName;
    }

    /**
     * Direct write access to the data model field name
     */
    public void setModelFieldName(String name) {
        modelFieldName = name;
    }

    /**
     * Direct write access to the final component name
     */
    public void setViewerId(String name) {
        viewerId = name;
    }

    /**
     * Direct access to the data model field name
     */
    public String getViewerId() {
        return viewerId;
    }

    /**
     * Direct write access to bounding rectangle
     */
    public void setBounds(int x, int y, int w, int h) {
        bounds = new Rectangle(x, y, w, h);
    }

    /**
     * get bounding rectangle
     */
    public Rectangle getBounds() {
        return bounds;
    }

    /**
     * Direct write access to position (upper left corner of the bounding rectangle)
     */
    public void setLocation(int x, int y) {
        bounds.setLocation(x, y);
    }

    /**
     * Direct write access to size of the bounding rectangle
     */
    public void setSize(int w, int h) {
        bounds.setSize(w, h);
    }

    /**
     * Create and layout the appropriate component in the parent viewer
     */
    public void doLayout(DataViewer parent_viewer) {
    }

    /**
     * Execute the layout properties which are defined in the DefaultLayoutItem. this
     * method should be called by subclasses at the end of the creation of the child
     * component.
     *
     * @param comp    The child component which will be layed out
     * @param factory The factory which created the component
     */
    public void defaultLayout(JComponent comp, ViewerFactory factory) {
        new LayoutChanger(comp, factory).run();
//		SwingUtilities.invokeLater(new LayoutChanger(comp, factory));
    }

    class LayoutChanger implements Runnable {
        JComponent comp;
        ViewerFactory factory;

        LayoutChanger(JComponent comp, ViewerFactory factory) {
            this.comp = comp;
            this.factory = factory;
        }

        public void run() {
            if (properties != null) {
                try {
                    properties.change(comp, factory);
                } catch (Exception ex) {
                    //FIX
                    FWLog.log("properties not assigned: " + properties);
                    throw new RuntimeException(ex);
                }
            }

            if (gridBagConstraints == null) {
                comp.setLocation(bounds.getLocation());

                if (bounds.width != 0 && bounds.height != 0) {
                    comp.setPreferredSize(bounds.getSize());
                    comp.setSize(bounds.getSize());
                } else {
                    Dimension d = comp.getPreferredSize();
                    int w = bounds.width;
                    int h = bounds.height;

                    if (w > 0) {
                        d.width = w;
                    }

                    if (h > 0) {
                        d.height = h;
                    }

                    comp.setSize(d);
                }
            } else {
                // access parent container and make sure it uses a GridBagLayout
                if (comp.getParent() != null) {
                    GridBagLayout layout;

                    if (comp.getParent().getLayout() instanceof GridBagLayout) {
                        layout = (GridBagLayout) comp.getParent().getLayout();
                    } else {
                        layout = new GridBagLayout();
                        comp.getParent().setLayout(layout);
                    }

                    layout.setConstraints(comp, gridBagConstraints);
                }
            }

            String componentName = getModelFieldName();

            if (getViewerId() != null && getViewerId().length() > 0) {
                componentName = getViewerId();
            }

            if (componentName != null && componentName.length() > 0) {

                if (comp instanceof JScrollPane) {
                    ((JScrollPane) comp).getViewport().getView().setName(componentName);
                    comp.setName(componentName + ".ScrollPane");
                } else {
                    comp.setName(componentName);
/* mb 28.10.2002

					if (comp instanceof CFJItem) {
						CFJItem status = (CFJItem)comp;

						status.setResourceKey(componentName);
						status.update(factory.getClientResource());
					}
*/
                }
            }
        }
    }
}
