//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import java.awt.*;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This layout manager is resizing to the lower right corner of the
 * outer most component. Each component is layed out according to its
 * preferred size. This manager is showing only one component at a time
 *
 * @author Marcel Baumann
 */
public class StackLayoutManager extends DefaultLayoutManager {



    Hashtable tab = new Hashtable();
    Component current = null;

    public StackLayoutManager() {
    }

    /**
     * Adds the specified component to this stack layout's internal
     * table of names. The object specified by <code>constraints</code>
     * must be a string. The card layout stores this string as a key-value
     * pair that can be used for random access to a particular card.
     * By calling the <code>show</code> method, an application can
     * display the component with the specified name.
     *
     * @param comp        the component to be added.
     * @param constraints a tag that identifies a particular
     *                    card in the layout.
     * @throws IllegalArgumentException if the constraint is not a string.
     * @see java.awt.CardLayout#show(java.awt.Container, java.lang.String)
     */
    public void addLayoutComponent(Component comp, Object constraints) {
        synchronized (comp.getTreeLock()) {
            if (constraints instanceof String) {
                addLayoutComponent((String) constraints, comp);
            } else {
                throw new IllegalArgumentException("cannot add to layout: constraint must be a string");
            }
        }
    }

    /**
     * @deprecated replaced by
     *             <code>addLayoutComponent(Component, Object)</code>.
     */
    public void addLayoutComponent(String name, Component comp) {
        synchronized (comp.getTreeLock()) {
            if (tab.size() > 0) {
                comp.setVisible(false);
            } else {
                setCurrent(comp);
            }

            tab.put(name, comp);
        }
    }

    /**
     * Removes the specified component from the layout.
     *
     * @param comp the component to be removed.
     * @see java.awt.Container#remove(java.awt.Component)
     * @see java.awt.Container#removeAll()
     */
    public void removeLayoutComponent(Component comp) {
        synchronized (comp.getTreeLock()) {
            for (Enumeration e = tab.keys(); e.hasMoreElements();) {
                String key = (String) e.nextElement();

                if (tab.get(key) == comp) {
                    tab.remove(key);

                    if (comp == current) {
                        setCurrent(null);
                    }
                    return;
                }
            }
        }
    }

    /**
     * Flips to the component that was added to this layout with the
     * specified <code>name</code>, using <code>addLayoutComponent</code>.
     * If no such component exists, then nothing happens.
     *
     * @param parent the name of the parent container
     *               in which to do the layout.
     * @param name   the component name.
     * @see java.awt.CardLayout#addLayoutComponent(java.awt.Component, java.lang.Object)
     */
    public void show(Container parent, String name) {
        synchronized (parent.getTreeLock()) {
            Component next = (Component) tab.get(name);

            if ((next != null) && !next.isVisible()) {
                setCurrent(next);
                parent.validate();
            }
        }
    }

    private void setCurrent(Component comp) {
        if (current == comp) {
            return;
        }

        if (current != null) {
            current.setVisible(false);
        }

        current = comp;

        if (current != null) {
            current.setVisible(true);
        } else {
            if (tab.size() > 0) {
                String key = (String) tab.keys().nextElement();

                comp = (Component) tab.get(key);
                setCurrent(comp);
            }
        }
    }
}
