//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.LayoutItem;
import com.nexirius.framework.dataviewer.ViewerFactory;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.*;

public class TableColumnLayoutItem implements LayoutItem {

    protected String fieldIdentifier; // a ';' separated list of fieldnames naming a substructure element in a structure
    protected String columnLabel;
    protected int columnWidth = 0;
    protected boolean columnVisible = true;
    protected int minColumnWidth = 0;
    protected int maxColumnWidth = 0;
    protected Class<?> clazz;
    protected PropertyEntry properties = null;
    protected ViewerFactory factory = null;

    protected String modelFieldName[] = null;

    public TableColumnLayoutItem()
    {
        super();
    }

    /**
     * @param field       The associated structure field name (the name is direct field member name or a ';' separated list of substructure field names)
     * @param columnLabel this name is used to create the table header string for the specified column
     * @param columnWidth the initial width of the specified table column (when <= 0 a default width which depends on the total table width is assigned)
     */
    public TableColumnLayoutItem(String field, String columnLabel, int columnWidth)
    {
        this(field, columnLabel, columnWidth, Object.class);
    }

    /**
     * @param field       The associated structure field name (the name is direct field member name or a ';' separated list of substructure field names)
     * @param columnLabel this name is used to create the table header string for the specified column
     * @param columnWidth the initial width of the specified table column (when <= 0 a default width which depends on the total table width is assigned)
     * @param The class of the cell entries. May be used to choose the correct renderer and editor. Use Object.class if you don't
     * have a better idea on what to use!
     */
    public TableColumnLayoutItem(String field, String columnLabel, int columnWidth, Class<?> clazz)
    {
        super();
        this.fieldIdentifier = field;
        this.columnLabel = columnLabel;
        this.columnWidth = columnWidth;
        this.clazz = clazz;
    }

    /**
     * @param columnLabel this name is used to create the table header string for the specified column
     * @param columnWidth the initial width of the specified table column (when <= 0 a default width which depends on the total table width is assigned)
     */
    public TableColumnLayoutItem(String columnLabel, int columnWidth) {
        super();
        this.columnLabel = columnLabel;
        this.columnWidth = columnWidth;
    }

    /**
     * @param field       The associated structure field name (the name is direct field member name or a ';' separated list of substructure field names)
     * @param columnLabel this name is used to create the table header string for the specified column
     * @param columnWidth the initial width of the specified table column (when <= 0 a default width which depends on the total table width is assigned)
     * @param properties  the properties are changing the DefaultTableCellRenderer (which is a JLabel)
     */
    public TableColumnLayoutItem(String field, String columnLabel, int columnWidth, PropertyEntry properties) {
        this(field, columnLabel, columnWidth);
        this.properties = properties;
    }

    @Override
    public void doLayout(DataViewer parent_viewer)
    {
        factory = parent_viewer.getFactory();
        JTable table = searchTable(parent_viewer.getJComponent());

        SwingUtilities.invokeLater(new SetColumnRenderer(table));
    }

    private JTable searchTable(Component comp)
    {
        if (comp instanceof JTable)
        {
            return((JTable)comp);
        }

        if (comp instanceof Container)
        {
            for (Component childComp : ((Container)comp).getComponents())
            {
                JTable table = searchTable(childComp);
                if (table != null)
                {
                    return(table);
                }
            }
        }
        return null;
    }

    public boolean isColumnVisible()
    {
        return columnVisible;
    }

    public void setColumnVisible(boolean columnVisible)
    {
        if (this.columnVisible != columnVisible)
        {
            this.columnVisible = columnVisible;
        }
    }

    private class SetColumnRenderer implements Runnable {
        JTable table;

        SetColumnRenderer(JTable t) {
            table = t;
        }

        @Override
        public void run() {
            TableColumn column = table.getColumn(factory.getText(columnLabel));

            column.setCellRenderer(getTableCellRenderer());
            TableCellEditor tableCellEditor = getTableCellEditor();
            if (tableCellEditor != null)
            {
                column.setCellEditor(tableCellEditor);
            }

            if (columnWidth > 0)
            {
                column.setPreferredWidth(columnWidth);
            }
            if (minColumnWidth > 0)
            {
                column.setMinWidth(minColumnWidth);
            }
            if (maxColumnWidth > 0)
            {
                column.setMaxWidth(maxColumnWidth);
            }
            if (isColumnVisible() == false)
            {
                table.getColumnModel().removeColumn(column);
            }
        }
    }

    public TableCellRenderer getTableCellRenderer() {
        return new LayoutTableCellRenderer(this);
    }

    public TableCellEditor getTableCellEditor() {
        return null;
    }

    public String getColumnLabel() {
        return columnLabel;
    }

    public void setColumnLabel(String columnLabel)
    {
        this.columnLabel = columnLabel;
    }

    public Class<?> getColumnClass()
    {
        return clazz;
    }

    public int getColumnWidth()
    {
        return columnWidth;
    }

    public void setColumnWidth(int columnWidth)
    {
        this.columnWidth = columnWidth;
    }

    public int getMinColumnWidth()
    {
        return minColumnWidth;
    }

    public void setMinColumnWidth(int minColumnWidth)
    {
        this.minColumnWidth = minColumnWidth;
    }

    public int getMaxColumnWidth()
    {
        return maxColumnWidth;
    }

    public void setMaxColumnWidth(int maxColumnWidth)
    {
        this.maxColumnWidth = maxColumnWidth;
    }

    public String getModelFieldName() {
        return fieldIdentifier;
    }

    public String[] getModelFieldNameArray() {
        if (modelFieldName == null) {
            modelFieldName = DataModel.getFieldNameArray(getModelFieldName());
        }

        return modelFieldName;
    }

    public PropertyEntry getProperties()
    {
        return properties;
    }

    public ViewerFactory getFactory()
    {
        return factory;
    }


}
