//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.swing;

import com.nexirius.util.assertion.Assert;
import com.nexirius.util.resource.ClientResource;
import com.nexirius.util.resource.ResourceChangeEvent;

import javax.swing.*;

/**
 * A JOptionPane which responds to changes in client resources.
 * Changes can include tool tip text and icon.
 * <p/>
 * Property name           Component Property
 * -------------           ------------------
 * resourceKey.icon        icon
 * resourceKey.tip         tooltiptext
 * <p/>
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1999.03.03  MB              Created
 */
public class CFJOptionPane
        extends JOptionPane
        implements CFJItem {



    // The key used to look up specific resources
    private String resourceKey;

    /**
     * Construct a CFJOptionPane    (Client Framework JOptionPane)
     * Need a null constructor so we can use this class in a GUI builder
     */
    public CFJOptionPane() {
        super();
    }

    /**
     * Construct a CFJOptionPane   (Client Framework JOptionPane)
     *
     * @param resource    The resource to be used to establish basic attributes
     *                    of the JOptionPane
     * @param resourceKey The key to be used in establishing resources
     */
    public CFJOptionPane(ClientResource resource, String resourceKey) {
        super(resourceKey);

        Assert.pre(resource != null, "Parameter resource is not null");
        Assert.pre(resourceKey != null, "Parameter resourceKey is not null");

        this.resourceKey = resourceKey;

        update(resource);
        resource.addResourceChangeListener(this);
    }


    /**
     * Get the resource key
     *
     * @return the resource key
     */
    public String getResourceKey() {
        return this.resourceKey;
    }

    /**
     * Set the resource key
     *
     * @param resourceKey the resource key
     */
    public void setResourceKey(String resourceKey) {
        Assert.pre(resourceKey != null, "Parameter resourceKey is not null");
        this.resourceKey = resourceKey;
    }


    /**
     * @see import com.nexirius.util.resource.ResourceChangeListener;
     */
    public void resourceChange(ResourceChangeEvent event) {
        update(event.getClientResource());
    }


    /**
     * Update properties based on client resource
     *
     * @param clientResource the resource to be used to establish properties
     */
    public void update(ClientResource clientResource) {
        if (resourceKey == null) {
            return;
        }

        final Icon icon = clientResource.getIcon(resourceKey);
        if (icon != null) {
            setIcon(icon);
        }

        final String tip = clientResource.getToolTipText(resourceKey);
        if (tip != null) {
            setToolTipText(tip);
        }

        invalidate();
        validate();
        repaint();
    }
}
