//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.swing;

import com.nexirius.util.SortedVector;
import com.nexirius.util.assertion.Assert;
import com.nexirius.util.resource.ClientResource;
import com.nexirius.util.resource.ResourceChangeEvent;

import javax.swing.*;
import java.awt.*;

/**
 * A JTabbedPane which responds to changes in client resources.
 * Changes can include text tool tip.
 * <p/>
 * Property name           Component Property
 * -------------           ------------------
 * resourceKey             title
 * resourceKey.icon        icon
 * <p/>
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1999.01.16  MB              Created
 */
public class CFJTabbedPane
        extends JTabbedPane
        implements CFJItem {



    // The key used to look up specific resources
    private String resourceKey;

    // The vector to store Tab properties for JTabbedPane e.g. name, icon etc.
    protected PropertyListVector propertyList = new PropertyListVector();
    protected ClientResource resource;

    /**
     * Construct a CFJTabbedPane  (Client Framework JTabbedPane)
     *
     * @param resource The resource to be used to establish basic attributes
     *                 of the JTabbedPane
     */
    public CFJTabbedPane(ClientResource resource) {
        super();

        this.resource = resource;

        Assert.pre(resource != null, "Parameter resource is not null");
        update(resource);
        resource.addResourceChangeListener(this);
    }

    /**
     * Get the resource key
     *
     * @return the resource key
     */
    public String getResourceKey() {
        return this.resourceKey;
    }

    /**
     * Set the resource key
     *
     * @param resourceKey the resource key
     */
    public void setResourceKey(String resourceKey) {
        Assert.pre(resourceKey != null, "Parameter resourceKey is not null");
        this.resourceKey = resourceKey;
    }

    /**
     * @see import com.nexirius.util.resource.ResourceChangeListener;
     */
    public void resourceChange(ResourceChangeEvent event) {
        update(event.getClientResource());
    }


    /**
     * Add a Tab to CFJTabbedPane  (Client Framework JTabbedPane)
     *
     * @param title The text of the Resource
     * @param comp  The Resource for which text is added
     */
    public void addTab(String title, Component comp) {
        super.addTab(title, comp);
        propertyList.append(new TabProperty(title));
        update(resource);
    }

    /**
     * Add a Tab to CFJTabbedPane  (Client Framework JTabbedPane)
     *
     * @param title The text of the Resource
     * @param icon  The icon of the Resource
     * @param comp  The Resource for which text is added
     */
    public void addTab(String title, String icon, Component comp) {
        super.addTab(title, comp);
        propertyList.append(new TabProperty(title, icon));
        update(resource);
    }

    /**
     * Sets the icon for a Tab in CFJTabbedPane  (Client Framework JTabbedPane)
     *
     * @param index The index of the Tabbedpane resource vector
     * @param icon  The icon of the Resource
     */
    public void setIconAt(int index, String icon) {
        TabProperty property = (TabProperty) propertyList.getObject(index);
        property.icon = icon;
        propertyList.set(property, index);
        update(resource);
    }

    /**
     * Sets the title for a Tab in CFJTabbedPane  (Client Framework JTabbedPane)
     *
     * @param index The index of the Tabbedpane resource vector
     * @param title The title of the Resource
     */
    public void setTitleAt(int index, String title) {
        TabProperty property = (TabProperty) propertyList.getObject(index);
        property.name = title;
        propertyList.set(property, index);
        update(resource);
    }

    /**
     * Insert a Tab to CFJTabbedPane  (Client Framework JTabbedPane) vector at
     * a location
     *
     * @param title The text of the Resource
     * @param icon  The icon of the Resource
     * @param comp  The Resource for which text is added
     * @param index The index of the Tappedpane resource vector
     */
    public void insertTab(String title, String icon, Component comp, int index) {
        super.insertTab(title, resource.getIcon(icon), comp, null, index);
        propertyList.insert(new TabProperty(title, icon), index);
        update(resource);
    }

    /**
     * Remove a Tab from CFJTabbedPane  (Client Framework JTabbedPane)
     *
     * @param index The index of the Tappedpane resource vector
     */
    public void removeTabAt(int index) {
        super.removeTabAt(index);
        propertyList.removeElementAt(index);
    }

    /**
     * Remove all the tabs from CFJTabbedPane  (Client Framework JTabbedPane)
     */

    public void removeAll() {
        super.removeAll();
        propertyList.removeAllElements();
    }

    /**
     * What is the position of Tab in vector CFJTabbedPane (Client Framework JTabbedPane)?
     *
     * @param title The text of the Resource
     */
    public int indexOfTab(String title) {
        return propertyList.indexOfName(title);
    }

    /**
     * Update the CFJTabbedPane vector(Client Framework JTabbedPane) for the changes
     *
     * @param res The Resource
     */
    public void update(ClientResource res) {
        if (res == null) {
            // ignore missing resource information
            return;
        } else {
            this.resource = res;
        }

        if (propertyList.size() != getTabCount()) {
            throw new RuntimeException("propertyList.size() != getTabCount()");
        }

        TabProperty prop = propertyList.firstItem();

        for (int i = 0; i < getTabCount(); ++i) {
            if (prop.name != null) {
                try {
                    String s = resource.getLabel(prop.name);

                    super.setTitleAt(i, s);
                    super.setToolTipTextAt(i, resource.getToolTipText(prop.name));

                    if (prop.icon == null) {
                        Icon icon = resource.getIcon(prop.name);

                        if (icon != null) {
                            super.setIconAt(i, icon);
                        }                        
                    }

                } catch (Exception ex) {
                    throw new RuntimeException(ex);
                    // ignore
                    // super.setTitleAt(i, prop.name);
                }
            }
            if (prop.icon != null) {
                try {
                    Icon icon = resource.getIcon(prop.icon);

                    if (icon != null) {
                        super.setIconAt(i, icon);
                    }
                } catch (Exception ex) {
                    throw new RuntimeException(ex);
                    // ignore
                }
            }

            prop = propertyList.nextItem();
        } // for loop

        invalidate();
        validate();
        repaint();
    }


    /**
     * Properties of each tab in CFJTabbedPane  (Client Framework JTabbedPane)
     */
    class TabProperty {
        public String name;
        public String icon = null;

        public TabProperty(String name) {
            this.name = (name == null ? "" : name);
        }

        public TabProperty(String name, String icon) {
            this(name);
            this.icon = icon;
        }
    }

    /**
     * Maintain a Vector for CFJTabbedPane  (Client Framework JTabbedPane)
     * This internal class provides following features:
     * 1. firstItem()
     * 2. nextItem()
     * 3. append(TabProperty)
     * 4. insert(TabProperty, index)
     * 5. set(TabProperty, index)
     * 6. remove()
     * 7. indexOfName(String)
     */
    class PropertyListVector extends SortedVector {
        public TabProperty firstItem() {
            return (TabProperty) firstObject();
        }

        public TabProperty nextItem() {
            return (TabProperty) nextObject();
        }

        public void append(TabProperty prop) {
            super.addElement(prop);
        }

        public void insert(TabProperty prop, int index) {
            super.insertElementAt(prop, index);
        }

        public void set(TabProperty prop, int index) {
            super.setElementAt(prop, index);
        }

        public void removeTabAt(int index) {
            removeElementAt(index);
        }

        public int indexOfName(String title) {
            String name = (title == null ? "" : title);

            for (int i = 0; i < size(); ++i) {
                if (((TabProperty) elementAt(i)).name.equals(name)) {
                    return i;
                }
            }

            return -1;
        }

    }
}
