//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.swing;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.command.Command;
import com.nexirius.framework.command.Processor;
import com.nexirius.util.assertion.Assert;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

interface CommandClient {

    /**
     * Inner class for handling button clicks.
     * A button click corresponds to command execution
     */
    static class ActionHandler implements ActionListener {
        private Processor processor = null;
        private Command command;
        private JComponent component;

        ActionHandler(Command command) {
            Assert.pre(command != null, "Parameter command is not null");
            this.command = command;
        }

        ActionHandler(JComponent component, Command command, Processor processor) {
            Assert.pre(component != null, "Parameter component is not null");
            Assert.pre(command != null, "Parameter command is not null");
            Assert.pre(processor != null, "Parameter processor is not null");
            this.component = component;
            this.command = command;
            this.processor = processor;
        }

        /**
         * Action performed corresponds to command excution.
         * Note: Variables come from outer class.
         */
        @Override
        public void actionPerformed(ActionEvent event) {
            try
            {
                if (component != null)
                {
                    DialogManager.setWaitCursor(component);
                }
                if (processor != null)
                {
                    processor.execute(command, event);
                }
                else
                {
                    command.initialiseAction(event);
                    command.doAction();
                    command.finaliseAction();
                }
            }
            finally
            {
                if (component != null)
                {
                    DialogManager.setDefaultCursorLater(component);
                }
            }
        }
    }


    /**
     * Inner class for handling changes in command properties
     */
    static class CommandPropertyChangeHandler implements PropertyChangeListener, AncestorListener {
        private Command command;
        private JComponent component;

        CommandPropertyChangeHandler(Command command, JComponent component) {
            Assert.pre(command != null, "Parameter command is not null");
            Assert.pre(component != null, "Parameter component is not null");
            this.command = command;
            this.component = component;

            /**
             * avoid garbage collector to remove this (connect its lifecycle to the actual visual component)
             */
            component.addAncestorListener(this);
        }


        @Override
        public void propertyChange(PropertyChangeEvent event) {
            if (event.getSource() != this.command) {
                return;
            }

            // Property "enabled" corresponds to the "enabled" value of
            // the button.
            if (event.getPropertyName().equals(Command.ENABLED)) {
                Object o = event.getNewValue();
                Assert.assertion(o instanceof Boolean, "Property is a boolean");

                boolean enabled = ((Boolean) o).booleanValue();
/*                if (enabled == false && component.isEnabled()) {
                    // If the component has the focus and is about to be disabled then move the focus away!
                    component.transferFocusBackward();
                }*/
                this.component.setEnabled(enabled);
            } else if (event.getPropertyName().equals(Command.VISIBLE)) {
                Object o = event.getNewValue();
                Assert.assertion(o instanceof Boolean, "Property is a boolean");

                boolean visible = ((Boolean) o).booleanValue();
                this.component.setVisible(visible);
            } else if (event.getPropertyName().equals(Command.RESOURCE_KEY)) {
                Object o = event.getNewValue();
                Assert.assertion(o instanceof String, "Property is a String");

                if (component instanceof CommandClient) {
                    ((CommandClient) component).changeResourceKey((String) o);
                }
            }
        }

        /**
         * AncestorListener implementation (empty functions)
         */
        @Override
        public void ancestorAdded(AncestorEvent event) {
        }

        @Override
        public void ancestorMoved(AncestorEvent event) {
        }

        @Override
        public void ancestorRemoved(AncestorEvent event) {
        }
    }


    /**
     * Return the command executed whth this button is clicked
     *
     * @return the command
     */
    public Command getCommand();


    /**
     * Return the processor which executes the command when this button is
     * clicked
     *
     * @return the processor (may be null)
     */
    public Processor getProcessor();


    /**
     * Set the command executed whth this button is clicked
     */
    public void setCommand(Command command);


    /**
     * Set the processor which executes the command when this button is
     * clicked
     *
     * @param processor the processor (may be null)
     */
    public void setProcessor(Processor processor);

    public void changeResourceKey(String resourceKey);
}
