//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.swing;

import com.nexirius.framework.command.Command;
import com.nexirius.framework.command.Processor;
import com.nexirius.util.assertion.Assert;
import com.nexirius.util.resource.ClientResource;

/**
 * This class extends CFJMenuItem and invokes a Command when the menu status
 * is clicked. The enabled/disabled status of the command is also automatically
 * reflected in the menu status status.
 * <pp>
 *
 * @author MB
 *         <p/>
 *         Date        Chng.Rq.   Author    Changes/Enhancements
 *         1999.02.09             MB       Created
 */
public class CommandJMenuItem extends CFJMenuItem implements CommandClient {



    // The command to be executed if the button is clicked
    private Command command;

    // The processor on which to execute the command if the button is clicked.
    // Note: May be null. If it is, then the command will be executed directly
    // on the event dispatch thread. Another consequence of not using a
    // processor is that undo/redo cannot be performed.
    private Processor processor = null;
    private CommandClient.CommandPropertyChangeHandler listener;


    /**
     * Construct a CommandJMenuItem
     *
     * @param resource to be used to obtain JMenuItem properties
     * @param command  the command to be executed when the button is clicked.
     *                 The command name will be used as a resource key.
     */
    public CommandJMenuItem(ClientResource resource, Command command) {
        // Use command's name as a default resource key
        this(resource, command.getCommandName(), command);
    }


    /**
     * Construct a CommandJMenuItem
     *
     * @param resource    to be used to obtain JMenuItem properties
     * @param resourceKey key to be used with clienr resource
     * @param command     the command to be executed when the button is clicked
     */
    public CommandJMenuItem(ClientResource resource, String resourceKey,
                            Command command) {
        // Use command's name as a default resource key
        super(resource, resourceKey);

        Assert.pre(command != null, "Parameter command is not null");

        this.command = command;

        // Get an action handler to handle the action events
        addActionListener(new CommandClient.ActionHandler(command));

        // Listen to the command's property change events for the purpose
        // of modifying menu-status's appearance (e.g. enabled/disabled)
        listener = new CommandClient.CommandPropertyChangeHandler(command, this);
        this.command.addPropertyChangeListener(listener);

        // Set initial state of the menu status to reflect that of the command
        setEnabled(command.isEnabled());
    }


    /**
     * Construct a CommandJMenuItem
     *
     * @param resource  to be used to obtain JMenuItem properties
     * @param command   the command to be executed when the button is clicked
     *                  The command name will be used as a resource key.
     * @param processor the processor on which to execute the command
     */
    public CommandJMenuItem(ClientResource resource, Command command,
                            Processor processor) {
        this(resource, command.getResourceKey(), command, processor);
    }


    /**
     * Construct a CommandJMenuItem
     *
     * @param resource    to be used to obtain JMenuItem properties
     * @param resourceKey resource key to use with client resource
     * @param command     the command to be executed when the button is clicked
     * @param processor   the processor on which to execute the command
     */
    public CommandJMenuItem(ClientResource resource, String resourceKey,
                            Command command, Processor processor) {
        this(resource, resourceKey, command);

        Assert.pre(processor != null, "Parameter processor is not null");

        this.processor = processor;
    }


    /**
     * Return the command executed whth this button is clicked
     *
     * @return the command
     */
    public Command getCommand() {
        return this.command;
    }


    /**
     * Return the processor which executes the command when this button is
     * clicked
     *
     * @return the processor (may be null)
     */
    public Processor getProcessor() {
        return this.processor;
    }


    /**
     * Set the command executed whth this button is clicked
     */
    public void setCommand(Command command) {
        Assert.pre(command != null, "Parameter command is not null");

        this.command = command;

        Assert.post(this.command == command, "command is set");
    }


    /**
     * Set the processor which executes the command when this button is
     * clicked
     *
     * @param processor the processor (may be null)
     */
    public void setProcessor(Processor processor) {
        this.processor = processor;
    }

    public void changeResourceKey(String resourceKey) {
        setResourceKey(resourceKey);
        update((ClientResource) null);
    }
}
