//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public class ProgressListenerMulticaster implements ProgressListener {



    protected final ProgressListener a, b;

    /**
     * Creates an event multicaster instance which chains listener-a
     * with listener-b.
     *
     * @param a listener-a
     * @param b listener-b
     */

    public void progress(String description, int total, int completed, int toGo) {
        a.progress(description, total, completed, toGo);
        b.progress(description, total, completed, toGo);
    }

    protected ProgressListenerMulticaster(ProgressListener a, ProgressListener b) {
        this.a = a;
        this.b = b;
    }

    /**
     * Removes a listener from this multicaster and returns the
     * resulting multicast listener.
     *
     * @param oldl the listener to be removed
     */
    protected ProgressListener remove(ProgressListener oldl) {
        if (oldl == a) return b;
        if (oldl == b) return a;
        ProgressListener a2 = removeInternal(a, oldl);
        ProgressListener b2 = removeInternal(b, oldl);
        if (a2 == a && b2 == b) {
            return this;	// it's not here
        }
        return addInternal(a2, b2);
    }

    /**
     * Returns the resulting multicast listener after removing the
     * old listener from listener-l.
     * If listener-l equals the old listener OR listener-l is null,
     * returns null.
     * Else if listener-l is an instance of AWTEventMulticaster,
     * then it removes the old listener from it.
     * Else, returns listener l.
     *
     * @param l    the listener being removed from
     * @param oldl the listener being removed
     */
    protected static ProgressListener removeInternal(ProgressListener l, ProgressListener oldl) {
        if (l == oldl || l == null) {
            return null;
        } else if (l instanceof ProgressListenerMulticaster) {
            return ((ProgressListenerMulticaster) l).remove(oldl);
        } else {
            return l;		// it's not here
        }
    }


    /**
     * Adds Progress-listener-a with Progress-listener-b and
     * returns the resulting multicast listener.
     *
     * @param a Progress-listener-a
     * @param b Progress-listener-b
     */
    public static ProgressListener add(ProgressListener a, ProgressListener b) {
        return (ProgressListener) addInternal(a, b);
    }


    /**
     * Removes the old Progress-listener from Progress-listener-l and
     * returns the resulting multicast listener.
     *
     * @param l    Progress-listener-l
     * @param oldl the Progress-listener being removed
     */
    public static ProgressListener remove(ProgressListener l, ProgressListener oldl) {
        return (ProgressListener) removeInternal(l, oldl);
    }


    /**
     * Returns the resulting multicast listener from adding listener-a
     * and listener-b together.
     * If listener-a is null, it returns listener-b;
     * If listener-b is null, it returns listener-a
     * If neither are null, then it creates and returns
     * a new ProgressListenerMulticaster instance which chains a with b.
     *
     * @param a event listener-a
     * @param b event listener-b
     */
    protected static ProgressListener addInternal(ProgressListener a, ProgressListener b) {
        if (a == null) return b;
        if (b == null) return a;
        return new ProgressListenerMulticaster(a, b);
    }


    /* Serialization support.  
     */

    protected void saveInternal(ObjectOutputStream s, String k) throws IOException {
        if (a instanceof ProgressListenerMulticaster) {
            ((ProgressListenerMulticaster) a).saveInternal(s, k);
        } else if (a instanceof Serializable) {
            s.writeObject(k);
            s.writeObject(a);
        }

        if (b instanceof ProgressListenerMulticaster) {
            ((ProgressListenerMulticaster) b).saveInternal(s, k);
        } else if (b instanceof Serializable) {
            s.writeObject(k);
            s.writeObject(b);
        }
    }

    protected static void save(ObjectOutputStream s, String k, ProgressListener l) throws IOException {
        if (l == null) {
            return;
        } else if (l instanceof ProgressListenerMulticaster) {
            ((ProgressListenerMulticaster) l).saveInternal(s, k);
        } else if (l instanceof Serializable) {
            s.writeObject(k);
            s.writeObject(l);
        }
    }
}

