//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.util.Date;
import java.util.Random;

/**
 * This class is used to synchronise process execution
 * <pre>
 * <p/>
 * File sharedFile = new File(filename);
 * SemaphoreFile semaphore = new SeamphoreFile(filename + ".lock");
 * <p/>
 * //wait for semaphore max 5000 milliseconds
 * if (semaphore.waitFor(5000)) {
 * 	  throw new Exception("Timeout while waiting for: " + filename");
 * }
 * <p/>
 * try {
 * // open shared file...
 * // work with shared file...
 * // close shared file...
 * } catch (IOException ex) {
 *    semaphore.release(); // do not forget to release even if exception occured
 * 	  throw ex;
 * }
 * <p/>
 * semaphore.release();
 * </pre>
 */
public class SemaphoreFile extends Object {



    static String fileString;
    XFile file;

    /**
     * create a semaphore to synchronize access to a shared resource (amongst multiple processes)
     *
     * @param fileName the name of the lock file
     */
    public SemaphoreFile(String fileName) {
        file = new XFile(fileName);
        init();
    }

    private void init() {
        if (fileString == null) {
            fileString = System.getProperty("user.name", "unknown user") + " " + new Random().nextInt();
        }
    }

    /**
     * access the current reservation string (if available)
     *
     * @return the string which is stored in the lock file (or "")
     */
    public String getReservation() {
        String ret = "";

        if (file.exists()) {
            try {
                ret = new String(file.getBytes());
            } catch (Exception ex) {
                // ignore
            }
        }

        return ret;
    }

    /**
     * check whether lock file exists and if it does exist check whether it belongs to myself
     *
     * @return false if lockfile does not exist or if lock file belongs to myself
     */
    public synchronized boolean reserved() {
        // check whether the file exists
        if (file.exists()) {
            // check whether the file belongs to me
            if (fileString.equals(getReservation())) {

                return false;
            }

            return true;
        }

        return false;
    }

    /**
     * try to reserve the lock file
     *
     * @throws Exception if lock is reseved by some other process or if lock can't be created
     */
    public synchronized void reserve()
            throws Exception {
        if (reserved()) {
            throw new Exception("Can't reserve semaphore file " + file.getPath() + " reserved by " + getReservation());
        }
        // do reserve
        try {
            file.writeText(fileString);
        } catch (Exception ex) {
            throw new RuntimeException("Can't reserve semaphore file " + file.getPath() + " " + ex);
        }

        if (reserved()) {
            throw new Exception("Can't reserve semaphore file " + file.getPath() + " reserved by " + getReservation());
        }
    }

    /**
     * deletes the lock file
     *
     * @throws Exception
     */
    public synchronized void release()
            throws Exception {
        // do release
        if (!file.exists()) {
            throw new Exception("Can't release semafore file " + file.getPath() + " (it does not exist)");
        }

        if (reserved()) {
            throw new Exception("Can't release semafore file which is not mine " + file.getPath() + " reserved by " + getReservation());
        }

        file.delete();
    }

    /**
     * Tries to create a lock file (waits forever if the lock file exists and belongs to
     * another process)
     *
     * @throws Exception if reservation fails
     */
    public synchronized void waitFor()
            throws Exception {
        // wait polling
        while (reserved()) {
            try {
                Thread.currentThread().wait(300);
            } catch (Exception ex) {

            }
        }

        reserve();
    }

    /**
     * Tries to create a lock file (waits maximum timeoutMillis if the lock file exists and belongs to
     * another process)
     *
     * @return true if reservation has failed due to elapsed timeout
     * @throws Exception if reservation fails
     */
    public synchronized boolean waitFor(int timeoutMillis)
            throws Exception {
        long time = new Date().getTime();

        // wait polling
        while (reserved()) {
            if (new Date().getTime() + 300 - time > timeoutMillis) {

                return true;
            }

            try {
                Thread.currentThread().wait(300);

            } catch (Exception ex) {

            }
        }

        reserve();

        return false;
    }
}
