//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.util.ArrayList;
import java.util.Iterator;

public class StringFilter implements IStringFilter {
    ArrayList filterPartList = null;

    public StringFilter() {
    }

    public StringFilter(String value) {
        try {
            setPattern(value);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    public void setPattern(String text) throws Exception {
        filterPartList = null;

        if (text == null || text.trim().length() == 0) {
            return;
        }

        parse(text);
    }

    public String getPattern() {
        if (filterPartList == null) {
            return "\"*\"";
        }

        StringBuffer result = new StringBuffer();

        boolean first = true;

        for (Iterator it = filterPartList.iterator(); it.hasNext();) {
            FilterPart filterPart = (FilterPart) it.next();

            filterPart.appendText(result, !first);
            first = false;
        }

        return result.toString();
    }

    private void parse(String text) {
        int state = 0;
        filterPartList = new ArrayList();

        for (int i = 0; i < text.length(); ++i) {
            char c = text.charAt(i);

            switch (state) {
                case 0: // expecting space or '+' or '-' or expression
                    if (c == '+' || c == '-') {
                        i = parseExpression(text, i + 1, c);
                        state = 1;
                    } else if (Character.isSpaceChar(c)) {
                        // ignore
                    } else {
                        i = parseExpression(text, i, '+');
                        state = 1;
                    }
                    break;

                case 1: // space or '+'  or '-'
                    if (c == '+' || c == '-') {
                        i = parseExpression(text, i + 1, c);
                        state = 1;
                    } else if (Character.isSpaceChar(c)) {
                        // ignore
                    }
                    break;

                default:
                    break;
            }
        }
    }

    private int parseExpression(String text, int index, char sign) {
        int state = 0;
        boolean done = false;
        StringBuffer expr = new StringBuffer();

        for (int i = index; i < text.length(); ++i) {
            char c = text.charAt(i);

            switch (state) {
                case 0: // expecting space or '"'
                    if (c == '"') {
                        state = 1;
                    } else if (Character.isSpaceChar(c)) {
                        // ignore
                    } else {
                        expr.append(c);
                        state = 3;
                    }
                    break;

                case 1: // inside string literal
                    if (c == '"') {
                        index = i;
                        done = true;
                    } else if (c == '\\') {
                        state = 2;
                    } else {
                        expr.append(c);
                    }
                    break;

                case 2:
                    expr.append(c);
                    state = 1;
                    break;

                case 3:
                    if (c == '+' || c == '-' || Character.isSpaceChar(c)) {
                        index = i - 1;
                        done = true;
                    } else {
                        expr.append(c);
                        break;
                    }
            }

            if (done) {
                break;
            }
            index = i;
        }

        filterPartList.add(new FilterPart(sign, expr.toString()));

        return index;
    }

    public boolean  match(String argument) {
        return match(argument, true);
    }

    public boolean  match(String argument, boolean ignoreCase) {
        if (filterPartList == null) {
            return true;
        }

        boolean fit = true;
        boolean first = true;

        for (Iterator it = filterPartList.iterator(); it.hasNext();) {
            FilterPart filterPart = (FilterPart) it.next();

            if (first) {
                if (filterPart.sign == '+') {
                    fit = false;
                }
                first = false;
            }

            if (!fit && filterPart.sign == '+') {
                if (filterPart.match(argument, ignoreCase)) {
                    fit = true;
                }
            }

            if (fit && filterPart.sign == '-') {
                if (filterPart.match(argument, ignoreCase)) {
                    fit = false;
                }
            }
        }

        return fit;
    }

    public static boolean match(String pattern, String s) {
        return new StringFilter(pattern).match(s);
    }

    private static class FilterPart {
        public char sign;
        private String filter;
        private String upper;

        public FilterPart(char sign, String filter) {
            this.sign = sign;
            this.filter = filter;
            this.upper = filter.toUpperCase();
        }

        public void appendText(StringBuffer buf, boolean appendSign) {
            if (sign == '-' || appendSign) {
                buf.append(sign);
                buf.append(' ');
            }
            buf.append(TextToken.quoteString(filter));
            buf.append(' ');
        }

        public boolean match(String argument, boolean ignoreCase)
        {
            if (ignoreCase) {
                return XString.match(upper, argument.toUpperCase());
            }

            return XString.match(filter, argument);
        }
    }

    public static void main(String argv[]) {
        String value = "Marcel Baumann";

        String[] patterns = new String[]{
            "   ",
            "M*",
            "Ma* + xxx*",
            "Ma* - xxx*",
            "Ma* - *Ba*",
            "xxx* + Ma*",
            "Ma* - *Ba* + *nn",
            "- *nn",
            "- *xxx",
            "- *nn + M*",
        };

        for (int i = 0; i < patterns.length; i++) {
            String pattern = patterns[i];

            StringFilter filter = new StringFilter(pattern);
            System.out.println(pattern + " [" + filter.getPattern() + "] ->" + filter.match(value));
        }
    }
}
