//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.*;
import java.util.Vector;

/**
 * a class that helps reading and writing text files. The
 * text files all have a specific file identifier and a file
 * format version number (eg FILE_ID:"source file" VERSION:1)
 */

public class TextFile extends File {



    String m_file_id;		// a string to identify the file
    int m_format_version;	// a number that identifies the format version
    PushbackInputStream in;
    OutputStream out;
    int m_act_line;			// the line is incrememnted each time a \n character appears when reading
    int m_act_byte;
    boolean m_ignore_tab = true;
    boolean m_ignore_new_line = true;
    boolean m_ignore_form_feed = true;
    TextToken m_pushback_token;
    TextToken m_last_token;

    static String id_string = new String("FILE_ID");
    static String version_string = new String("VERSION");

    /**
     * Creates a File instance that represents the file whose pathname is
     * the given path argument.
     */
    public TextFile(String path) {
        super(path);
    }

    /**
     * Only for method openTokenString
     */
    public TextFile() {
        super(" ");
    }

    /**
     * Creates a File instance whose pathname is the pathname of the specified
     * directory, followed by the separator character, followed by the
     * name argument.
     */
    public TextFile(File dir, String name) {
        super(dir, name);
    }

    /**
     * Creates a File instance that represents the file with the specified
     * name in the specified directory.
     */
    public TextFile(String path, String name) {
        super(path, name);
    }

    /**
     * closes the file (this function is automatically called if all the contents
     * of a file a read)
     */
    public void close()
            throws IOException {
        if (in != null) {
            try {
                in.close();
            } catch (IOException ex) {
                // ignore because stream could already be closed
            }
        }

        if (out != null) {
            try {
                out.close();
            } catch (IOException ex) {
                // ignore because stream could already be closed
            }
        }

        in = null;
        out = null;
    }

    /**
     * each text file needs an individual file identifier and a file format version.
     * If the program needs to write the same information in a new format, the format_version
     * must be incremented. The read function can then distinguish between different file
     * formats by reading the format_version number.
     */
    public void setFileId(String file_id, int format_version)
            throws TextFileException {
        if (file_id != null) {

            if (file_id.length() != 0) {
                m_file_id = file_id;
                m_format_version = format_version;
            } else {
                throw new TextFileException(1);
            }
        } else {
            throw new TextFileException(2);
        }
    }

    /**
     * Creates and opens the file only if it does not exist already. The file_id and
     * format_version are written into the new file.
     */
    public void openNewFileForWrite(String file_id, int format_version)
            throws IOException, TextFileException {
        if (in != null || out != null) {
            throw new TextFileException(3);
        }

        if (exists()) {
            throw new TextFileException(4);
        }

        setFileId(file_id, format_version);

        out = createOutputStream(this);

        writeHeader();
    }

    public OutputStream createOutputStream(File f)
            throws IOException {
        return new BufferedOutputStream(new FileOutputStream(f));
    }

    public PushbackInputStream createPushbackInputStream(File f)
            throws IOException {
        XFile xfile = new XFile(f.getPath());

        try {
            return new PushbackInputStream(xfile.getBufferedInputStream());
        } catch (final Exception ex) {
            if (ex instanceof IOException) {
                throw (IOException) ex;
            }
            throw new IOException(ex.getMessage()) {
                @Override
                public Throwable getCause() {
                    return ex;
                }
            };
        }
    }

    /**
     * Checks if the file id of an existing file is the same. If so the file
     * will be opened. The original contents are lost. The file_id and format_version
     * are written into the file.
     */
    public void openExistingFileForWrite(String file_id, int format_version)
            throws IOException, TextFileException {
        if (in != null || out != null) {
            throw new TextFileException(3);
        }

        if (!exists()) {
            throw new TextFileException(5);
        }

        setFileId(file_id, format_version);

        out = createOutputStream(this);

        writeHeader();
    }

    /**
     * Returns the file version number
     */
    public int getFileVersion() {
        return m_format_version;
    }

    /**
     * If the file was already opened (for read) this function just returns
     * the file_id which was read.
     * Otherwise this function reads the file header and closes the file right after that.
     *
     * @return the file id string
     */
    public String getFileId()
            throws IOException, TextFileException {
        if (m_file_id == null) {
            readHeader();
            close();
        }

        return m_file_id;
    }

    public void openTokenString(String s) {
        in = new PushbackInputStream(new ByteArrayInputStream(s.getBytes()));
    }

    /**
     * The file is opened and the file id and version are read.
     * Checks the file_id for equality (else throws TextFileException).
     * If a null file_id is given the id check is skipped
     */
    public void openForRead(String file_id)
            throws IOException, TextFileException {
        if (in != null || out != null) {
            throw new TextFileException(3);
        }
        // create in
        try {
            in = createPushbackInputStream(this);
        } catch (FileNotFoundException ex) {
            throw new TextFileException(6);
        }

        try {
            readHeader();
        } catch (TextFileException ex) {
            close();

            throw ex;
        }

        if (file_id != null && !m_file_id.equals(file_id)) {
            close();

            throw new TextFileException(7);
        }
    }

    /**
     * @return the next token if the file was opened for read
     */
    public TextToken nextToken()
            throws IOException, TextFileException {
        if (in != null) {
            if (m_pushback_token != null) {
                m_last_token = m_pushback_token;
                m_pushback_token = null;
            } else {
                try {
                    // remove all spaces
                    skipSpaces();

                    m_last_token = TextToken.nextToken(in);
                } catch (final IOException ex) {
                    close();
                    throw new IOException("On line " + m_act_line + ": " + ex.getMessage()) {
                        @Override
                        public Throwable getCause() {
                            return ex;
                        }
                    };
                }

                if (m_last_token != null) {
                    m_act_byte += m_last_token.getBytes();
                    m_act_line += m_last_token.getLines();
                }
            }

            if (m_last_token == null) {
                close();
            }

            return m_last_token;
        }

        throw new TextFileException(9);
    }

    /**
     * @return the actual line (only if the file is exclusively read with the nextToken() function
     */
    public int getActLine() {
        return m_act_line;
    }

    /**
     * @return the actual file position (only if the file is exclusively read with the nextToken() function
     */
    public int getActByte() {
        return m_act_byte;
    }

    /**
     * @return the read progress in percent
     */
    public double getProgress() {
        if (length() > 0) {
            return 100.0 * (double) getActByte() / (double) length();
        }

        return 0.0;
    }

    /**
     * reads all characters that are considered to be spaces
     *
     * @return the number of space characters skipped
     */
    public int skipSpaces()
            throws IOException {
        int ret = 0;

        if (in != null) {
            boolean do_break = false;
            int c = -1;

            while (!do_break) {
                c = in.read();
                ++m_act_byte;

                switch (c) {
                    case ' ':
                    case '\r': // carriage returns are considered to be spaces
                        continue;

                    case '\t':
                        if (m_ignore_tab) {
                            continue;
                        } else {
                            do_break = true;
                        }
                        break;

                    case '\n':
                        ++m_act_line;

                        if (m_ignore_new_line) {
                            continue;
                        } else {
                            do_break = true;
                        }
                        break;

                    case '\f':
                        if (m_ignore_form_feed) {
                            continue;
                        } else {
                            do_break = true;
                        }
                        break;

                    case -1:
                        do_break = true;
                        break;

                    default:
                        do_break = true;
                }

                if (!do_break) {
                    ++ret;
                }
            }

            if (c != -1) {
                in.unread(c);
                --m_act_byte;
            }
        }

        return ret;
    }

    /**
     * puts back the last token. This token will be returned by the next call to
     * nextToken()
     */
    public void pushbackToken() {
        m_pushback_token = m_last_token;
    }

    /**
     * If the file was nor opened, then this function opens it and reads all the tokens.
     *
     * @return all the tokens of the file
     */
    public Vector readTokens()
            throws IOException, TextFileException {
        Vector v = new Vector();
        TextToken t = nextToken();

        while (t != null) {
            v.addElement(t);
            t = nextToken();
        }

        close();

        return v;
    }

    public StringVector readTokenAsStrings()
            throws IOException, TextFileException {
        StringVector v = new StringVector();
        TextToken t = nextToken();

        while (t != null) {
            v.addElement(t.toString());
            t = nextToken();
        }

        close();

        return v;
    }

    /**
     * writes a TextToken to the file (the file must be open for write)
     */
    public void write(TextToken token)
            throws IOException {
        if (out != null) {
            String s = token.toString();

            write(s);
        } else
            throw new TextFileException(15);
    }

    /**
     * writes a new line character (\n) to the file (the file must be open for write)
     */
    public void writeNewLine()
            throws IOException {
        if (out != null) {
            out.write((int) '\n');
        } else
            throw new TextFileException(15);
    }

    /**
     * writes a tab character (\t) to the file (the file must be open for write)
     */
    public void writeTab()
            throws IOException {
        if (out != null) {
            out.write((int) '\t');
        } else
            throw new TextFileException(15);
    }

    public void write(String s)
            throws IOException {
        out.write(s.getBytes());
    }

    /**
     * writes a form feed character (\f) to the file (the file must be open for write)
     */
    public void writeFormFeed()
            throws IOException {
        if (out != null) {
            out.write((int) '\f');
        } else
            throw new TextFileException(15);
    }

    /**
     * writes an identifier to the file (the file must be open for write)
     * the string must be a legal Java identifier (no spaces or leading digits)
     */
    public void writeIdentifier(String identifier)
            throws IOException {
        if (out != null) {
            write(identifier);
        } else
            throw new TextFileException(15);
    }

    /**
     * writes a string to the file (the file must be open for write)
     * there are no restrictions to the string argument. Unicode characters
     * are encoded as \ uXXXX strings where XXXX is a hex number. The string
     * is pasted into the file between double quote characters.
     * \ characters and " are replaced by \\ or \ " respectively.
     */
    public void writeString(String s)
            throws IOException {
        write(new TextToken(s, TextToken.STRING));
    }

    /**
     * writes the character into the file  (the file must be open for write)
     */
    public void write(char c)
            throws IOException {
        if (out != null) {
            out.write((int) c);
        } else
            throw new TextFileException(15);
    }

    /**
     * writes the integer into the file  (the file must be open for write)
     */
    public void write(int i)
            throws IOException {
        write(new TextToken(i));
    }

    /**
     * writes the long integer into the file (the file must be open for write).
     * This function appends a L to the number.
     */
    public void write(long l)
            throws IOException {
        write(new TextToken(l));
    }

    /**
     * writes the double into the file  (the file must be open for write)
     */
    public void write(double d)
            throws IOException {
        write(new TextToken(d));
    }

    /**
     * This function may be used to write information in a free format to the file
     * (only the part after the header)
     */
    public OutputStream getOutputStream() {
        return out;
    }

    /**
     * This function may be used to read the file (only the part after the header)
     */
    public PushbackInputStream getInputStream() {
        return in;
    }

    /**
     * The space characters. By default all three characters are considered to be spaces (true, true, true)
     */
    public void setIgnore(boolean tab, boolean new_line, boolean form_feed) {
        m_ignore_tab = tab;
        m_ignore_new_line = new_line;
        m_ignore_form_feed = form_feed;
    }

    /**
     * reads the file header
     */
    private void readHeader()
            throws IOException, TextFileException {
        TextToken t;
        int i = 0;

        m_act_line = 1;
        m_act_byte = 0;

        for (t = nextToken(); t != null; t = nextToken()) {
            if (t.isSpace()) {
                continue;
            }

            switch (i) {
                case 0:
                    if (!t.isIdentifier(id_string)) {
                        throw new TextFileException(10);
                    }
                    break;

                case 1:
                case 4:
                    if (!t.isChar(':')) {
                        throw new TextFileException(11);
                    }
                    break;

                case 2:
                    if (!t.isString()) {
                        throw new TextFileException(12);
                    } else {
                        m_file_id = t.getString();
                    }
                    break;

                case 3:
                    if (!t.isIdentifier(version_string)) {
                        throw new TextFileException(13);
                    }
                    break;

                case 5:
                    if (!t.isInt()) {
                        throw new TextFileException(13);
                    } else {
                        m_format_version = t.getInt();
                    }
                    break;
            }

            if (i == 5) {
                int c = in.read();	// read the pending new line character
                ++m_act_byte;

                if (c == '\n') {
                    ++m_act_line;
                }

                return;
            }

            ++i;
        }

        throw new TextFileException(8);
    }

    /**
     * writes the file header
     */
    private void writeHeader()
            throws IOException, TextFileException {
        if (out != null) {
            writeIdentifier(id_string);
            write(':');
            writeString(m_file_id);
            writeNewLine();
            writeIdentifier(version_string);
            write(':');
            write(m_format_version);
            writeNewLine();

            return;
        }

        throw new TextFileException(14);
    }
}
