//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

public class TextTokenReader extends Object {
    Reader reader;
    int unreadCharacter = -1;
    int lastCharacter = -1;
    int numberChars = 0;
    int numberLines = 0;

    public TextTokenReader(Reader reader) {
        this.reader = reader;
    }

    /**
     * @return the number of bytes that where read to produce the last token read
     */
    public int getBytes() {
        return numberChars;
    }

    /**
     * @return the number of lines that where read to produce the last token read
     */
    public int getLines() {
        return numberLines;
    }

    /**
     * Get the next token of the specified stream
     *
     * @return the next token or null if the last token has been returned already (or on error)
     */
    public TextToken nextToken() throws IOException {
        TextToken ret = null;

        for (ret = nextTokenIncludingSpaces(); ret != null; ret = nextTokenIncludingSpaces()) {
            if (!ret.isSpace()) {
                break;
            }
        }

        return ret;
    }

    private int read() throws IOException {
        if (unreadCharacter != -1) {
            lastCharacter = unreadCharacter;
            unreadCharacter = -1;
        } else {
            lastCharacter = reader.read();
        }

        if (lastCharacter >= 0) {
            ++numberChars;
        }

        if (lastCharacter == '\n') {
            ++numberLines;
        }

        return lastCharacter;
    }

    private void unread() {
        --numberChars;
        unreadCharacter = lastCharacter;
    }

    public void skipLine() throws IOException {
        int c;

        while (true) {
            c = read();

            switch (c) {
                case -1:
                    return;

                case '\n':
                    return;
            }
        }
    }

    public TextToken nextTokenIncludingSpaces() throws IOException {
        int c;

        c = read();

        switch (c) {
            case -1:
                return null;

            case '\"':
                return readString();

            case '\t':
                return new TextToken(null, TextToken.TAB);

            case '\n':
                return new TextToken(null, TextToken.NEW_LINE);

            case '\f':
                return new TextToken(null, TextToken.FORM_FEED);

            default:
                break;
        }

        if (Character.isDigit((char) c) || c == '-') {

            return readNumber(c);
        }

        if (Character.isJavaIdentifierStart((char) c)) {
            return readIdentifier(c);
        }

        return new TextToken((char) c);
    }

    /**
     * This function scans the given string and returns a vector containing all the tokens
     * which are in the string.
     */
    public static TextToken[] getTokensFrom(String s) {
        if (s == null) {
            return new TextToken[0];
        }

        Vector vect = new Vector();
        Reader reader = new StringReader(s);
        TextTokenReader textTokenReader = new TextTokenReader(reader);
        TextToken t;

        while (true) {
            try {
                t = textTokenReader.nextToken();

                if (t == null) {
                    break;
                }

                vect.addElement(t);
            } catch (Exception ex) {
                break;
            }
        }

        if (vect.size() == 0) {
            return new TextToken[0];
        }

        TextToken ret[] = new TextToken[vect.size()];
        Enumeration e = vect.elements();
        int i = 0;

        while (e.hasMoreElements()) {
            ret[i++] = (TextToken) e.nextElement();
        }

        return ret;
    }

    private TextToken readString() throws IOException {
        int c;
        int state = 0;

        StringBuffer buf = new StringBuffer();
        StringBuffer unicode = new StringBuffer("0000");

        while (true) {
            c = read();

            if (c == -1) {
                throw new IOException("Unexpected end of file inside a string constant");
            }

            switch (state) {
                case 0: // no escape
                    switch (c) {
                        case '\\':
                            state = 1;
                            break;

                        case '\"':
                            state = 6; // done
                            break;

                        case '\n':
                            // FALLTHROUGHT
                        default:
                            buf.append((char) c);
                    }
                    break;

                case 1: // after escape
                    switch (c) {
                        case '\\':
                            state = 0;
                            buf.append('\\');
                            break;

                        case '\"':
                            state = 0;
                            buf.append('\"');
                            break;

                        case 'u':
                            state = 2;
                            break;

                        default:
                            state = 0;
                            buf.append(c); // unexpected escape character
                    }
                    break;

                case 2: // first unicode character
                case 3: // second unicode character
                case 4: // third unicode character
                case 5: // forth unicode character
                    unicode.setCharAt(state - 2, (char) c);
                    if (state < 5) {
                        ++state;
                    } else {
                        // append unicode character

                        try {
                            buf.append((char) Integer.parseInt(new String(unicode), 16));
                        } catch (Exception ex) {
                            buf.append("\\u" + unicode);
                        }
                        state = 0;
                    }

                    break;
            }

            if (state == 6) {

                break;
            }
        }

        return new TextToken(new String(buf), TextToken.STRING);
    }

    private TextToken readNumber(int first_char) throws IOException {
        boolean is_double = false;
        boolean is_long = false;
        int state = 0;
        int c = first_char;
        StringBuffer number = new StringBuffer();

        while (state != -1) {

            switch (state) {
                case 0: // beginning
                    switch (c) {
                        case '-':
                            number.append('-');
                            break;

                        case '.':
                            state = 2; // after comma
                            number.append('.');
                            break;

                        default:
                            if (Character.isDigit((char) c)) {
                                number.append((char) c);
                                state = 1;
                            } else {
                                unread();
                                return new TextToken((char) first_char);
                            }
                    }
                    break;

                case 1: // after first number
                    if (Character.isDigit((char) c)) {
                        number.append((char) c);
                    } else if (c == '.') {
                        number.append('.');
                        state = 2; // after comma
                    } else if (c == 'l' || c == 'L') {
                        is_long = true;
                        state = -1;
                    } else {
                        unread();
                        state = -1;
                    }
                    break;

                case 2: // after comma
                    is_double = true;

                    if (Character.isDigit((char) c)) {
                        number.append((char) c);
                    } else {
                        unread();
                        state = -1;
                    }
                    break;
            }

            if (state != -1) {
                c = read();

                if (c == -1) {
                    break;
                }
            }
        }

        String s = new String(number);

        try {
            if (is_double) {
                Double d = Double.valueOf(s);

                return new TextToken(d.doubleValue());
            } else if (is_long) {
                return new TextToken(Long.parseLong(s));
            }

            long l = Long.parseLong(s);

            if (l > Integer.MAX_VALUE || l < Integer.MIN_VALUE) {
                return new TextToken(l);
            }

            return new TextToken((int) l);
        } catch (NumberFormatException ex) {
            throw new IOException("Can't read number " + s);
        }
    }

    private TextToken readIdentifier(int first_char) throws IOException {
        int c = first_char;
        StringBuffer buf = new StringBuffer();

        while (c != -1) {
            if (Character.isJavaIdentifierPart((char) c)) {
                buf.append((char) c);
            } else {
                unread();
                break;
            }

            c = read();
        }

        return new TextToken(new String(buf), TextToken.IDENTIFIER);
    }

    public static void main(String[] argv) throws IOException {
        String s = "Dies ist ein \"Test\" 12345 \"‰äöü\\u2030a\" -1234.5678 \"*\\\\\\\"*\"\nnextLine";

        System.out.println("s = " + s);
        System.out.println("withQuotes = " + TextToken.quoteString(s));


        TextToken[] tokens = TextTokenReader.getTokensFrom(s);

        for (int i = 0; i < tokens.length; i++) {
            TextToken token = tokens[i];

            System.out.println("token = " + token);

            System.out.println("token.getString() = " + token.getString());
        }

    }
}
