//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.resource;

import com.nexirius.util.locale.LocaleListener;
import com.nexirius.util.textresource.TextResource;

import javax.swing.*;
import java.awt.*;
import java.net.URL;
import java.text.DateFormat;

/**
 * ClientResource is meant to be the single source of external resources for Java Swing clients.
 * It provides both language dependent (menu text, tooltips) and language indepenent (font info.)
 * resources.  All resouces are currently refrenced by a resourceKey String. See the following
 * regarding naming conventions in your resource files.
 * <p/>
 * Please paste the following documentation into your resource (properties) file:
 * #===========================================================================================================
 * # Resources are accessed from Java Swing clients via the com.nexirius.util.resource.ClientResource interface.
 * # For the current information see the JavaDoc documentation for the above class.
 * #
 * # Resources consist of key-value pairs sepperated by =.
 * # A resource name can be associated with many different resource types (tool tips, icons, menu text).
 * # The following naming patterns are used to associate a resource key name with its resource type value.
 * # For example if the resourceName key is "paste" here is how you write your resource keys:
 * # paste.icon=someIconName.gif
 * # paste.accelerator=Ctrl+V
 * # paste.menu=$Paste
 * # paste.tip=This is the tool tip text
 * # paste.buttonText=Paste
 * #
 * # Note: There is no paste.mnemonic, mnemonic keys are indicated in the .menu text by the $ symbol.
 * #===========================================================================================================
 * <p/>
 */
public interface ClientResource extends LocaleListener, TextResource {
    public static final String DEFAULT = "default";
    public static final String SYSTEMCOLOR = "SystemColor_";

    /**
     * The background color for captions in window borders.
     */
    public static final String COLOR_ACTIVE_CAPTION = SYSTEMCOLOR + SystemColor.ACTIVE_CAPTION;
    /**
     * The border color for captions in window borders.
     */
    public static final String COLOR_ACTIVE_CAPTION_BORDER = SYSTEMCOLOR + SystemColor.ACTIVE_CAPTION_BORDER;
    /**
     * The text color for captions in window borders.
     */
    public static final String COLOR_ACTIVE_CAPTION_TEXT = SYSTEMCOLOR + SystemColor.ACTIVE_CAPTION_TEXT;
    /**
     * The background color for control objects.
     */
    public static final String COLOR_CONTROL = SYSTEMCOLOR + SystemColor.CONTROL;
    /**
     * The dark shadow color for control objects.
     */
    public static final String COLOR_CONTROL_DK_SHADOW = SYSTEMCOLOR + SystemColor.CONTROL_DK_SHADOW;
    /**
     * The regular highlight color for control objects.
     */
    public static final String COLOR_CONTROL_HIGHLIGHT = SYSTEMCOLOR + SystemColor.CONTROL_HIGHLIGHT;
    /**
     * The light highlight color for control objects.
     */
    public static final String COLOR_CONTROL_LT_HIGHLIGHT = SYSTEMCOLOR + SystemColor.CONTROL_LT_HIGHLIGHT;
    /**
     * The regular shadow color for control objects.
     */
    public static final String COLOR_CONTROL_SHADOW = SYSTEMCOLOR + SystemColor.CONTROL_SHADOW;
    /**
     * The text color for control objects.
     */
    public static final String COLOR_CONTROL_TEXT = SYSTEMCOLOR + SystemColor.CONTROL_TEXT;
    /**
     * The color of the desktop background.
     */
    public static final String COLOR_DESKTOP = SYSTEMCOLOR + SystemColor.DESKTOP;
    /**
     * The background color for inactive captions in window borders.
     */
    public static final String COLOR_INACTIVE_CAPTION = SYSTEMCOLOR + SystemColor.INACTIVE_CAPTION;
    /**
     * The border color for inactive captios in window borders.
     */
    public static final String COLOR_INACTIVE_CAPTION_BORDER = SYSTEMCOLOR + SystemColor.INACTIVE_CAPTION_BORDER;
    /**
     * The text color for inactive captions in window borders.
     */
    public static final String COLOR_INACTIVE_CAPTION_TEXT = SYSTEMCOLOR + SystemColor.INACTIVE_CAPTION_TEXT;
    /**
     * The background color for info(help) text.
     */
    public static final String COLOR_INFO = SYSTEMCOLOR + SystemColor.INFO;
    /**
     * The text color for info(help) text.
     */
    public static final String COLOR_INFO_TEXT = SYSTEMCOLOR + SystemColor.INFO_TEXT;
    /**
     * The background color for menus.
     */
    public static final String COLOR_MENU = SYSTEMCOLOR + SystemColor.MENU;
    /**
     * The text color for menus.
     */
    public static final String COLOR_MENU_TEXT = SYSTEMCOLOR + SystemColor.MENU_TEXT;
    /**
     * The background color for scrollbars.
     */
    public static final String COLOR_SCROLLBAR = SYSTEMCOLOR + SystemColor.SCROLLBAR;
    /**
     * The background color for text components.
     */
    public static final String COLOR_TEXT = SYSTEMCOLOR + SystemColor.TEXT;
    /**
     * The background color for highlighted text.
     */
    public static final String COLOR_TEXT_HIGHLIGHT = SYSTEMCOLOR + SystemColor.TEXT_HIGHLIGHT;
    /**
     * The text color for highlighted text.
     */
    public static final String COLOR_TEXT_HIGHLIGHT_TEXT = SYSTEMCOLOR + SystemColor.TEXT_HIGHLIGHT_TEXT;
    /**
     * The text color for inactive text.
     */
    public static final String COLOR_TEXT_INACTIVE_TEXT = SYSTEMCOLOR + SystemColor.TEXT_INACTIVE_TEXT;
    /**
     * The text color for text components.
     */
    public static final String COLOR_TEXT_TEXT = SYSTEMCOLOR + SystemColor.TEXT_TEXT;
    /**
     * The background color for windows.
     */
    public static final String COLOR_WINDOW = SYSTEMCOLOR + SystemColor.WINDOW;
    /**
     * The border color for windows.
     */
    public static final String COLOR_WINDOW_BORDER = SYSTEMCOLOR + SystemColor.WINDOW_BORDER;
    /**
     * The text color for windows.
     */
    public static final String COLOR_WINDOW_TEXT = SYSTEMCOLOR + SystemColor.WINDOW_TEXT;

    /**
     * Checks wheter the resourceKey is defined
     *
     * @pre hasResource(resourceKey)
     */
    public boolean isDefined(String resourceKey);

    /**
     * Gets the resource value as text and translates it into an int
     *
     * @pre isDefined(resourceKey)
     */
    public int getInt(String resourceKey);

    /**
     * Returns the text displayed for a button in the current language.
     *
     * @return java.lang.String
     */
    public String getButtonText(String resourceKey);

    /**
     * NOT USED!!!!
     *
     * @return java.net.URL
     * @depricated
     */
    public URL getHelpURL(String resourceKey);

    public String getHelpString(String resourceKey);

    /**
     * Appends .icon to the resourceKey and returns a Icon from the resource bundel.
     * Return null if resource not found.
     *
     * @return com.sun.java.swing.Icon;
     */
    public Icon getIcon(String resourceKey);

    /**
     * Same as getText() but returns the key itself instead of null when not defined
     */
    public String getLabel(String resourceKey);

    /**
     * Appends .accelerator to the resourceKey and returns a string representation of the accelerator.
     * The accelerator should be in the following format: Meta+Key
     * Examples: Ctrl+X, Alt+F3, Shift+Del, Meta+8, Del, PgDn, Esc, F1, Tab.
     * !! should support multiple mode keys: Ctrl+Shift+Key
     * !! TODO: Document all Key representations.
     * Note this is not the same as a mnemonic key, see getMnemonic().
     * Return null if resource not found.
     *
     * @return java.lang.String
     */
    public String getKeyAcceleratorText(String resourceKey);

    /**
     * Appends .menu to the resourceKey and returns a String from the resource bundel.
     * Mnemonic marked is stripped out.
     * Return null if resource not found.
     *
     * @return java.net.URL
     */
    public String getMenuText(String resourceKey);

    /**
     * A mnemonic is "hot" key you can type to activate a button or menu status.  It is
     * indicated by an underlined letter (in windows anyway).  It is not necisaraly the
     * same as a accelerator key, because an accelerator key can fire an event even when
     * its menu is not visible, for example.
     * getMnemonic calls getMenuText() and uses the character that directly preceeds $ as
     * the mnemonic key, returning its position.  If no $ marker is found returns -1 and no mnemonic is set.
     * <p/>
     * TODO!! probably shouldn't use $ for the mnemonic because then the bank can't put
     * dallor amounts in the menus!!
     * TODO!! Clarify accelerator key and mnemonic relationship and naming standards.
     * Is a bug in Swing: mnemonic can't be different from accelerator.
     *
     * @return int
     */
    public int getMnemonic(String resourceKey, String extension);

    /**
     * Appends .tip to the resourceKey and returns a tool tip string from the resource bundel.
     * Return null if resource not found.
     *
     * @return java.lang.String
     */
    public String getToolTipText(String resourceKey);

    /**
     * Appends .dateformat to the resourceKey and returns a DateFormat Instance from the resource bundel.
     * Return a default date format if resource not found.
     *
     * @return java.text.DateFormat
     */
    public DateFormat getDateFormat(String resourceKey);

    /**
     * Appends .color to the resourceKey and returns a Color instance from the resource bundel.
     * Return a default color if resource not found.
     *
     * @return java.awt.Color
     */
    public Color getColor(String resourceKey);

    /**
     * Appends .font to the resourceKey and returns a Font instance from the resource bundel.
     * Return a default font if resource not found.
     *
     * @return java.awt.Font
     */
    public Font getFont(String resourceKey);

    /**
     * Appends .fontFamily to the resourceKey and returns a font family name from the resource bundel.
     * Return a default string if resource not found.
     *
     * @return font family identifier
     */
    public String getFontFamily(String resourceKey);

    /**
     * Appends .fontSize to the resourceKey and returns an int from the resource bundel.
     * Return a default size if resource not found.
     *
     * @return a font size
     */
    public int getFontSize(String resourceKey);

    /**
     * Appends .currencyNumberFormat to the currencyCode and returns a format string (e.g. 'Fr.' #,##0.00;'Fr.' -#,##0.00)
     * Returns null if resource not found.
     *
     * @return a format description string
     */
    public String getCurrencyNumberFormat(String currencyCode);

    /**
     * Appends .currencyRoundingDenominator to the currencyCode and returns a double (e.g. 0.05)
     * Returns 0.0 if resource not found.
     *
     * @return the smallest denominator
     */
    public double getCurrencyRoundingDenominator(String currencyCode);

    /**
     * @param newListener com.nexirius.util.resource.ResourceChangeListener
     */
    public void addResourceChangeListener(com.nexirius.util.resource.ResourceChangeListener newListener);

    /**
     * @param newListener com.nexirius.util.resource.ResourceChangeListener
     */
    public void removeResourceChangeListener(com.nexirius.util.resource.ResourceChangeListener newListener);

    /**
     * @param clientResource com.nexirius.util.resource.ClientResource
     */
    void setParent(ClientResource clientResource);

    /**
     * This method was created in VisualAge.
     */
    void updateListeners();
}
