//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.resource;

import com.nexirius.framework.FWLog;
import com.nexirius.util.assertion.Assert;

import java.util.MissingResourceException;

/**
 * A class providing a single point of reference to objects which require
 * client resources
 * <p/>
 *
 * @author Marcel Baumann
 *         Date        Author           Changes/Enhancements
 *         1999.03.24  MB              Created
 */
public class ClientResourceToolkit {



    // Flag to indicate whether or not initialise has been called
    private static boolean initialiseCalled = false;

    // The component relative to which the popup is displayed
    private static ClientResourceToolkit instance = null;

    // Arbitrary object used for waiting/notifying
    private static Object lock = new Object();

    // The resource used to obtain client information
    private ClientResource resource;

    /**
     * Initialise the ClientResourceToolkit with a DefaultClientResource
     */
    public static void initialise() {
        initialise(new DefaultClientResource());
    }


    /**
     * Initialise the ClientResourceToolkit
     *
     * @param resourceName The name of the client resource
     */
    public static void initialise(String resourceName) {
        ClientResourceToolkit.initialiseCalled = true;
        final String _resourceName = resourceName;

        Runnable runner = new Runnable() {
            public void run() {
                try {
                    setInstance(new ClientResourceToolkit(_resourceName));
                } catch (MissingResourceException e) {
                    FWLog.error(e);
                    ClientResource resource = new DefaultClientResource();
                    setInstance(new ClientResourceToolkit(resource));
                }
            }
        };

        Thread worker = new Thread(runner);
        worker.start();
    }


    /**
     * Initialise the ClientResourceToolkit
     *
     * @param resource The client resource to be provided as a singleton instance
     */
    public static void initialise(ClientResource resource) {
        final ClientResource _resource = resource;

        Runnable runner = new Runnable() {
            public void run() {
                setInstance(new ClientResourceToolkit(_resource));
            }
        };

        Thread worker = new Thread(runner);
        worker.start();
    }


    public static ClientResourceToolkit instance() {
        Assert.pre(initialiseCalled, "initialise() has been called");
        awaitInstance(); // 2nd precondition is that the instance is available
        return ClientResourceToolkit.instance;
    }


    /**
     * Wait for the instance to be availavle
     */
    public static void awaitInstance() {
        synchronized (ClientResourceToolkit.lock) {
            while (ClientResourceToolkit.instance == null) {
                try {
                    ClientResourceToolkit.lock.wait();
                } catch (InterruptedException ie) {   // Do nothing
                }
            }
        }
    }

    /**
     * Set what will become the singleton instance and notify any waiting
     * threads
     *
     * @param instance The object which will become the singleton instance
     */
    private static void setInstance(ClientResourceToolkit instance) {
        Assert.pre(instance != null, "Parameter instance is not null");
        synchronized (ClientResourceToolkit.lock) {
            ClientResourceToolkit.instance = instance;
            ClientResourceToolkit.initialiseCalled = true;

            // Notify waiting threads of new instance
            ClientResourceToolkit.lock.notifyAll();
        }
    }


    /**
     * Construct a ClientResourceToolkit
     */
    protected ClientResourceToolkit(ClientResource resource) {
        super();
        Assert.pre(resource != null, "Parameter resource is not null");

        this.resource = resource;
    }


    /**
     * Construct a ClientResourceToolkit
     */
    protected ClientResourceToolkit(String resourceName)
            throws MissingResourceException {
        super();
        Assert.pre(resourceName != null, "Parameter resourceName is not null");

        this.resource = new ClientResourceImpl(resourceName);
    }


    /**
     * Return the client resource
     *
     * @return the client resource
     */
    public ClientResource getClientResource() {
        return this.resource;
    }
}
