//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.statemachine;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * A StateMachine implementation
 *
 * @author Marcel Baumann
 */
public class StateMachine implements Serializable {



    State beginning;
    State act = null;
    Vector listenerList = new Vector();

    /**
     * The state machine only holds the beginning and the actual state. The connection to
     * the other states is held in the states which are attached to the beginning state
     * through Transitions. As soon as you define a transition it registers itself to the
     * first state. So all you have to do is to definie a set of State instances and
     * a set of Transtion instances. Then define a state machine instance with this constructor.
     *
     * @param beginning
     */
    public StateMachine(State beginning) {
        this.beginning = beginning;
        act = beginning;
    }

    /**
     * Attach a listener to this state machine
     *
     * @param listener
     */
    public void addStateMachineListener(StateMachineListener listener) {
        if (listener != null) {
            listenerList.addElement(listener);
        }
    }

    /**
     * remove a state machine listener
     *
     * @param listener
     */
    public void removeStateMachineListener(StateMachineListener listener) {
        if (listener != null) {
            listenerList.removeElement(listener);
        }
    }

    /**
     * Fire the before transition event on all attached listeners.
     * If one of the listeners returns false then the other listeners are skipped and
     * the transition is not performed and this method itself returns false
     */
    public synchronized boolean fireBeforeTransition(State act, String event, State next) {
        Enumeration e = listenerList.elements();

        while (e.hasMoreElements()) {
            StateMachineListener l = (StateMachineListener) e.nextElement();

            boolean ret = l.stateMachineBeforeTransition(this, act, event, next);

            if (!ret) {

                return false;
            }
        }

        return true;
    }

    /**
     * Fire the after transition event on all attached listeners.
     *
     * @param prev
     * @param event
     * @param act
     */
    public synchronized void fireAfterTransition(State prev, String event, State act) {
        Enumeration e = listenerList.elements();

        while (e.hasMoreElements()) {
            StateMachineListener l = (StateMachineListener) e.nextElement();

            l.stateMachineAfterTransition(this, prev, event, act);
        }
    }

    /**
     * Fire the reset event on all attached listeners.
     */
    public synchronized void fireReset() {
        Enumeration e = listenerList.elements();

        while (e.hasMoreElements()) {
            StateMachineListener l = (StateMachineListener) e.nextElement();

            l.stateMachineReset(this);
        }
    }

    /**
     * Set the state machine back to the beginning and fire a reset event to the attached listeners.
     * Reset is not fired when the state machine is already at the beginning state.
     *
     * @return The beginning state
     */
    public State reset() {
        if (act != beginning) {
            act = beginning;

            fireReset();
        }

        return act;
    }

    /**
     * Access the actual state.
     *
     * @return
     */
    public State getActState() {
        return act;
    }


    /**
     * Access the next state using the event source which is registered with the actual state.
     * throws an Exception if one of the listeners returns false on stateMachineBeforeTransition method
     */
    public State getNextState()
            throws Exception {
        if (act == null) {

            return null;
        }

        return getNextState(act.getEventSource());
    }

    /**
     * Access the next state using the event source which is given as parameter.
     *
     * @param es The event soure which is used to produce the next event.
     * @return
     * @throws Exception
     */
    public State getNextState(EventSource es)
            throws Exception {
        if (act == null) {

            return null;
        }

        String event = es.getNextEvent();

        try {
            State nextState = step(event);

            return nextState;
        } catch (Exception ex) {
            es.pushBackEvent(event);

            throw ex;
        }
    }

    /**
     * Step to the next state (if not vetoed by one of the listeners) and
     * fire the appropriate events to the attached listeners.
     *
     * @param event The event which should be used for the next step.
     * @return The next State (act state)
     * @throws Exception if step has been vetoed by one of the listeners or if the event is not supported by this state
     */
    public State step(String event)
            throws Exception {
        if (act == null) {

            return act;
        }

        State next = act.getNextStateForEvent(event);

        if (next == null) {

            throw new Exception("Event " + event + " not supported in state " + act.getName());
        }

        if (!fireBeforeTransition(act, event, next)) {

            throw new Exception("Must not step");
        }

        State prev = act;

        act = act.step(event);

        fireAfterTransition(prev, event, act);

        return act;
    }

    /**
     * Calls reset and getNextState until it returns null or it reaches the end state
     *
     * @throws Exception
     */
    public void run()
            throws Exception {
        reset();

        while (act != null && !act.isEnd()) {
            act = getNextState();
        }
    }

    /**
     * Calls reset and getNextState until it returns null or it reaches the end state
     *
     * @param es Use the given event source (not the event source which maybe attached to the states)
     */
    public void run(EventSource es)
            throws Exception {
        reset();

        while (act != null && !act.isEnd()) {
            act = getNextState(es);
        }
    }

    /**
     * Directly jump to a certain state (use with care)
     *
     * @param next The state in the state machine where it sould jump to
     * @throws Exception If the step is vetoed by one of the listeners
     */
    public void setActState(State next)
            throws Exception {
        if (!fireBeforeTransition(act, null, next)) {

            throw new Exception("Must not step");
        }

        State prev = act;

        act = next;

        fireAfterTransition(prev, null, act);
    }
}

