//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.textresource;

import com.nexirius.util.locale.LocaleEvent;
import com.nexirius.framework.FWLog;

import java.util.Hashtable;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * This class essentially wraps a ResourceBundle containing
 * locale-specific text information. The information is looked up
 * by visual components at runtime.
 * This class is also responsible for notifying TextResourceListeners
 * of TextResourceEvents.
 *
 * @author Marcel Baumann
 */
public class TextResourceImpl implements TextResource {
    protected boolean debugging = false;
    protected transient TextResourceListener aTextResourceListener = null;
    protected Hashtable<Locale, ResourceBundle> resourceBundles = new Hashtable<Locale, ResourceBundle>(1);
    private Locale currentLocale;
    private String resourceName;
    private TextResource parentTextResource = null;

    /**
     * TextSourceManager constructor comment.
     */
    public TextResourceImpl(String resourceName)
            throws MissingResourceException {
        super();

        // Name of resource (used to find resource files)
        this.resourceName = resourceName;

        // Base locale is the current global setting
        this.currentLocale = Locale.getDefault();

        // Obtain a default resource bundle based in the framework's current locale
        ResourceBundle defaultResource;
        try {
        	defaultResource = ResourceBundle.getBundle(resourceName, currentLocale);

        	// Put in table of resource bundles
            resourceBundles.put(currentLocale, defaultResource);
        }
        catch (MissingResourceException mrExp) {
        	// Ignore!
        }
        
        debugging = (System.getProperty("debugResources") != null);
    }

    /**
     * @param newListener com.nexirius.framework.TextResourceListener
     */
    public void addTextResourceListener(TextResourceListener newListener) {
        // FIX (memory blocking) aTextResourceListener = TextResourceEventMulticaster.add(aTextResourceListener, newListener);
        return;
    }

    /**
     * Method to support listener events.
     *
     * @param event com.nexirius.framework.TextSourceEvent
     */
    protected void fireTextResourceChanged(TextResourceEvent event) {
        if (aTextResourceListener == null) {
            return;
        }

        aTextResourceListener.textResourceChanged(event);
    }

    public TextResource getParent() {
        return this.parentTextResource;
    }

    public String getText(String key) {
        String ret = getText(key, currentLocale);

        return ret;
    }

    public String getText(String key, Locale locale) {
        if (key == null) {
            throw new IllegalArgumentException("Parameter key cannot be null");
        }
        if (locale == null) {
            throw new IllegalArgumentException("Parameter locale cannot be null");
        }

        // Is the resource bundle for this locale already loaded?
        if (!resourceBundles.containsKey(locale)) {

            // Attempt to load resource bundle for this locale
            try {
                ResourceBundle resourceBundle = ResourceBundle.getBundle(resourceName, locale);

                // Add it to the collection of resource bundles
                resourceBundles.put(locale, resourceBundle);
            } catch (MissingResourceException e) {
                // This should never happen because ResourceBundle.getBundle should
                // return the default resource if the specific locale-specific resource
                // is not found. If this didn't happen, can only assume an error in
                // the Java API
                return key;
            }
        }

        /*
            // Find out if resource file exists for this resourcename and locale:
            String searchName = resourceName + locale.toString() + ".properties";
            java.io.InputStream dummy = getClass().getClassLoader().getResourceAsStream(searchName);
            if (dummy == null)
            { // DO SOMETHING USEFUL WITH THIS INFORMATION
              // BECAUSE THE CODE BELOW DOES NOT WORK!!!!!
              // presumably check parent before allowing to default
              // to default locale's lookup string
            }
        */

        // We can now assume a resource bundle has been loaded for this locale
        ResourceBundle resourceBundle = resourceBundles.get(locale);

        // Try and get the string associated with this key
/*
		final Enumeration e = resourceBundle.getKeys();
		while (e.hasMoreElements())
		{
			String aKey = (String)e.nextElement();
			if (aKey.equals(key)) {
				return (String)resourceBundle.getString(key);
			}
		}
*/
        try {
            return resourceBundle.getString(key);
        } catch (MissingResourceException ex) {

        }

        // Didn't work, so try the parent
        return getParent() == null ? null : getParent().getText(key, locale);
        /*
        String text = resourceBundle.getString(key);
        if (text == null) {
            text = getParent().getText(key, locale);
        }
        return text;
        else
        {
            return key;
        }
        */
    }

    public void localeChanged(LocaleEvent event) {
        this.currentLocale = event.getLocale();
        FWLog.getLogger().debug("Locale changed: " + this.currentLocale);
        updateListeners();
    }

    public void removeTextResourceListener(TextResourceListener newListener) {
        aTextResourceListener = TextResourceEventMulticaster.remove(aTextResourceListener, newListener);
        return;
    }

    public void setParent(TextResource textResource) {
        if (textResource == this) {
            throw new IllegalArgumentException("TextResource cannot be a parent of itself");
        }
        this.parentTextResource = textResource;
    }

    public void updateListeners() {
        fireTextResourceChanged(new TextResourceEvent(this, this, TextResourceEvent.LOCALE_CHANGED));
    }
}
